c  coordinate transformation subroutines
c
c  definition of coordinates
c
c  C(celestial)-coordinates (x,y,z)
c    vector components are generally given in this coordinate system
c    x-axis in celestial EW direction
c    y-axis in celestial NS direction
c    z-axis toward the observer
c    unit vectors are (1,0,0),(0,1,0),(0,0,1)
c    the origin (x=0,y=0,z=0) is normally at the solar disk center
c    center of the sun is at xxsp, which is normally at (0,0,-rsp)
c    the disk center is always at xxsp + rsp*(0,0,1)
c
c  D(data)-coordinates (xd,yd,zd)
c    xd-axis along observational x-direction 
c    yd-axis along observational y-direction
c    zd-axis toward the observer
c    the origin (xd=0,yd=0,zd=0) is at the solar disk center
c    unit vectors are xdu,ydu,zdu
c    In the C-coordinates, the yd-axis is at the position angle ypos
c    ypos > 0 means the yd-axis is tilted counterclockwise from the y-axis
c
c  T(tangential)-coordinates (xt,yt,zt)
c    zt=0 plane is tangent to the solar surface at xxtod=(xtod,ytod,ztod)
c    in data coordinates (which is xxto in Cartesian coordinates)
c    xt-axis along phi direction
c    (i.e. along xd-direction when projected onto the sky plane)
c    yt-axis along theta direction (phi and theta are the S-coordinates)
c    zt-axis along r-direction
c    the origin at xxto (normally the center of observed area)
c    unit vectors are xtu,ytu,ztu
c    ypu is the projection of ydu onto the tangent plane
c
c  S(spherical)-coordinates (ap,al,h)
c    origin is at xxsp (center of the sun)
c    ap=longitude, al=co-latitude, h=r-rsp (height above the surface)
c    (ap=0,al=0) is the disk center
c    polar axis is along the yd-axis, at the position angle ypos in the
c    plane of the sky
c
c  R(rotation axis)-coordinates (apr,alr,h)
c  (this is the heliographic coordinate system)
c    origin is at xxsp (center of the sun)
c    apr=longitude, alr=co-latitude, h=r-rsp (height above the surface)
c    the central meridian is at apr=cl0eff
c    if cl0eff=0, this is the heliographic coordinate system
c    if cl0eff=cl0 (the Carrington longitude of the central meridian),
c    then this is the Carrington coordinate system
c    polar axis is given by the usual (p,b0) angle
c
c----------------------------------------------------------------------
c
c  projection of cartesian position xx onto the solar surface
c  depending on the value of 'mesh'
c  xx is modified
c
	subroutine proj(xx, mesh1)
	dimension xx(3)
c
	if(mesh1.eq.0) then
	  xx(3)=0.0
	else if(iabs(mesh1).eq.1) then
	  call projt(xx,xt,yt)
	else if(mesh1.eq.2) then
	  call projs(xx,ap,al)
	else if(mesh1.ge.3) then
	  call projr(xx,ap,al)
	endif
	return
	end
c
c  projection of cartesian position xx onto the spherical surface
c  of radius rsp1
c  z=xx(3) is calculated from x=xx(1) and y=xx(2)
c
	subroutine projsr(xx,rsp1)
	dimension xx(3)
	include 'coord.fc'
c
	xx(3)=xxsp(3)
	r2=(xx(1)-xxsp(1))**2+(xx(2)-xxsp(2))**2
	if(r2.lt.rsp1**2) xx(3)=sqrt(abs(rsp1**2-r2))+xxsp(3)
	return
	end
c
c  projection of cartesian position xx onto the solar surface
c  return value: S-coordinates (ap=longitude, al=co-latitude)
c                xx(3) is modified in this routine
c  ypos is given in common coord
c  setdu must be called prior to this routine
c
	subroutine projs(xx,ap,al)
c
	include 'sys.fc'
	include 'coord.fc'
	dimension xx(3),xx1(3)
	logical extend
	data halfpi/1.5707963/
c
	extend=.false.
	go to 1
c
	entry projse(xx,ap,al)
	extend=.true.
c
 1	continue
c
	if(.not.dset) then
	  write(msg,*) '(subr.projs ) error: setdu has not been called'
	  errflg=2
	  return
	endif
c
	xx(3)=xxsp(3)
	r2=(xx(1)-xxsp(1))**2+(xx(2)-xxsp(2))**2
	if(r2.lt.rsp**2) then
	  xx(3)=sqrt(abs(rsp**2-r2))+xxsp(3)
	  call ctos(xx,ap,al,h)
c  h should be nearly 0
c
	else if(abs(xx(2)-xxsp(2)).lt.rsp) then
c  the point is outside of the sphere but there exists y-intercept
	  if(extend) then
	    xx1(1)=sign(sqrt(abs(rsp**2-xx(2)**2)), xx(1))
	    xx1(2)=xx(2)
	    xx1(3)=xxsp(3)
	    call ctos(xx1,ap,al,h)
	  else
	    ap=ufl
	    al=ufl
	  endif
	else
c
c  abs(y-coord) is larger than rsp
	  if(extend) then
	    ap=sign(halfpi,xx(1))
	    al=sign(halfpi,xx(2)-xxsp(2))
	  else
	    ap=ufl
	    al=ufl
	  endif
	  h=0.0
	endif
c
	return
	end
c
c  projection of cartesian position xx onto the source surface
c  return value: S-coordinates (ap=longitude, al=co-latitude)
c                xx(3) is modified in this routine
c  ypos is given in common coord
c  setdu must be called prior to this routine
c
      subroutine projs2(xx,ap,al)
c
      include 'sys.fc'
      include 'coord.fc'
      include 'lparam.fc'
      dimension xx(3)
c
      if(.not.dset) then
        write(msg,*) '(subr.projs2) error: setdu has not been called'
        errflg=2
        return
      endif
c
      if(rrss.eq.0.0) then
        write(msg,*)
     &  '(subr.projs2) error: source surface radius is not set'
        errflg=2
        return
      endif
c
	call projsr(xx,rss)
	call ctos(xx,ap,al,h)
	return
c  h should be nearly rss-rsp
      end
c
c  projection of cartesian position xx onto the solar surface
c  return value: R-coordinates (ap=longitude, al=co-latitude)
c                xx(3) is modified in this routine
c  p and b0 are given in common coord
c  setdu and setar must be called prior to this routine
c
	subroutine projr(xx,apr,alr)
c
	include 'sys.fc'
	include 'coord.fc'
	dimension xx(3),xx1(3)
	logical extend
	data pi/3.141592/, eps/0.01/
c
	extend=.false.
	go to 1
c
	entry projre(xx,apr,alr)
	extend=.true.
 1	continue
c
	if(.not.dset) then
	  write(msg,*) '(subr.projr ) error: setdu has not been called'
	  errflg=2
	  return
	endif
c
	if(.not.rset) then
	  write(msg,*) '(subr.projr ) error: setar has not been called'
	  errflg=2
	  return
	endif
c
	xx(3)=xxsp(3)
	r2=(xx(1)-xxsp(1))**2+(xx(2)-xxsp(2))**2
	if(r2.lt.rsp**2) then
	  xx(3)=sqrt(abs(rsp**2-r2))+xxsp(3)
	  call ctor(xx,apr,alr,h)
c  h should be nearly 0
c
	else if(abs(xx(2)-xxsp(2)).lt.rsp) then
c  the point is outside of the sphere but there exists y-intercept
	  if(extend) then 
	    xx1(1)=sign(sqrt(abs(rsp**2-xx(2)**2)), xx(1))
	    xx1(2)=xx(2)
	    xx1(3)=xxsp(3)
	    call ctor(xx1,apr,alr,h)
	  else
	    apr=ufl
	    alr=ufl
	  endif
c
c  abs(y-coord) is larger than rsp
c
	else if(abs(b0rad).lt.eps) then
c  both poles are nearly exactly at the border
	  if(extend) then
	    if(xx(2)-xxsp(2).gt.0.0) then
	      apr=cl0eff+sign(0.5*pi-eps,xx(1))
	      alr= 0.5*pi
	    else
	      apr=cl0eff+sign(0.5*pi-eps,xx(1))
	      alr=-0.5*pi+b0rad
	    endif
	  else
	    apr=ufl
	    alr=ufl
	  endif
c
	else if(b0rad.gt.0.0) then
c  north pole is visible
	  if(extend) then
	    if(xx(2)-xxsp(2).gt.0.0) then
	      apr=cl0eff+sign(pi-eps,xx(1))
	      alr= 0.5*pi
	    else
	      apr=cl0eff+sign(0.5*pi-eps,xx(1))
	      alr=-0.5*pi+b0rad
	    endif
	  else
	    apr=ufl
	    alr=ufl
	  endif
	else
c  south pole is visible
	  if(extend) then
	    if(xx(2)-xxsp(2).lt.0.0) then
	      apr=cl0eff+sign(pi-eps,xx(1))
	      alr=-0.5*pi
	    else
	      apr=cl0eff+sign(0.5*pi-eps,xx(1))
	      alr= 0.5*pi+b0rad
	    endif
	  else
	    apr=ufl
	    alr=ufl
	  endif
	endif
c
	return
	end
c
c  projection of cartesian position xx onto the source surface
c  return value: R-coordinates (ap=longitude, al=co-latitude)
c                xx(3) is modified in this routine
c  p and b0 are given in common coord
c  setdu and setar must be called prior to this routine
c
      subroutine projr2(xx,apr,alr)
c
      include 'sys.fc'
      include 'coord.fc'
      include 'lparam.fc'
      dimension xx(3)
c
      if(.not.dset) then
        write(msg,*) '(subr.projr2) error: setdu has not been called'
        errflg=2
        return
      endif
c
      if(.not.rset) then
        write(msg,*) '(subr.projr2) error: setar has not been called'
        errflg=2
        return
      endif
c
      if(rrss.eq.0.0) then
        write(msg,*)
     &  '(subr.projr2) error: source surface radius is not set'
        errflg=2
        return
      endif
c
	call projsr(xx,rss)
	call ctor(xx,apr,alr,h)
c  h should be nearly rss-rsp
      return
      end
c
c  projection of cartesian position xx onto T-coordinates
c  (local cartesian coordinate defined by the tangent plane)
c  return value: (xt,yt)=local cartesian coordinates
c                 xx(3) is modified in this routine
c  xxto(1-3) = local origin
c  setdu and settu must be called prior to this routine
c
 	subroutine projt(xx,xt,yt)
c
	include 'sys.fc'
	include 'coord.fc'
	dimension xx(3)
c
	if(.not.dset) then
	  write(msg,*) '(subr.projt ) error: setdu has not been called'
	  errflg=2
	  return
	endif
c
	if(.not.tset) then
	  write(msg,*) '(subr.projt ) error: settu has not been called'
	  errflg=2
	  return
	endif
c
	call ctod2(xx(1),xx(2),xd,yd)
	deltax=xd-xxtod(1)
	deltay=yd-xxtod(2)
	den=cosals*cosaps
	xt=(cosals*deltax + sinals*sinaps*deltay)/den
	yt= cosaps*deltay/den
	zd=-(sinaps*cosals*deltax + sinals*deltay)/den
	xx(3)=zd +xxto(3)
	return
	end
c
c  transform from S-coordinates to C-coordinates
c  ap=longitude, al=co-latitude
c  ypos is given in common coord
c
c  setdu must be called prior to this routine
c
	subroutine stoc(ap,al,h,xx)
	include 'coord.fc'
	dimension xx(3),xxd(3)
c
	call stod(ap,al,h,xxd)
	call dtoc(xxd,xx)
	return
c
c  transform from C-coordinates to S-coordinates
c
	entry ctos(xx,ap,al,h)
c
	call ctod(xx,xxd)
	call dtos(xxd,ap,al,h)
	return
	end
c
c  transform from R-coordinates to C-coordinates
c  apr=longitude, alr=co-latitude
c  p and b0 are given in common coord
c
c  setdu and setar must be called prior to this routine
c
	subroutine rtoc(apr,alr,h,xx)
	include 'sys.fc'
	include 'coord.fc'
	dimension xx(3),xxr(3)
	data halfpi/1.5707963/
c
	if(.not.dset) then
	  write(msg,*) '(subr.rtoc  ) error: setdu has not been called'
	  errflg=2
	  return
	endif
c
	if(.not.rset) then
	  write(msg,*) '(subr.rtoc  ) error: setar has not been called'
	  errflg=2
	  return
	endif
c
	call convsx( rsp+h, halfpi-alr, apr-cl0eff, convx,convy,convz )
	xxr(1)=convy
	xxr(2)=convz
	xxr(3)=convx
	xx1=armat(1,1)*xxr(1)+armat(1,2)*xxr(2)+armat(1,3)*xxr(3)
	xx2=armat(2,1)*xxr(1)+armat(2,2)*xxr(2)+armat(2,3)*xxr(3)
	xx3=armat(3,1)*xxr(1)+armat(3,2)*xxr(2)+armat(3,3)*xxr(3)
	xx(1)=xx1+xxsp(1)
	xx(2)=xx2+xxsp(2)
	xx(3)=xx3+xxsp(3)
	return
c
c  transform from C-coordinates to R-coordinates
c
	entry ctor(xx,apr,alr,h)
c
	xx1=xx(1)-xxsp(1)
	xx2=xx(2)-xxsp(2)
	xx3=xx(3)-xxsp(3)
	xxr(1)=xx1*armat(1,1)+xx2*armat(2,1)+xx3*armat(3,1)
	xxr(2)=xx1*armat(1,2)+xx2*armat(2,2)+xx3*armat(3,2)
	xxr(3)=xx1*armat(1,3)+xx2*armat(2,3)+xx3*armat(3,3)
	convx=xxr(3)
	convy=xxr(1)
	convz=xxr(2)
	call convxs( convx,convy,convz, r,theta,phi )
	apr=phi+cl0eff
c  Carrington longitude is from 0 to 2*pi
	if(cl0eff.ne.0.0) then
	  if(apr.lt.0.0) apr=apr+4.0*halfpi
	endif
	alr=halfpi-theta
	h=r-rsp
	return
	end
c
c  transform from T-coordinates to D-coordinates
c  xxt=(xt,yt,zt)=local cartesian coordinate
c  xxto = local origin
c
c  setdu and settu must be called prior to this routine
c
	subroutine ttod(xxt,xxd)
	include 'sys.fc'
	include 'coord.fc'
	dimension xxt(3),xxd(3),xx(3)
c
	if(.not.dset) then
	  write(msg,*) '(subr.ttod  ) error: setdu has not been called'
	  errflg=2
	  return
	endif
c
	if(.not.tset) then
	  write(msg,*) '(subr.ttod  ) error: settu has not been called'
	  errflg=2
	  return
	endif
c
	xx(1)=xxt(1)
	xx(2)=xxt(2)
	xx(3)=xxt(3)
c
	xxd(1)=atmat(1,1)*xx(1)+atmat(1,2)*xx(2)+atmat(1,3)*xx(3)
     &	      +xxtod(1)
	xxd(2)=atmat(2,1)*xx(1)+atmat(2,2)*xx(2)+atmat(2,3)*xx(3)
     &	      +xxtod(2)
	xxd(3)=atmat(3,1)*xx(1)+atmat(3,2)*xx(2)+atmat(3,3)*xx(3)
     &	      +xxtod(3)
c  buffer area xx is used so that this routine can be invoked as
c  call ttod(xxx,xxx)
	return
c
c  transform from D-coordinates to T-coordinates
c  xxt=(xt,yt,zt)=local cartesian coordinate
c  xxtod = local origin in data coordinates
c
	entry dtot(xxd,xxt)
c
	deltax=xxd(1)-xxtod(1)
	deltay=xxd(2)-xxtod(2)
	deltaz=xxd(3)-xxtod(3)
	xxt(1)=deltax*atmat(1,1)+deltay*atmat(2,1)+deltaz*atmat(3,1)
	xxt(2)=deltax*atmat(1,2)+deltay*atmat(2,2)+deltaz*atmat(3,2)
	xxt(3)=deltax*atmat(1,3)+deltay*atmat(2,3)+deltaz*atmat(3,3)
	return
	end
c
c  transform from T-coordinates to C-coordinates
c  xxt=(xt,yt,zt)=local cartesian coordinate
c  xxtod = local origin in data coordinates
c
c  setdu and settu must be called prior to this routine
c
	subroutine ttoc(xxt,xx)
	include 'coord.fc'
	dimension xx(3),xxd(3),xxt(3)
c
	call ttod(xxt,xxd)
	call dtoc(xxd,xx)
	return
c
c  transform from C-coordinates to T-coordinates
c  xxt=(xt,yt,zt)=local cartesian coordinate
c
	entry ctot(xx,xxt)
c
	call ctod(xx,xxd)
	call dtot(xxd,xxt)
	return
	end
c
c  2-d transform from D-coordinates to C-coordinates
c
c  setdu must be called prior to this routine
c
	subroutine dtoc2(xd,yd,x,y)
	include 'sys.fc'
	include 'coord.fc'
c
	if(.not.dset) then
	  write(msg,*) '(subr.dtoc2 ) error: setdu has not been called'
	  errflg=2
	  return
	endif
c
	x1=xdu(1)*xd +ydu(1)*yd +xxsp(1)
	y =xdu(2)*xd +ydu(2)*yd +xxsp(2)
	x=x1
c  buffer x1 is used to make 'call dtoc2(x,y,x,y)' possible 
	return
c
c  2-d transform from C-coordinates to D-coordinates
c
	entry ctod2(x,y,xd,yd)
c
	xx1=x-xxsp(1)
	xx2=y-xxsp(2)
	xd=xx1*xdu(1)+xx2*xdu(2)
	yd=xx1*ydu(1)+xx2*ydu(2)
	return
	end
c
c  transform from D-coordinates to C-coordinates
c
c  setdu must be called prior to this routine
c
	subroutine dtoc(xxd,xx)
	include 'coord.fc'
	dimension xxd(3),xx(3)
c
	call dtoc2(xxd(1),xxd(2),xx(1),xx(2))
	xx(3)=xxd(3)+xxsp(3)+rsp
	return
c
c  transform from C-coordinates to D-coordinates
c
	entry ctod(xx,xxd)
c
	call ctod2(xx(1),xx(2),xxd(1),xxd(2))
	xxd(3)=xx(3)-xxsp(3)-rsp
	return
	end
c
c  transform from S-coordinates to D-coordinates
c
	subroutine stod(ap,al,h,xxd)
	include 'coord.fc'
	dimension xxd(3)
	data halfpi/1.5707963/
c
	call convsx( rsp+h, halfpi-al, ap, convx,convy,convz )
	xxd(1)=convy
	xxd(2)=convz
	xxd(3)=convx-rsp
	return
c
c  transform from D-coordinates to S-coordinates
c
	entry dtos(xxd,ap,al,h)
c
	convx=xxd(3)+rsp
	convy=xxd(1)
	convz=xxd(2)
	call convxs( convx,convy,convz, r,theta,phi )
	ap=phi
	al=halfpi-theta
	h=r-rsp
	return
	end
c
c  transform from R-coordinates to D-coordinates
c  setdu and setar must be called prior to this routine
c
	subroutine rtod(apr,alr,h,xxd)
	include 'coord.fc'
	dimension xxd(3),xx(3)
c
	call rtoc(apr,alr,h,xx)
	call ctod(xx,xxd)
	return
c
c  transform from D-coordinates to R-coordinates
c
	entry dtor(xxd,apr,alr,h)
c
	call dtoc(xxd,xx)
	call ctor(xx,apr,alr,h)
	return
	end
c
c  conversion between (x,y,z) and (r,theta,phi)
c  conventional definition is adopted here, namely
c  the z-axis is the polar axis (theta=0 or pi)
c  the phi-angle is measured from the x-axis toward the y-axis
c  in the xz-plane, phi=0 in x>0 and phi=pi in x<0
c
 	subroutine convxs( convx,convy,convz, r,theta,phi )
c
	r1    = sqrt(convx**2+convy**2+convz**2)
	theta1= acos(convz/r1)
	phi1  = atan2(convy,convx)
	r=r1
	theta=theta1
	phi=phi1
	return
c  (r,theta,phi) are buffered so that 'call convsx(x,y,z,x,y,z)' is possible
c
	entry convsx( r,theta,phi, convx,convy,convz )
c
	r1=r
	theta1=theta
	phi1=phi
	convx=r1*sin(theta1)*cos(phi1)
	convy=r1*sin(theta1)*sin(phi1)
	convz=r1*cos(theta1)
	return
	end
c
c  calculate xdu and ydu vectors from ypos
c
	subroutine setdu
	include 'coord.fc'
c
	xdu(1)= cos(ypos)
	xdu(2)= sin(ypos)
	xdu(3)= 0.0
	ydu(1)=-sin(ypos)
	ydu(2)= cos(ypos)
	ydu(3)= 0.0
	zdu(1)=0.0
	zdu(2)=0.0
	zdu(3)=1.0
c
	dset=.true.
	call setas
	return
	end
c
c  calculate xtu, ytu, ztu, and ypu vectors from xxto
c  set atmat
c  xxd-xxtod = atmat * xxt, xxt = (xxd-xxtod) * transpose of atmat
c  setdu must be called prior to this routine
c
	subroutine settu
c
	include 'sys.fc'
	include 'coord.fc'
c
	if(.not.dset) then
	  write(msg,*) '(subr.settu ) error: setdu has not been called'
	  errflg=2
	  return
	endif
c
	call ctod(xxto,xxtod)
	call projs(xxto,aps,als)
	sinals=sin(als)
	cosals=cos(als)
	sinaps=sin(aps)
	cosaps=cos(aps)
	xtu(1)= cosaps*xdu(1) - sinaps*zdu(1)
	xtu(2)= cosaps*xdu(2) - sinaps*zdu(2)
	xtu(3)= cosaps*xdu(3) - sinaps*zdu(3)
	ytu(1)=-sinals*sinaps*xdu(1)+cosals*ydu(1)-sinals*cosals*zdu(1)
	ytu(2)=-sinals*sinaps*xdu(2)+cosals*ydu(2)-sinals*cosals*zdu(2)
	ytu(3)=-sinals*sinaps*xdu(3)+cosals*ydu(3)-sinals*cosals*zdu(3)
	ztu(1)= cosals*sinaps*xdu(1)+sinals*ydu(1)+cosals*cosaps*zdu(1)
	ztu(2)= cosals*sinaps*xdu(2)+sinals*ydu(2)+cosals*cosaps*zdu(2)
	ztu(3)= cosals*sinaps*xdu(3)+sinals*ydu(3)+cosals*cosaps*zdu(3)
	ypnorm=sqrt(sinals**2 + (cosals*cosaps)**2)
	ypu(1)=(cosals*cosaps*ydu(1) - sinals*zdu(1))/ypnorm
	ypu(2)=(cosals*cosaps*ydu(2) - sinals*zdu(2))/ypnorm
	ypu(3)=(cosals*cosaps*ydu(3) - sinals*zdu(3))/ypnorm
c
	atmat(1,1)= cosaps
	atmat(1,2)=-sinaps*sinals
	atmat(1,3)= sinaps*cosals
	atmat(2,1)= 0.0
	atmat(2,2)= cosals
	atmat(2,3)= sinals
	atmat(3,1)=-sinaps
	atmat(3,2)=-cosaps*sinals
	atmat(3,3)= cosaps*cosals
c
	tset=.true.
	return
	end
c
c  calculate armat from p and b0
c  xx = armat * xxr, xxr = xx * transpose of armat
c
	subroutine setar
	include 'coord.fc'
c
	armat(1,1)= cos(prad)
	armat(1,2)=-sin(prad)*cos(b0rad)
	armat(1,3)= sin(prad)*sin(b0rad)
	armat(2,1)= sin(prad)
	armat(2,2)= cos(prad)*cos(b0rad)
	armat(2,3)=-cos(prad)*sin(b0rad)
	armat(3,1)= 0.0
	armat(3,2)= sin(b0rad)
	armat(3,3)= cos(b0rad)
c
	rset=.true.
	return
	end
c
c  calculate asmat from ypos
c  xx = asmat * xxs, xxs = xx * transpose of asmat
c  asmat describes the rotation around the z-axis by the 
c  angle ypos
c
	subroutine setas
	include 'coord.fc'
c
	asmat(1,1)= cos(ypos)
	asmat(1,2)=-sin(ypos)
	asmat(1,3)= 0.0
	asmat(2,1)= sin(ypos)
	asmat(2,2)= cos(ypos)
	asmat(2,3)= 0.0
	asmat(3,1)= 0.0
	asmat(3,2)= 0.0
	asmat(3,3)= 1.0
c
	sset=.true.
	return
	end
c
c  unit vectors of S-coordinates
c
	subroutine svu( ap,al, apu,alu,hu )
	include 'coord.fc'
	dimension apu(3),alu(3),hu(3),apu0(3),alu0(3),hu0(3)
c
	apu0(1)= cos(ap)
	apu0(2)= 0.0
	apu0(3)=-sin(ap)
	alu0(1)=-sin(al)*sin(ap)
	alu0(2)= cos(al)
	alu0(3)=-sin(al)*cos(ap)
	hu0 (1)= cos(al)*sin(ap)
	hu0 (2)= sin(al)
	hu0 (3)= cos(al)*cos(ap)
c
	apu(1)=asmat(1,1)*apu0(1)+asmat(1,2)*apu0(2)+asmat(1,3)*apu0(3)
	apu(2)=asmat(2,1)*apu0(1)+asmat(2,2)*apu0(2)+asmat(2,3)*apu0(3)
	apu(3)=asmat(3,1)*apu0(1)+asmat(3,2)*apu0(2)+asmat(3,3)*apu0(3)
	alu(1)=asmat(1,1)*alu0(1)+asmat(1,2)*alu0(2)+asmat(1,3)*alu0(3)
	alu(2)=asmat(2,1)*alu0(1)+asmat(2,2)*alu0(2)+asmat(2,3)*alu0(3)
	alu(3)=asmat(3,1)*alu0(1)+asmat(3,2)*alu0(2)+asmat(3,3)*alu0(3)
	hu (1)=asmat(1,1)* hu0(1)+asmat(1,2)* hu0(2)+asmat(1,3)* hu0(3)
	hu (2)=asmat(2,1)* hu0(1)+asmat(2,2)* hu0(2)+asmat(2,3)* hu0(3)
	hu (3)=asmat(3,1)* hu0(1)+asmat(3,2)* hu0(2)+asmat(3,3)* hu0(3)
	return
	end
c
c  unit vectors of R-coordinates
c
	subroutine rvu( apr,alr, apru,alru,hu )
	include 'coord.fc'
	dimension apru(3),alru(3),hu(3),apru0(3),alru0(3),hu0(3)
c
	apru0(1)= cos(apr-cl0eff)
	apru0(2)= 0.0
	apru0(3)=-sin(apr-cl0eff)
	alru0(1)=-sin(alr)*sin(apr-cl0eff)
	alru0(2)= cos(alr)
	alru0(3)=-sin(alr)*cos(apr-cl0eff)
	hu0 (1)= cos(alr)*sin(apr-cl0eff)
	hu0 (2)= sin(alr)
	hu0 (3)= cos(alr)*cos(apr-cl0eff)
	apru(1)=
     &	armat(1,1)*apru0(1)+armat(1,2)*apru0(2)+armat(1,3)*apru0(3)
	apru(2)=
     &	armat(2,1)*apru0(1)+armat(2,2)*apru0(2)+armat(2,3)*apru0(3)
	apru(3)=
     &	armat(3,1)*apru0(1)+armat(3,2)*apru0(2)+armat(3,3)*apru0(3)
	alru(1)=
     &	armat(1,1)*alru0(1)+armat(1,2)*alru0(2)+armat(1,3)*alru0(3)
	alru(2)=
     &	armat(2,1)*alru0(1)+armat(2,2)*alru0(2)+armat(2,3)*alru0(3)
	alru(3)=
     &	armat(3,1)*alru0(1)+armat(3,2)*alru0(2)+armat(3,3)*alru0(3)
	hu (1) =
     &	armat(1,1)*  hu0(1)+armat(1,2)*  hu0(2)+armat(1,3)*  hu0(3)
	hu (2) =
     &	armat(2,1)*  hu0(1)+armat(2,2)*  hu0(2)+armat(2,3)*  hu0(3)
	hu (3) =
     &	armat(3,1)*  hu0(1)+armat(3,2)*  hu0(2)+armat(3,3)*  hu0(3)
	return
	end
c
c  unit vectors of the coordinate system specified by 'mesh'
c  xx is given in the C-coordinates
c
	subroutine vu( xx, vu1,vu2,vu3)
c
	include 'sys.fc'
	include 'coord.fc'
	dimension xx(3),vu1(3),vu2(3),vu3(3)
c
	if( mesh.eq.0 ) then
	  vu1(1)=xdu(1)
	  vu1(2)=xdu(2)
	  vu1(3)=xdu(3)
	  vu2(1)=ydu(1)
	  vu2(2)=ydu(2)
	  vu2(3)=ydu(3)
	  vu3(1)=zdu(1)
	  vu3(2)=zdu(2)
	  vu3(3)=zdu(3)
	elseif( mesh.eq.1 ) then
	  vu1(1)=xtu(1)
	  vu1(2)=xtu(2)
	  vu1(3)=xtu(3)
	  vu2(1)=ytu(1)
	  vu2(2)=ytu(2)
	  vu2(3)=ytu(3)
	  vu3(1)=ztu(1)
	  vu3(2)=ztu(2)
	  vu3(3)=ztu(3)
	elseif( mesh.eq.2 ) then
	  call ctos(xx,ap,al,h)
	  call svu(ap,al, vu1,vu2,vu3)
	elseif( mesh.ge.3 ) then
	 call ctor(xx,apr,alr,h)
	 call rvu(apr,alr,vu1,vu2,vu3)
	else
	  write(msg,*) '(subr.vu    ) error: mesh=',mesh
	  errflg=2
	endif
	return
	end
c
c  relation between latitude(al) and y-value in S and R coordinates
c  -halfpi <= al <= halfpi
c  -halfpi <= y <= halfpi
c
c  y as a function of al
c
	function yofal(al)
c
	data halfpi/1.5707963/
	if(al.gt.halfpi) then
	  yofal= halfpi
	else if(al.lt.-halfpi) then
	  yofal=-halfpi
	else
	  yofal=asin(al/halfpi)
	endif
	return
c
c  al as a function of y
c
	entry alofy(y)
c
	if(y.gt.halfpi) then
	  alofy=halfpi
	else if(y.lt.-halfpi) then
	  alofy=-halfpi
	else
	  alofy=halfpi*sin(y)
	endif
	return
c
c  dy/d(al) as a function of al
c
	entry dyofal(al)
c
	if(abs(al).gt.halfpi) then
	  dyofal=0.0
	else
	  dyofal=1.0/sqrt(abs(1.0-(al/halfpi)**2))/halfpi
	endif
	return
c
c  d(al)/dy as a function of y
c
	entry dalofy(y)
c
	if(abs(y).gt.halfpi) then
	  dalofy=0.0
	else
	  dalofy=halfpi*cos(y)
	endif
	return
	end
c
c  height above the lower boundary
c
	function height(xx,mesh1)
c
	include 'coord.fc'
c
      dimension xx(8),xxt(8)
c
	if(nsp.lt.2) then
        if (mesh1.eq.0) then
            height=xx(3)
          else if (iabs(mesh1).eq.1) then
            call ctot(xx,xxt)
            height=xxt(3)
          else if (mesh1.eq.2) then
            call ctos(xx,ap,al,height)
          else if (mesh1.ge.3) then
            call ctor(xx,apr,alr,height)
          endif
	else
          height=1.0e20
          do 10 n=1,nsp
	    h1=highm(n,xx)
          height = min(height,h1)
 10	  continue
	endif
      return
      end
c
c  add/subtract 2*pi and make ap's compatible with longitudes of
c  spherical coordinates specified by 'mesh'
c
	subroutine modulo(ap,mesh1,np)
c
c  np = 2 or 6
c
	include 'sys.fc'
	include 'coord.fc'
	dimension ap(np)
	data pi,halfpi/3.14159265, 1.5707963/
	pi2=2.0*pi
	e=1.0e-3
c
c  mesh points 1,3,5 are eastern mesh points
c  mesh points 2,4,6 are western mesh points
c  modulo 2*pi adjustments are done for all odd- or even-numbered
c  mesh points as a group
c
c  for mesh=2,3: -pi < ap < pi, cl0eff=0
c  for mesh=4:     0 < ap, cl0eff < 2*pi
c
	if(mesh1.eq.4) then
	  if(ap(1).lt.0.0-e) then
     	    ap(1)=ap(1)+ pi2*float(int((-ap(1)+pi2)/pi2))
	    if(msglvl.gt.2)
     &	    write(lst,*) 'modulo ap1-=',ap(1),mesh1
	  else if(ap(1).gt.pi2+e) then
     	    ap(1)=ap(1)- pi2*float(int((ap(1)-pi2+pi2)/pi2))
	    if(msglvl.gt.2)
     &	    write(lst,*) 'modulo ap1+=',ap(1),mesh1
	  endif
	else
	  if(ap(1).lt.-pi-e) then
     	    ap(1)=ap(1)+ pi2*float(int((pi-ap(1))/pi2))
	    if(msglvl.gt.2)
     &	    write(lst,*) 'modulo ap1-=',ap(1),mesh1
	  else if(ap(1).gt.pi+e) then
     	    ap(1)=ap(1)- pi2*float(int((ap(1)+pi)/pi2))
	    if(msglvl.gt.2)
     &	    write(lst,*) 'modulo ap1+=',ap(1),mesh1
	  endif
	endif
c
c  points 1,3,5 should have nearly the same ap
c  points 2,4,6 should have nealry the same ap
c
	if(np.gt.2) then
	  do 10 n=3,np,2
	  if(ap(n)-ap(1).lt.-pi) then
	    ap(n)=ap(n)+pi2*float(int((pi+ap(1)-ap(n))/pi2))
	    if(msglvl.gt.2)
     &	    write(lst,*) 'modulo o-,n,ap=',n,ap(n)
	  else if(ap(n)-ap(1).gt.pi) then
	    ap(n)=ap(n)-pi2*float(int((pi+ap(n)-ap(1))/pi2))
	    if(msglvl.gt.2)
     &	    write(lst,*) 'modulo o+,n,ap=',n,ap(n)
	  endif
 10	  continue
	  do 20 n=4,np,2
	  if(ap(n)-ap(2).lt.-pi) then
	    ap(n)=ap(n)+pi2*float(int((pi+ap(2)-ap(n))/pi2))
	    if(msglvl.gt.2)
     &	    write(lst,*) 'modulo e-,n,ap=',n,ap(n)
	  else if(ap(n)-ap(2).gt.pi) then
	    ap(n)=ap(n)-pi2*float(int((pi+ap(n)-ap(2))/pi2))
	    if(msglvl.gt.2)
     &	    write(lst,*) 'modulo e+,n,ap=',n,ap(n)
	  endif
 20	  continue
	endif
c
c  longitude extent should be smaller than 360 deg
c  points 2,4,6 must be westward of point 1
c
	if(ap(2)-ap(1).gt. pi2+e) then
	  do 30 n=2,np,2
     	  ap(n)=ap(n)- pi2*float(int((ap(n)-ap(1))/pi2))
	  if(msglvl.gt.2)
     &	    write(lst,*) 'modulo1 n,ap(n)=',n,ap(n)
 30	  continue
	else if(ap(2)-ap(1).lt. 0.0-e) then
	  do 35 n=2,np,2
     	  ap(n)=ap(n)+ pi2*float(int((ap(1)-ap(n)+pi2)/pi2))
	    if(msglvl.gt.2)
     &	    write(lst,*) 'modulo2 n,ap(n)=',n,ap(n)
 35	  continue
	endif
c
c  as a result, -pi < ap < 3*pi for mesh1=2,3
c                 0 < ap < 4*pi for mesh1=4
c
	return
	end
c
c  extension for binary star (multi-sphere) model
c
c  projection of cartesian position xx onto the surface of a star
c  the selected star number is returned as n
c  xx is modified
c
	subroutine projm(xx,n,ap,al)
	include 'coord.fc'
	dimension xx(3)
c
	do 10 n=1,nsp
	r2=(xx(1)-xxspm(1,n))**2+(xx(2)-xxspm(2,n))**2
	if(r2.gt.rspm(n)) go to 10
	call projrm(n,xx,ap,al)
	return
 10	continue
	n=nsp
c  the projection was not successful, and return the results for n=nsp
	return
	end
c
c  projection of cartesian position xx onto the spherical surface
c  of radius rsp1 centered at star No.n
c  z=xx(3) is calculated from x=xx(1) and y=xx(2)
c
        subroutine prjsrm(n,xx,rsp1)
        include 'coord.fc'
        dimension xx(3)
c
        xx(3)=xxspm(3,n)
        r2=(xx(1)-xxspm(1,n))**2+(xx(2)-xxspm(2,n))**2
        if(r2.lt.rsp1**2) xx(3)=sqrt(abs(rsp1**2-r2))+xxspm(3,n)
        return
        end
c
c  projection of cartesian position xx onto the surface of star No.n
c  return value: R-coordinates (ap=longitude, al=co-latitude)
c                xx(3) is modified in this routine
c  p and b0 are given in common coord
c  setdu and setarm must be called prior to this routine
c
        subroutine projrm(n,xx,apr,alr)
c
        include 'sys.fc'
        include 'coord.fc'
        dimension xx(3)
c
        if(.not.dset) then
          write(msg,*) '(subr.projrm) error: setdu has not been called'
          errflg=2
          return
        endif
c
        if(.not.rsetm(n)) then
          write(msg,*) '(subr.projrm) error: setarm has not been called'
          errflg=2
          return
	endif
c
        call prjsrm(n,xx,rspm(n))
        call ctorm(n,xx,apr,alr,h)
c  h should be nearly 0
        return
        end
c
c  transform from R-coordinates to C-coordinates
c  apr=longitude, alr=co-latitude
c  p and b0 are given in common coord
c
c  setdu and setarm must be called prior to this routine
c
        subroutine rtocm(n,apr,alr,h,xx)
        include 'sys.fc'
        include 'coord.fc'
        dimension xx(3),xxr(3)
        data halfpi/1.5707963/
c
        if(.not.dset) then
          write(msg,*) '(subr.rtocm ) error: setdu has not been called'
          errflg=2
          return
        endif
c
        if(.not.rsetm(n)) then
          write(msg,*) '(subr.rtocm ) error: setarm has not been called'
          errflg=2
          return
        endif
c
        call convsx( rspm(n)+h, 
     &	             halfpi-alr, apr, convx,convy,convz )
        xxr(1)=convy
        xxr(2)=convz
        xxr(3)=convx
        xx1=armatm(1,1,n)*xxr(1)
     &	   +armatm(1,2,n)*xxr(2)
     &	   +armatm(1,3,n)*xxr(3)
        xx2=armatm(2,1,n)*xxr(1)
     &	   +armatm(2,2,n)*xxr(2)
     &	   +armatm(2,3,n)*xxr(3)
        xx3=armatm(3,1,n)*xxr(1)
     &	   +armatm(3,2,n)*xxr(2)
     &	   +armatm(3,3,n)*xxr(3)
        xx(1)=xx1+xxspm(1,n)
        xx(2)=xx2+xxspm(2,n)
        xx(3)=xx3+xxspm(3,n)
        return
c
c  transform from C-coordinates to R-coordinates
c
        entry ctorm(n,xx,apr,alr,h)
c
        xx1=xx(1)-xxspm(1,n)
        xx2=xx(2)-xxspm(2,n)
        xx3=xx(3)-xxspm(3,n)
        xxr(1)=xx1*armatm(1,1,n)+xx2*armatm(2,1,n)+xx3*armatm(3,1,n)
        xxr(2)=xx1*armatm(1,2,n)+xx2*armatm(2,2,n)+xx3*armatm(3,2,n)
        xxr(3)=xx1*armatm(1,3,n)+xx2*armatm(2,3,n)+xx3*armatm(3,3,n)
        convx=xxr(3)
        convy=xxr(1)
        convz=xxr(2)
        call convxs( convx,convy,convz, r,theta,phi )
        apr=phi
        alr=halfpi-theta
        h=r-rspm(n)
        return
        end
c
c  transform from R-coordinates to D-coordinates
c  setdu and setarm must be called prior to this routine
c
        subroutine rtodm(n,apr,alr,h,xxd)
        include 'coord.fc'
        dimension xxd(3),xx(3)
c
        call rtocm(n,apr,alr,h,xx)
        call ctod(xx,xxd)
        return
c
c  transform from D-coordinates to R-coordinates
c
        entry dtorm(n,xxd,apr,alr,h)
c
        call dtoc(xxd,xx)
        call ctorm(n,xx,apr,alr,h)
        return
        end
c
c  calculate armatm from p and b0 for star No.n
c  xx = armatm * xxr, xxr = xx * transpose of armatm
c
        subroutine setarm(n)
        include 'coord.fc'
c
        armatm(1,1,n)= cos(pradm(n))
        armatm(1,2,n)=-sin(pradm(n))*cos(b0radm(n))
        armatm(1,3,n)= sin(pradm(n))*sin(b0radm(n))
        armatm(2,1,n)= sin(pradm(n))
        armatm(2,2,n)= cos(pradm(n))*cos(b0radm(n))
        armatm(2,3,n)=-cos(pradm(n))*sin(b0radm(n))
        armatm(3,1,n)= 0.0
        armatm(3,2,n)= sin(b0radm(n))
        armatm(3,3,n)= cos(b0radm(n))
c
        rsetm(n)=.true.
        return
        end
c
c  unit vectors of R-coordinates
c
	subroutine rvum(n, apr,alr, apru,alru,hu )
	include 'coord.fc'
	dimension apru(3),alru(3),hu(3),apru0(3),alru0(3),hu0(3)
c
	apru0(1)= cos(apr)
	apru0(2)= 0.0
	apru0(3)=-sin(apr)
	alru0(1)=-sin(alr)*sin(apr)
	alru0(2)= cos(alr)
	alru0(3)=-sin(alr)*cos(apr)
	hu0 (1)= cos(alr)*sin(apr)
	hu0 (2)= sin(alr)
	hu0 (3)= cos(alr)*cos(apr)
	apru(1)=armatm(1,1,n)*apru0(1)
     &	       +armatm(1,2,n)*apru0(2)
     &	       +armatm(1,3,n)*apru0(3)
	apru(2)=armatm(2,1,n)*apru0(1)
     &	       +armatm(2,2,n)*apru0(2)
     &	       +armatm(2,3,n)*apru0(3)
	apru(3)=armatm(3,1,n)*apru0(1)
     &	       +armatm(3,2,n)*apru0(2)
     &	       +armatm(3,3,n)*apru0(3)
	alru(1)=armatm(1,1,n)*alru0(1)
     &	       +armatm(1,2,n)*alru0(2)
     &	       +armatm(1,3,n)*alru0(3)
	alru(2)=armatm(2,1,n)*alru0(1)
     &	       +armatm(2,2,n)*alru0(2)
     &	       +armatm(2,3,n)*alru0(3)
	alru(3)=armatm(3,1,n)*alru0(1)
     &	       +armatm(3,2,n)*alru0(2)
     &	       +armatm(3,3,n)*alru0(3)
	hu (1) =armatm(1,1,n)*  hu0(1)
     &	       +armatm(1,2,n)*  hu0(2)
     &	       +armatm(1,3,n)*  hu0(3)
	hu (2) =armatm(2,1,n)*  hu0(1)
     &	       +armatm(2,2,n)*  hu0(2)
     &	       +armatm(2,3,n)*  hu0(3)
	hu (3) =armatm(3,1,n)*  hu0(1)
     &	       +armatm(3,2,n)*  hu0(2)
     &	       +armatm(3,3,n)*  hu0(3)
	return
	end
c
c  height from the surface of star No.n
c
	function highm(n,xx)
	include 'coord.fc'
c
	dimension xx(8)
c
	highm = sqrt((xx(1)-xxspm(1,n))**2
     &                 +(xx(2)-xxspm(2,n))**2
     &                 +(xx(3)-xxspm(3,n))**2) - rspm(n)
	return
	end
