;+
; NAME :	cardate
; PURPOSE :
;	for a given Carrington rotation number,
;	calculate the date when Carrington longitude 180 deg
;	is at the central meridian
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	cardate, rotno, date, time, doy
; INPUTS :
;	rotno = Carrington rotation number
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	date = 'yyyymmdd'
;	time = 'hh:00:00' (not accurate to min/sec)
;       doy =  day of year
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/06
;-
;------------------------------------------------------------------------
;
      pro carsub, fyear, rotno,  cl0
;	copied from sunparam.pro
;	fyear is the year and date expressed in 365.25-day year
;	rotno = Carrington rotation number
;	cl0 = Carrington longitude of the central meridian

	rd=3.14159265358979d0/180.0d0

	t1 = (fyear-1900)*365.25d0 - 3293.5d0
; t1 = JD -2418313.5
        t =(t1+3293.5d0)/36525.0d0
; t = (JD -2415020)/36525
	tt=t1+20093.5d0
; tt = JD -2398220
	gml=(279.691d0 + 36000.769d0*t)*rd
	g =(358.476d0 + 35999.050d0*t)*rd
	tgl=gml+(1.919d0-0.0048d0*t)*rd*sin(g)+0.020d0*rd*sin(2.0d0*g)
	ai =7.250d0*rd
	omg=(73.739167d0+0.013958d0*tt/365.25d0)*rd
	cbclm=-cos(tgl-omg)
	cbslm=-sin(tgl-omg)*cos(ai)
	xlm=atan(cbslm,cbclm)
	itgl=long( ((tgl-omg)/rd) /360.0d0)
        cl0=(360.0d0-360.0d0/25.38d0*tt)*rd+xlm+itgl*360.0d0*rd
	cl0=cl0/rd
	lc=long(cl0/360.0d0)
	cl0=float(cl0-360.0d0*(lc-1))
	rotno=-lc-42
	return
end

pro cardate, rotno, date, time, doy=doy, fyear=fyear

	if n_params() eq 0 then begin
		rotno=0
		read,'enter Carrington rotation number > ',rotno
	endif
; guess
	fyear1 = rotno*0.0744663d0 + 1854.1628d0
 
retry:
	carsub,fyear1,rotno1,cl1
	if rotno1 eq rotno then begin
		if cl1 lt 180.0 then begin
			fyear2=fyear1-1.0d0/365.0d0
			carsub,fyear2,rotno2,cl2
			if rotno2 ne rotno1 then begin
				print,'unexpected error'
				return
			endif
			fyear=fyear1-(fyear2-fyear1)/(cl2-cl1)*(cl1-180.0d0)
		endif else begin
			fyear2=fyear1+1.0d0/365.0d0
			carsub,fyear2,rotno2,cl2
			if rotno2 ne rotno1 then begin
				print,'unexpected error'
				return
			endif
			fyear=fyear1-(fyear2-fyear1)/(cl2-cl1)*(cl1-180.0d0)
		endelse
	endif else begin
		fyear1 = fyear1 - (rotno1-rotno)*0.0744663d0
		goto, retry
	endelse

; print,'fyear1,rotno1,cl1=',fyear1,rotno1,cl1
; print,'fyear2,rotno2,cl2=',fyear2,rotno2,cl2
; carsub,fyear,rotno0,cl0
; print,'fyear0,rotno0,cl0=',fyear,rotno0,cl0

	year=fix(fyear)
	doy=365.25*(fyear-year)+1.0
	idoy=fix(doy)
	feb=28
	if (year mod 100) eq 0 and (year mod 400 ne 0) then feb=28 $
	else if (year mod 4 eq 0) then feb = 29
	bom=[1,32,60,91,121,152,182,213,244,274,305,335]
	if feb eq 29 then bom[2:11]=bom[2:11]+1
	loc = where(bom le idoy, count)
	month = loc(count-1) + 1
	day = doy - bom(loc(count-1)) +1
	iday = fix(day)
	hour = fix(24.0*(day-iday))
	date=string(year,month,day, $
		format='(i4,i2.2,i2.2)' )
	time = string(hour,format="(i2.2,':00:00')")
        if n_params() eq 0 then begin
		print,'Carrington rotation number',rotno
		print,'longitude=180 deg on ', $
		strmid(date,0,4)+'/'+strmid(date,4,2)+'/'+ $
		strmid(date,6,2)+', ', strmid(time,0,2)+' UT'
	endif
	return
end

