;+
; NAME :	cff1
; PURPOSE :
;	calculate current-free field based on the longitudinal
;	field (BL) in rdata (planar geometry)
; CATEGORY :
;	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	cff1, mparam, rdata [,bxp, byp] [,/faceon] [,/nosave]
;		[,h=h] [,bzp=bzp]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	/faceon: if set, assume that given magnetogram is for BN, regardless
;	of the actual position on the solar disk
;	/nosave: if set, bxp and byp are not stored in rdata
;	h : height above the surface (not along the line of sight,
;	    but perpendicular to the boundary plane)
;	bzp : line-of-sight component of current-free field
;	      if h=0, this is identical to the input BL
;
; OUTPUTS :
;	bxp, byp : current-free transverse fields
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	mparam will be modified
; RESTRICTIONS :	none
; PROCEDURE :
;	use FFT to calculate current-free fields
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/10/16: /nosave added
;	TS 2001/01/31: h and bzp parameters added
;-
;----------------------------------------------------------------------
;

pro setbtp, kwx,kwy, alx,aly, cx,cy, amat, h=h,cfactor=cf
;	calculate complex coefficients (cx,cy) to be multiplied
;	on the Fourier transform of BL
;	kwx,kwy = integer indices
;	alx = x-width /(2*pi)
;	amat = matrix of coordinate transform between data and tangential
;	coordinates
;	h = height above the boundary plane
;	cfactor = exp(i*h*ckz)
;	
	ci = complex (0.0, 1.0)
	akwx=kwx/alx
	akwy=kwy/aly
	akx=akwx*amat(0,0)
	aky=akwx*amat(0,1)+akwy*amat(1,1)
	akz=sqrt(akx*akx+aky*aky)
	ckz=ci*akz-akwx*amat(0,2)-akwy*amat(1,2)
	cx=(akwx+ckz*amat(0,2))/(ckz*amat(2,2))
	cy=(akwy+ckz*amat(1,2))/(ckz*amat(2,2))
	if keyword_set(h) then begin
		cf=exp(ci*ckz*h)
		cx=cx*cf
		cy=cy*cf
	endif else cf=1.0
	return
	end

pro cff1, mparam, rdata, bxp, byp, faceon = faceon, nosave=nosave, $
	h=h, bzp=bzp

	common blkt, coord_t

	if not keyword_set(h) then h=0.0

	nx = mparam.nx
	ny = mparam.ny
	dx = mparam.dx
	dy = mparam.dy

	pi = 3.141592
	nx2 = 2*nx
	ny2 = 2*ny
	alx = dx*nx2/(2.0*pi)
	aly = dy*ny2/(2.0*pi)

	kz = kloc( mparam, /z)

        if kz eq -1 then begin
                print,'(proc.cff1) error:kz=',kz
                return
        endif

	bl0 = total(rdata(*,*,kz))/(nx*ny)

	rtemp = fltarr(nx2,ny2) + bl0
	rtemp( 0:(nx-1), 0:(ny-1) ) = rdata(*,*,kz)
	cftbz = fft( rtemp, -1)
	cx = cftbz
	cy = cx
	cz = cx

	if keyword_set(faceon) then $
		amat= [ [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0] ] $
	else begin
		setdu, mparam
		settu, mparam
		amat = coord_t.atmat
	endelse

	bn0 = bl0/amat(2,2)
	bx0 = amat(0,2)*bn0
	by0 = amat(1,2)*bn0

	for j=1, ny2 do begin
	for i=1, nx2 do begin
		if i eq 1 and j eq 1 then goto, endloop
		kwx = i-1
		kwy = j-1
		if kwx gt nx then kwx = kwx -nx2
		if kwy gt ny then kwy = kwy -ny2
		setbtp, kwx,kwy, alx,aly, cx1, cy1, amat, $
			h=h,cfactor=cf1
		cx(i-1,j-1) = cx1
		cy(i-1,j-1) = cy1
		cz(i-1,j-1) = cf1
	endloop:
	endfor
	endfor

	kwy = ny
	for i=1, nx2 do begin
		kwx = i-1
		if kwx gt nx then kwx = kwx -nx2
		setbtp, kwx, kwy, alx,aly, cx1, cy1, amat, $
			h=h,cfactor=cf1
		setbtp, kwx,-kwy, alx,aly, cx2, cy2, amat, $
			h=h,cfactor=cf2
		cx(i-1, kwy) = 0.5*(cx1+cx2)
		cy(i-1, kwy) = 0.5*(cy1+cy2)
		cz(i-1, kwy) = 0.5*(cf1+cf2)
	endfor

	kwx = nx
	for j=1, ny2 do begin
		kwy = j-1
		if kwy gt ny then kwy = kwy -ny2
		setbtp,  kwx,kwy, alx,aly, cx1, cy1, amat, $
			h=h,cfactor=cf1
		setbtp, -kwx,kwy, alx,aly, cx2, cy2, amat, $
			h=h,cfactor=cf2
		cx(kwx, j-1) = 0.5*(cx1+cx2)
		cy(kwx, j-1) = 0.5*(cy1+cy2)
		cz(kwx, j-1) = 0.5*(cf1+cf2)
	endfor

	kwx = nx
	kwy = ny
	setbtp,  kwx, kwy, alx,aly, cx1, cy1, amat, $
		h=h,cfactor=cf1
	setbtp, -kwx, kwy, alx,aly, cx2, cy2, amat, $
		h=h,cfactor=cf2
	setbtp,  kwx,-kwy, alx,aly, cx3, cy3, amat, $
		h=h,cfactor=cf3
	setbtp, -kwx,-kwy, alx,aly, cx4, cy4, amat, $
		h=h,cfactor=cf4
	cx(kwx, kwy) = 0.25*(cx1+cx2+cx3+cx4)
	cy(kwx, kwy) = 0.25*(cy1+cy2+cy3+cy4)
	cz(kwx, kwy) = 0.25*(cf1+cf2+cf3+cf4)

	cx(0,0) = 0.0
	cy(0,0) = 0.0
	cz(0,0) = 0.0

	cx = cx*cftbz
	cy = cy*cftbz
	cz = cz*cftbz

	cx = fft( cx, 1)
	bxp = float( cx( 0:(nx-1), 0:(ny-1) ) ) + bx0

	cy = fft( cy, 1)
	byp = float( cy( 0:(nx-1), 0:(ny-1) ) ) + by0

	cz = fft( cz, 1)
	bzp = float( cz( 0:(nx-1), 0:(ny-1) ) ) + bl0

      kindm2 = (mparam.kindm/10) *10

      kxp = kloc( mparam, /px )
      kyp = kloc( mparam, /py )
      if not keyword_set(nosave) then begin
            mparam.alpha = 0.0
            if kxp eq -1 then begin
                  kxp = newk( mparam, rdata )
                  mparam.status.bxp = kxp
            endif
            if kyp eq -1 then begin
                  kyp = newk( mparam, rdata )
                  mparam.status.byp = kyp
            endif
            rdata(*,*,kxp) = bxp
            rdata(*,*,kyp) = byp
            mparam.status.blp = kz
            mparam.nframe = mparam.nframe +3
            mparam.kindm = kindm2 + 7L
      endif else begin
            if kxp ge 0 and kyp ge 0 then mparam.kindm = kindm2 + 7L $
		else mparam.kindm = kindm2 +3L
      endelse

	return
end

