;+
; NAME :	curlb1
; PURPOSE :
;	calculate curl B from (BX,BY) (planar geometry)
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	curlb1, mparam, rdata [,jn]
;		[,/faceon] [,smooth=smooth] [,/nosave] [,method=method]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	/faceon : if set, assume the region is at disk center
;	smooth : if /smooth is set, apply 3x3 smoothing filter to (BX,BY)
;		     for smooth >1, 2*smooth+1 is the size of the smoothing filter
;	/nosave : if set, jn is not stored in rdata
;	method: 1=usual differentiation, 2=parity insensitive differentiation 
; OUTPUTS :
;	jn : normal component of electric current
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	mparam will be modified
; RESTRICTIONS :	none
; PROCEDURE :
;	use spline fitting to obtain derivatives of BX and BY
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/10/16 jn added as argument, /nostore, method added
;-
;------------------------------------------------------------------------
;

pro curlb1, mparam, rdata, jn, faceon=faceon, smooth=smooth, $
	nosave=nosave, method=method

	common blkt, coord_t

	if not keyword_set(method) then method=1

	kindm = mparam.kindm

	if kindm lt 0 then pol2b, mparam, rdata

	kindmb = kindm mod 10

	if kindmb eq 1 or kindmb eq 5 then begin
		print, '(proc.curlb1) error: magnetogram is not vector magnetogram'
		return
	endif

	if kindmb eq 2 or kindmb eq 6 then begin
		vector1, mparam, rdata, faceon=faceon
		kindmb = mparam.kindm mod 10
	endif

	nx = mparam.nx
	ny = mparam.ny
	dx = mparam.dx
	dy = mparam.dy

	pi = 3.141592

	kx = kloc( mparam, /x)
	ky = kloc( mparam, /y)
	kz = kloc( mparam, /z)
        if kx eq -1 or ky eq -1 or kz eq -1 then begin
                print,'(proc.curlb1) error:kx,ky,kz=',kx,ky,kz
                return
        endif
	
	if keyword_set(faceon) then begin
		ainv= [ [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0] ] 
	endif else begin
		setdu, mparam
		settu, mparam
		amat = coord_t.atmat
		ainv = coord_t.atinv
	endelse

	Bx = rdata(*,*,kx)
	By = rdata(*,*,ky)
	Bz = rdata(*,*,kz)
	if keyword_set(smooth) then begin
		Bx = smooth(Bx, 2*smooth+1)
		By = smooth(By, 2*smooth+1)
		Zz = smooth(Bz, 2*smooth+1)
	endif
	
	if method eq 1 then begin
		dBxdy1 = surface3(Bx,dxydata=[dx,dy],ndiv=1,diffy=1,/zeroderiv)
		dBydx1 = surface3(By,dxydata=[dx,dy],ndiv=1,diffx=1,/zeroderiv)
		curlbz = dBydx1 - dBxdy1
	endif

	if method eq 2 then begin
		bxy=Bx*By
		bxy2=Bx*Bx-By*By
		bt2 = ( Bx*Bx +By*By ) > 1.0e-4

		dbxydx = surface3(bxy,dxydata=[dx,dy],ndiv=1,diffx=1, $
			solution=solution,/zeroderiv)
		dbxydy = surface3(bxy,dxydata=[dx,dy],ndiv=1,diffy=1, $
			known=solution,/zeroderiv)
		dbxy2dx =surface3(bxy2,dxydata=[dx,dy],ndiv=1,diffx=1, $
			solution=solution,/zeroderiv)
		dbxy2dy =surface3(bxy2,dxydata=[dx,dy],ndiv=1,diffy=1, $
			known=solution,/zeroderiv)

		curlbz = (-0.5*By*dbxy2dx + Bx*dbxydx $
			 - 0.5*Bx*dbxy2dy - By*dbxydy)/bt2

;		bx2=bx*bx
;		by2=by*by
;		bxy=bx*by
;		bxy2=bx2-by2
;		dBx2dy = surface3(bx2,dxydata=[dx,dy],ndiv=1,diffy=1)
;		dBy2dx = surface3(by2,dxydata=[dx,dy],ndiv=1,diffx=1)
;		dBxydx = surface3(bxy,dxydata=[dx,dy],ndiv=1,diffx=1, $
;				solution=solution)
;		dBxydy = surface3(bxy,dxydata=[dx,dy],ndiv=1,diffy=1, $
;				known=solution)
;		dBxy2dx = surface3(bxy2,dxydata=[dx,dy],ndiv=1,diffx=1, $
;				solution=solution)
;		dBxy2dy = -surface3(bxy2,dxydata=[dx,dy],ndiv=1,diffy=1, $
;				known=solution)
;		curlbz = bx/(bt2*bt2)* $
;			(bx2*dBxydx -0.5*bxy*dBxy2dx -0.5*bt2*dBx2dy) $
;		       - by/(bt2*bt2)* $
;			(by2*dBxydx -0.5*bxy*dBxy2dy -0.5*bt2*dBy2dx)
	endif	

	dBzdx = surface3(Bz,dxydata=[dx,dy],ndiv=1,diffx=1, $
		solution=solution,/zeroderiv)
	dBzdy = surface3(Bz,dxydata=[dx,dy],ndiv=1,diffy=1, $
		known=solution,/zeroderiv)

	jn = ainv(2,0)*dBzdy - ainv(2,1)*dBzdx + ainv(2,2)*curlbz

	if not keyword_set(nosave) then begin
		kj = kloc( mparam, /j)
		if kj eq -1 then begin
			kj = newk( mparam, rdata )
			mparam.nframe = mparam.nframe +1
		endif	

		rdata(*,*,kj) = jn

		kindm3 = ( mparam.kindm/100 ) * 100

		if keyword_set(faceon) then begin
			mparam.status.jl = kj
			mparam.kindm = kindm3 + 10L + kindmb
		endif else begin
			mparam.status.jn = kj
			mparam.kindm = kindm3 + 20L + kindmb
		endelse
	endif

	return
end

