;+
; NAME :	disp3
; PURPOSE :
;	display magnetograms and field lines
;	arbitrary view angles and scaling/positioning
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	disp3 mparam,rdata,ldata
;		[,magfile=magfile] [,loffile=loffile] [,dir=dir]
;		[,/nomenu] [,/whitebg]
;		[,printer=printer] [,printdir=printdir]
;		[,/fixedlevel] [,/autolevel] 
;		[,btmin=btmin] [,/btmax] [,arrowloc=arrowloc]
;
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
;	ldata  : field line data
;	set /nomenu in order to indicate that valid data are in these arrays
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	magfile : name of magnetogram file
;	if this is not specified and /nomenu is absent, dialog_pickfile is
;	called for file selection
;	loffile : name of field line file
;	if this is not specified and /nomenu is absent, dialog_pickfile is
;	called for file selection
;	dir : directory where data exist
;	      if not specified, assume the current directory
;	/nomenu : if specified, data are assumed to be already in 
;		   rdata array, and file selection menu is not displayed
;	/whitebg : white background (default is dark background)
;	printer : name of printing device :'ps'(default) or 'eps'
;	printdir : directory where printer output file is created
;	/autolevel : contour levels are automatically adjusted to data (default)
;	/fixedlevel : contour levels are fixed as given by setlevel.pro
;	btmin : minimum value of BT to be plotted
;	btmax : value of BT corresponding to longest arrows
;	arrowloc : array of pointers where arrows will be plotted
; OUTPUTS :
;	creates magnetogram plot
;	if arguments mparam, rdata, and ldata are given, these data
;	are returned to the calling program
; COMMON BLOCKS : 	gparam as an include file @gparam.pro
; SIDE EFFECTS :		none
; RESTRICTIONS :	X-window environment is assumed
; PROCEDURE :	this is a front end interface to
;				dispm3 (display magnetograms),
;				displ3 (display field lines),
;				plotm3 (print out magnetograms),
;				plotl3 (print out field lines)
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/11/02
;-
;------------------------------------------------------------------------
;
pro disp3, mparam, rdata, ldata, $
	magfile=magfile, loffile=loffile, dir=dir, $
	nomenu=nomenu, whitebg=whitebg, $
	printer=printer, printdir=printdir, $
	fixedlevel=fixedlevel, autolevel=autolevel, $
	btmin = btmin, btmax = btmax, arrowloc=arrowloc

@gparam

	olddir = ' '
	spawn, 'pwd', olddir
	set_plot,'x'

	if not keyword_set(printer) then printer='ps'
	if printer eq 'ps' or printer eq 'eps' then print_dev = 'ps'
	disp_dev = !d.name

	if n_elements(printdir) eq 0 then printdir = getenv('HOME')
	outfile= printdir+'/magpack2_'+getenv('USER')+'.'+printer
	plotted=0
	if not keyword_set(dir) then dir = '.'

	magdir=dir
	if not keyword_set(magfile) and not keyword_set(nomenu) then $
		magfile = dialog_pickfile(path=magdir, $
			get_path=magdir, $
			title='magnetogram file selection')
	if not keyword_set(loffile) and not keyword_set(nomenu) then $
		loffile = dialog_pickfile(path=magdir, $
			title='field line file selection')

	if keyword_set(magfile) then readm,mparam,rdata,filename=magfile
	if keyword_set(loffile) then begin
		readl,ldata,filename=loffile
		meshl = lparam.mesh mod 10
	endif

	if not keyword_set(btmax) then btmax=1000.0
	if not keyword_set(btmin) then btmin=100.0

	papersize=18.0	; 18cm

top:
	region = 0
	lof = 0

plot_again:

; display magnetogram
	dispm3, mparam, rdata, whitebg=whitebg, /noask, $
		fixedlevel=fixedlevel, autolevel=autolevel, $
		btmin=btmin, btmax=btmax, arrowloc=arrowloc

; display field lines

	if n_elements(ldata) gt 0 and region eq 0 then begin
		print,format="('field lines : 0=skip, 1=show ',$)"
		repeat read, lof until ( lof eq 0 or lof eq 1 )
		if lof ne 0 then  $
			displ3, mparam, ldata, whitebg=whitebg
	endif

	cmd = ' '

re_enter1:
	print, format="('1=plot again, 2=hardcopy, 9=exit > ',$)"
re_enter2:
	read, cmd 

	case cmd of
	'1':	goto, plot_again
	'2':	begin
		set_plot, print_dev
		if not plotted then begin
			device, filename=outfile, $
				xsize=papersize, ysize=papersize, $
				yoffset=2
;		use xsize=ysize to ensure isotropic scaling
			if printer eq 'eps' then device,/encapsulated
		endif
		plotted =1
		plotm3, mparam, rdata, papersize=papersize
		if lof ne 0 then  $
			plotl3, mparam, ldata
		set_plot, disp_dev
		goto, re_enter1
		end
	'9':	begin
		if plotted then begin
			set_plot, print_dev
			device, /close
			set_plot, disp_dev
			print, '(proc.disp3 ) output is written to ',outfile
		endif
		goto, fin
		end
	else:	goto, re_enter2
	endcase
	goto, top

fin:
	cd, olddir(0)
	return
end

