;+
; NAME :	dispazm
; PURPOSE :
;	display field vector azimuths (pinwheel diagram)
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	dispazm, mparam, rdata
;		[,nlevel=nlevel] [,btmax=btmax] [,btmin=btmin]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	nlevel : a value from 1 to 4, intensity levels assigned to BT
;	         if nlevel>1 azimuths of weaker fields appear dimmer
;	btmin : minimum value of BT to be plotted
;	btmax : value of BT corresponding to longest arrows
;
; OUTPUTS :
;	creates field azimuth plot
;	azimuths are color-coded as
;		-180 deg : red
;		-120 deg : yellow
;		 -60 deg : green
;		   0 deg : cyan (toward +X direction)
;		  60 deg : blue
;		 120 deg : purple
;		 
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	X-window environment is assumed
;					data must contain vector magnetogram
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/10/20
;-
;------------------------------------------------------------------------
;
pro dispazm, mparam, rdata, $
	nlevel=nlvl, btmax=btmax,btmin=btmin

	if mparam.mesh ne 0 then begin
		print,'mesh ne 0'
		return
	endif 

	if not keyword_set(nlvl) then nlvl=1
	nlvl = nlvl <4
	if not keyword_set(btmax) then btmax=1000.0
	if not keyword_set(btmin) then btmin=100.0


	kx = kloc(mparam,/x)
	ky = kloc(mparam,/y)
	kz = kloc(mparam,/z)
	if kx lt 0 or ky lt 0 or kz lt 0 then begin
		print,'error: kx,ky,kz=',kx,ky,kz
		return
	endif
	if mparam.status.s1 lt 0 then begin
		ka = newk(mparam,rdata)
		mparam.status.s1 =ka
	endif else ka = mparam.status.s1

	if mparam.status.s2 lt 0 then begin
		kt = newk(mparam,rdata)
		mparam.status.s2 =kt
	endif else kt = mparam.status.s2
	mparam.nframe = mparam.nframe+2

	rdata(*,*,kt) = sqrt(rdata(*,*,kx)^2 +rdata(*,*,ky)^2)
	azim = atan(rdata(*,*,ky),rdata(*,*,kx))*180.0/3.141592
		; from -180 to +180

	xs=mparam.xs
	ys=mparam.ys
	nx=mparam.nx
	ny=mparam.ny
	dx=mparam.dx
	dy=mparam.dy
	xe = xs + dx *(nx-1)
	ye = ys + dy *(ny-1)
	kindm = mparam.kindm

	maxcolor = fix( !d.n_colors )
	if (maxcolor ge 255 ) then maxcolor=230

;	generate pinwheel pattern
;	36 angular points (10 deg step)
;	level max. 4 levels

	bw =12
	ddeg = 360/(3*bw)
	index = indgen(bw)
	incr = 1+indgen(bw)
	decr = bw-indgen(bw)
	zero = intarr(bw)

	rtable=intarr(maxcolor)
	gtable=intarr(maxcolor)
	btable=intarr(maxcolor)

	mapping=intarr(3*bw,nlvl+1)
;		nlvl+1 includes level below btmin

	f = 255/float(nlvl)/bw
	for lvl=1,nlvl do begin
		rtable(index+lvl*3*bw)=lvl*decr*f
		gtable(index+lvl*3*bw)=lvl*incr*f
		btable(index+lvl*3*bw)=lvl*zero*f
		mapping(index,lvl)=index+lvl*3*bw
		rtable(index+bw+lvl*3*bw)=lvl*zero*f
		gtable(index+bw+lvl*3*bw)=lvl*decr*f
		btable(index+bw+lvl*3*bw)=lvl*incr*f
		mapping(index+bw,lvl)=index+bw+lvl*3*bw
		rtable(index+2*bw+lvl*3*bw)=lvl*incr*f
		gtable(index+2*bw+lvl*3*bw)=lvl*zero*f
		btable(index+2*bw+lvl*3*bw)=lvl*decr*f
		mapping(index+2*bw,lvl)=index+2*bw+lvl*3*bw
	endfor

	gray = 3*bw*(nlvl+1)
	ngray = maxcolor-gray-8
	for i=gray,maxcolor-9 do begin
		rtable(i)=(i-gray)/float(maxcolor-9-gray)*255
		gtable(i)=(i-gray)/float(maxcolor-9-gray)*255
		btable(i)=(i-gray)/float(maxcolor-9-gray)*255
	endfor

	white	= maxcolor - 1
	red	= white - 1
	green	= red - 1
	blue	= green - 1
	yellow	= blue - 1
	cyan	= yellow - 1
	magenta	= cyan - 1
	black	= magenta - 1

	mx=1.0
	rtable( black:white )	= 127.0*[0, 2, 0, 2, 0,mx, 2, 2]
	gtable( black:white )	= 127.0*[0, 0, 2, 2,mx, 2,mx, 2]
	btable( black:white )	= 127.0*[0, 2, 2, 0, 2,mx, 0, 2]

	tvlct,rtable,gtable,btable

	btlog = loglevel(rdata(*,*,kt),btmin,btmax)

	rdata(*,*,ka) = mapping( $
		 (azim+180-ddeg/2)/ddeg > 0, nlvl*btlog+0.99)
;		bt = btmin <==> brightness level = 1-eps

	np = 50
	pie = intarr(2*np,2*np)

	for i=0,2*np-1 do for j=0,2*np-1 do begin
		xx = float(i-np)
		yy = float(j-np)
		r = sqrt(xx*xx+yy*yy)
		ang = atan(yy,xx)*180.0/3.141592
		if r lt np then begin
			pie(i,j)=mapping( $
		 	((ang+180-ddeg/2)/ddeg) > 0, nlvl*r/np+0.99)
		endif
	endfor

	x = xs + indgen(nx)*dx
	y = ys + indgen(ny)*dy

setlevel,mparam,rdata, $
      title1,intlevels,dummylevels,bplevels,bmlevels,dplevels,dmlevels, $
      jplevels,jmlevels,fplevels,fmlevels, $
      bunit,dunit,iunit,junit,funit

top:
	chrh = ' '
	chrc = ' '
	arrow=1
	title2 = ''
	title3 = ''
	
	kxlist = [kx]
	kylist = [ky]
	khlist = [ka, kt]
	kclist = [kz, kt]
	strx = [ 'BX' ]
	stry = [ 'BY' ]
	strh = [ 'AZ','BT' ]
	strc = [ 'BL','BT' ]
	nz = 2
	nxy=1
	nh = 2
	nc = 2
	nv = 1
	kc =-1
	kh =-1	
	if ( nz gt 0 ) then begin
		print,format='("halftone map: 0=skip",$)
		for i=1, nz do print,format='(", ",i1,"=",a5,$)', i,strh(i-1)
		repeat read,nh until ( nh ge 0 and nh le nz )
		if ( nh ne 0 ) then begin
			kh = khlist(nh-1)
			chrh = strmid( strh(nh-1), 0,2)
		endif

		print,format='("contour map : 0=skip",$)
		for i=1, nz do print,format='(", ",i1,"=",a5,$)', i,strc(i-1)
		repeat read,nc until ( nc ge 0 and nc le nz )
		if ( nc ne 0 ) then begin
			kc = kclist(nc-1)
			chrc = strmid( strc(nc-1), 0,2)
		endif
	endif
	if ( nxy gt 0 ) then begin
		if kloc(mparam,/px) ge 0 and mparam.alpha ne 0.0 then $
			print,'(BXP,BYP) is for alpha=',mparam.alpha
		print,format='("vector map  : 0=skip",$)'
		for i=1, nxy do print, $
		format='(", ",i1,"=(",a5,",",a5,")",$)', i,strx(i-1),stry(i-1)
		repeat read,nv until ( nv ge -nxy-1 and nv le nxy+1 )
		if ( nv lt 0 ) then begin
			nv = -nv
			arrow = 0
		endif
		if ( nv ne 0 and nv le nxy ) then begin
			kx = kxlist(nv-1)
			ky = kylist(nv-1)
		endif
		if ( nxy gt 1 and nv eq nxy+1 ) then begin
			kx = kxlist(1)
			ky = kylist(1)
		endif
	endif	
		
	window, 0, title='IDL 0 : MAGPACK2 display', xsize=600, ysize=700
	xwsize = !d.x_vsize
	ywsize = !d.y_vsize
;	xpixwidth= fix(xwsize) - 100
;	ypixwidth= fix(ywsize) - 80
	xpixwidth= fix(0.875*xwsize)
	ypixwidth= fix(0.875*ywsize)
; xpixwidth,ypixwidth = size of plot window in device coordinates

;	xcorner = 50
;	ycorner = 40
	xcorner = fix(0.1*xpixwidth)
	ycorner = fix(0.1*ypixwidth)
; xcorner,ycorner = lower left corner of plot window in device coordinates

; determine magnification factor for grayscale

	nx1 = fix(max([nx,nx*(dx/dy)]))
	ny1 = fix(max([ny,ny*(dy/dx)]))
	mag = min( [ xpixwidth/float(nx1), ypixwidth/float(ny1) ] )
	mag = fix(mag)

; determine minification factor for contour
	mini = max( [ nx, ny] )/100 > 1
	x = xs + dx*float(mini-1)*0.5 + indgen(nx/mini)*dx*mini
	y = ys + dy*float(mini-1)*0.5 + indgen(ny/mini)*dy*mini

; lower left and upper right corners of contour plot in device coordinates
	mapposition = [xcorner, ycorner, $
	               xcorner+mag*(nx1-1), ycorner+mag*(ny1-1) ]
; locations of title strings
	h = fix(0.04*ywsize)
	x3 = xcorner & y3 = ycorner + mag*ny +h
	x2 = x3      & y2 = y3 + h
	x1 = x3      & y1 = y2 + h

; character size
	cs=0.75

	erase

; halftone map

	if ( kh ge 0 ) then begin
		buf = congrid(rdata(*,*,kh), nx1,ny1)
		
		if ( chrh eq 'AZ' ) then begin
			bytebuf = buf
			interp=0
		endif else begin
			valmax = max(abs(buf))
			valmin=0.01*valmax
			bytebuf = gray + $
				 bytscl( loglevel( buf, valmin, valmax)) $
				/255.0*(ngray-1)
			interp=1
		endelse

		px = [[0.0, 0.0], [1.0/mag, 0.0]]
		py = [[0.0, 1.0/mag], [0.0, 0.0]]
		bytebuf = poly_2d(bytebuf,px,py,interp, $
				mag*nx1,mag*ny1, missing=0)

		tv, bytebuf( 0:(mag*(nx1-1)), 0:(mag*(ny1-1)) ), $
			xcorner, ycorner, order=0

		title3 = strmid('HALFTONE:'+strh(nh-1)+'                    ',0,20)
	endif 

; contour map

	nlevel =3
	if ( kc ge 0 ) then begin
		plevels = bplevels & mlevels = bmlevels
		unit=bunit

		if kc eq kt then $
			lvl125,rdata(*,*,kc),nlevel=nlevel, $
			plevel=plevels,mlevel=mlevels

     	sz = size(plevels)
     	nlevels = fix(sz(1))
     	plinestyle = fix(plevels)
     	mlinestyle = fix(mlevels)
     	pcolors = plinestyle
     	mcolors = mlinestyle
     	plabels=plinestyle
     	plabels(*)=0
     	mlabels=plabels

		plinestyle(*) = 0 ; solid
;		mlinestyle(*) = 1 ; dotted --  no good
		mlinestyle(*) = 2 ; dashed
		mlinestyle(*) = 5 ; long-dashed

		case chrc of
			'BL':  begin
				pcolors(*) = red
				mcolors(*) = blue
				end
            'BT':  begin
				pcolors(*) = green
				mcolors(*) = green
				end
		endcase

		title2 = strmid( $
			'CONTOURS:'+strc(nc-1) +'                    ',0,20) + $
  			'LEVELS='
		title2 = title2 + strlevel( plevels) +' '+ unit

		if mini gt 1 then begin
			qx = [[0.0, 0.0], [mini, 0.0]]
			qy = [[0.0, mini], [0.0, 0.0]]
			interp=1
			rbuf = poly_2d(rdata(*,*,kc),qx,qy,interp, $
					nx/mini,ny/mini,missing=0)
		endif else $
			rbuf=rdata(*,*,kc)

		contour, rbuf, x,y, $
		/noerase, $
		levels=[mlevels,plevels], $
		c_linestyle = [mlinestyle,plinestyle], $
		c_color = [mcolors, pcolors], $
		c_labels = [mlabels, plabels], $
		xstyle=1, ystyle=1, $
		color = white, $
		charsize=cs, $
;		/downhill, $
		position = mapposition, /device, $
		xtitle='!6X (10!E4!Nkm)', ytitle='!6Y (10!E4!Nkm)'
	endif

	if kc lt 0 then begin
		contour, bytarr(nx/mini,ny/mini), x,y, $
		/noerase, /nodata, $
		xstyle=1, ystyle=1, $
		color = white, $
		charsize=cs, $
		position = mapposition, /device, $
		xtitle='!6X (10!E4!Nkm)', ytitle='!6Y (10!E4!Nkm)'
	endif

; vector map

	if nv gt 0 then begin

		btmax = 1000.0
;           btmax = max ( rdata(*,*,kt) )
		kindmb = kindm mod 10
		if ( arrow gt 0 and kindmb gt 2 ) then arrow = 1 else arrow = 0
		if ( kx gt 18 and kindmb eq 6 ) then arrow =0
;		kindmb=6: data are pseudo B and Btp, and pseudo B is selected
		nbin = max([nx,ny])/50 +1

		plotvec, rdata(*,*,kx),rdata(*,*,ky), $
			xs=xs, ys=ys, dx=dx, dy=dy, $
			btmax=btmax, rbtmin=rbtmin, nbin=nbin, $
			arrow=arrow, color=fgcolor, /noinit

		title3 = title3 + $
			strmid( 'VECTORS :' + strx(nv-1) + ', ' + stry(nv-1) + $
			'          ', 0, 20 ) + $
			'MAX,MIN=' + string(btmax, btmin, format='(2f6.0)')
	endif


;	font = 0	; hardware font
;	font = 3	; default font
;	xyouts, x1,y1, /device, font=font, title1, color=fgcolor
;	xyouts, x2,y2, /device, font=font, title2, color=fgcolor
;	xyouts, x3,y3, /device, font=font, title3, color=fgcolor
	xyouts, x1,y1, /device, title1, color=white, charsize=cs
	xyouts, x2,y2, /device, title2, color=white, charsize=cs
	xyouts, x3,y3, /device, title3, color=white, charsize=cs

	if kh eq ka then tv,pie, !d.x_vsize-2*np-10, y3
	yn = ' '
	print, format='("repeat? (y/n) > ",$)
	repeat read, yn until $
		 (yn eq 'y' or yn eq 'Y' or yn eq 'n' or yn eq 'N') 

	if ( yn eq 'y' or yn eq 'Y' ) then goto, top

	return
end

;  .run dispasm executes the following.
; dispazm, mparam,rdata
; end
