;+
; NAME :	dispm
; PURPOSE :
;	display magnetograms
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	dispm, mparam, rdata [/oaodefault] [,/whitebg] [,/noask]
;		[/nowindow] [,/autolevel] [,/fixedlevel]
;		[,btmin=btmin] [,btmax=btmax] [,arrowloc=arrowloc]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	oaodefault : for 'show' program on spot@mtk.nao.ac.jp
;		plot I in gray scale, BL in contours, BT by bars
;		if oaodefault is not specified, interactive mode will be invoked
;	/whitebg : white background (default is dark background)
;	/noask :	if not set, you are asked to repeat plotting
;	/nowindow : use already-existing window
;	/autolevel : contour levels are automatically adjusted to data (default)
;	/fixedlevel : contour levels are fixed as given by setlevel.pro
;	btmin : minimum value of BT to be plotted
;	btmax : value of BT corresponding to longest arrows
;	arrowloc : array of pointers where arrows will be plotted
;
; OUTPUTS :
;	creates magnetogram plot
; COMMON BLOCKS : 	gparam is created (as included in @gparam.pro)
; SIDE EFFECTS : 	 window #0 is created and stays there after exit
; RESTRICTIONS :	X-window environment is assumed
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 1994/10: contour level setting is moved to @setlevel
;	TS 2000/08: vector plot has been separated to plotvec.pro
;	TS 2000/09: setlevel converted to procedure
;          binning uses congrid so that halftone maps always fill the screen
;          case of dx.ne.dy taken care
;	TS 2000/10: automatic level adjustment of contours
;	            graphics parameters moved from keyword parameter to
;	            common block 
;-
;------------------------------------------------------------------------
;
pro dispm, mparam, rdata, $
	oaodefault=oaodefault, whitebg=whitebg, noask=noask, $
	nowindow=nowindow, $
	autolevel=autolevel, fixedlevel=fixedlevel, $
	btmin = btmin, btmax = btmax, arrowloc=arrowloc

@gparam

	if mparam.mesh ne 0 then begin
		print,'(proc.dispm ) mesh ne 0'
		print,'     currently this procedure can only handle mesh=0'
;		return
	endif 
	if mparam.mesh lt 2 then km = ' (10!E4!Nkm)' else km=''

	if keyword_set(oaodefault) then fixedlevel=1
	if keyword_set(fixedlevel) then autolevel=0 else autolevel=1
		; default is autolevel as of 2000/10
	if not keyword_set(btmax) then btmax=1000.0
	if not keyword_set(btmin) then btmin=100.0

	setcolor, bandwidth, w_start, rb_start, g_start, color8, $
			whitebg = whitebg

	if keyword_set(whitebg) then begin
		fgcolor = color8.black 
		bgcolor = color8.white
	endif else begin
		fgcolor = color8.white
		bgcolor = color8.black
	endelse

	xs=mparam.xs
	ys=mparam.ys
	nx=mparam.nx
	ny=mparam.ny
	dx=mparam.dx
	dy=mparam.dy
	xe = xs + dx *(nx-1)
	ye = ys + dy *(ny-1)

	xwsize=600
	ywsize=700
	xfr = fix(0.925*xwsize)
	yfr = fix(0.925*xwsize)
	xfl = fix(0.075*xwsize)
	yfl = fix(0.075*xwsize)
; xfl,yfl = lower left corner of plot window in device coordinates
; xfr,yfr = upper right corner of plot window in device coordinates

; xsize, ysize = size of plot window in device coordinates
	xsize = xfr-xfl
	ysize = yfr-yfl

; determine magnification factor for grayscale

	nx1 = fix(max([nx,nx*(dx/dy)]))
	ny1 = fix(max([ny,ny*(dy/dx)]))
	mag = min( [ xsize/float(nx1), ysize/float(ny1) ] )

; determine minification factor for contour
	mini = max( [ nx, ny] )/100 > 1
	x = xs + dx*float(mini-1)*0.5 + indgen(nx/mini)*dx*mini
	y = ys + dy*float(mini-1)*0.5 + indgen(ny/mini)*dy*mini

; lower left and upper right corners of contour plot in device coordinates
	mapposition = [xfl, yfl, $
	               xfl+mag*(nx1-1), yfl+mag*(ny1-1) ]
; locations of title strings
	h = fix(0.04*ysize)
	x3 = xfl	 & y3 = yfl + mag*ny +h
	x2 = x3      & y2 = y3 + h
	x1 = x3      & y1 = y2 + h

; character size
	cs=1.0 ; 0.75

	if ( mparam.kindm lt 0 ) then pol2b, mparam, rdata
	kindm = mparam.kindm
	mklist, mparam, nxy,nz,kxlist,kylist,kzlist,strx,stry,strz

	setlevel,mparam,rdata, $
		oaodefault=oaodefault, $
		title1,intlevels,dummylevels,bplevels,bmlevels,dplevels,dmlevels, $
		jplevels,jmlevels,fplevels,fmlevels, $
		bunit,dunit,iunit,junit,funit

top:
	nh=0
	kh=-1
	nc=0
	kc=-1
	kx=-1
	ky=-1
	kx2=-1
	ky2=-1
	nv=0
	chrh = ' '
	chrc = ' '
	arrow=1
	title2 = ''
	title3 = ''
	
	if keyword_set (oaodefault) then begin
		kh = mparam.status.i
		kc = mparam.status.bl
		kx = mparam.status.bx
		ky = mparam.status.by
		chrh = 'I'
		chrc = 'B'
		strx = [ 'BX' ]
		stry = [ 'BY' ]
		strz = [ 'I', 'BL' ]
		nh = 1
		nc = 2
		nv = 1
	endif else begin
		if ( nz gt 0 ) then begin
			print,format='("halftone map: 0=skip",$)'
			for i=1, nz do print,format='(", ",i1,"=",a5,$)', i,strz(i-1)
			repeat read,nh until ( nh ge 0 and nh le nz )
			if ( nh ne 0 ) then begin
				kh = kzlist(nh-1)
				chrh = strmid( strz(nh-1), 0,1)
			endif

			print,format='("contour map : 0=skip",$)'
			for i=1, nz do print,format='(", ",i1,"=",a5,$)', i,strz(i-1)
			repeat read,nc until ( nc ge 0 and nc le nz )
			if ( nc ne 0 ) then begin
				kc = kzlist(nc-1)
				chrc = strmid( strz(nc-1), 0,1)
			endif
		endif
		if ( nxy gt 0 ) then begin
			if kloc(mparam,/px) ge 0 and mparam.alpha ne 0.0 then $
				print,'(BXP,BYP) is for alpha=',mparam.alpha
			print,format='("vector map  : 0=skip",$)'
			for i=1, nxy do print, $
			format='(", ",i1,"=(",a5,",",a5,")",$)', i,strx(i-1),stry(i-1)
			repeat read,nv until ( nv ge -nxy-1 and nv le nxy+1 )
			if ( nv lt 0 ) then begin
				nv = -nv
				arrow = 0
			endif
			if ( nv ne 0 and nv le nxy ) then begin
				kx = kxlist(nv-1)
				ky = kylist(nv-1)
			endif
			if ( nxy gt 1 and nv eq nxy+1 ) then begin
				kx = kxlist(1)
				ky = kylist(1)
				kx2 = kxlist(0)
				ky2 = kylist(0)
			endif
		endif	
	endelse
		
	window, 0, title='IDL 0 : MAGPACK2 display', xsize=600, ysize=700


	erase, bgcolor

; halftone map

	if ( kh ge 0 ) then begin
		buf = congrid(rdata(*,*,kh), nx1,ny1)/max(rdata(*,*,kh))
		
		if ( chrh eq 'I' ) then begin
			color = w_start
			bw = fix(0.7*bandwidth)	; max. brightness is 70%*255
			bytebuf = bytscl( buf, min=0.3, max=1.0 )
		endif else begin
			color = rb_start
			bw = 2*bandwidth
			valmax = max(abs(buf))
			if chrh eq 'J' then valmin=0.05*valmax $
					else valmin=0.01*valmax
			if ( chrh eq 'V' ) then begin
				bytebuf = bytscl( $
					loglevel( -buf, valmin, valmax), $
					max=1.0, min=-1.0 )
			endif else $
			if (chrh eq 'B' or chrh eq 'J' or chrh eq 'F') then begin
				bytebuf = bytscl( $
					loglevel( buf, valmin, valmax), $
					max=1.0, min=-1.0 )
			endif else begin
				bytebuf = bytscl( $
					loglevel( buf, valmin, valmax), $
					max=1.0, min=-1.0 )
			endelse
		endelse

		px = [[0.0, 0.0], [1.0/mag, 0.0]]
		py = [[0.0, 1.0/mag], [0.0, 0.0]]
		interp=1
		bytebuf = poly_2d(bytebuf,px,py,interp, $
				mag*nx1,mag*ny1, missing=0)

		tv, $
		color + fix( bytebuf( 0:(mag*(nx1-1)), 0:(mag*(ny1-1)) ) $
		/255.0*(bw-1) ), $
		xfl, yfl, order=0

		title3 = strmid('HALFTONE:'+strz(nh-1)+'                    ',0,20)
	endif 

; contour map

	nlevel =5
	if ( kc ge 0 ) then begin
		case chrc of
		'B':	begin
			plevels = bplevels & mlevels = bmlevels
			unit=bunit
			end
		'V':  begin
			plevels = dplevels & mlevels = dmlevels
			unit=dunit
			end
		'I':	begin
			plevels = intlevels & mlevels = dummylevels
			unit=iunit
			end
		'J':	begin
			plevels = jplevels & mlevels = jmlevels
			unit=junit
			end
		'F':	begin
			plevels = fplevels & mlevels = fmlevels
			unit=funit
			end
		else: begin
			plevels = fplevels & mlevels = fmlevels
			unit=funit
			end
		endcase
		if keyword_set(autolevel) and chrc ne 'I' then $
			lvl125,rdata(*,*,kc),nlevel=nlevel, $
			plevel=plevels,mlevel=mlevels

		sz = size(plevels)
		nlevels = fix(sz(1))
		plinestyle = intarr(nlevels)	; 0=solid
		mlinestyle = plinestyle +2		; 2=dashed (1=dotted -- no good)
		pcolors = intarr(nlevels)
		mcolors = pcolors
		plabels= intarr(nlevels)
		mlabels=plabels

		case chrc of
		'I':	begin
			pcolors(*) = color8.yellow
			mcolors(*) = color8.yellow ; dummy
			end
		'V':	begin
			pcolors(*) = color8.blue
			mcolors(*) = color8.red
			end
		else:	begin
			pcolors(*) = color8.red
			mcolors(*) = color8.blue
			end
		endcase

		title2 = strmid( $
			'CONTOURS:'+strz(nc-1) +'                    ',0,20) + $
  			'LEVELS='
		title2 = title2 + strlevel( plevels) +' '+ unit

		if mini gt 1 then begin
			qx = [[0.0, 0.0], [mini, 0.0]]
			qy = [[0.0, mini], [0.0, 0.0]]
			interp=1
			rbuf = poly_2d(rdata(*,*,kc),qx,qy,interp, $
					nx/mini,ny/mini,missing=0)
		endif else $
			rbuf=rdata(*,*,kc)

		contour, rbuf, x,y, $
		/noerase, $
		levels=[mlevels,plevels], $
		c_linestyle = [mlinestyle,plinestyle], $
		c_color = [mcolors, pcolors], $
		c_labels = [mlabels, plabels], $
		xstyle=1, ystyle=1, $
		color = fgcolor, $
		charsize=cs, $
		position = mapposition, /device, $
		xtitle='!6X'+km, ytitle='!6Y'+km
	endif

	if kc lt 0 then begin
		contour, bytarr(nx/mini,ny/mini), x,y, $
		/noerase, /nodata, $
		xstyle=1, ystyle=1, $
		color = fgcolor, $
		charsize=cs, $
		position = mapposition, /device, $
		xtitle='!6X'+km, ytitle='!6Y'+km
	endif

; vector map

	if ( kx ge 0 and ky ge 0 ) then begin

		kindmb = kindm mod 10
		if ( arrow gt 0 and kindmb gt 2 ) then arrow = 1 else arrow = 0
		if ( kx ne kloc(mparam,/px) and kindmb eq 6 ) then arrow =0
;		kindmb=6: data are pseudo B and Btp, and pseudo B is selected
		nbin = max([nx,ny])/75 +1

		plotvec, rdata(*,*,kx),rdata(*,*,ky), $
			xs=xs, ys=ys, dx=dx, dy=dy, $
			btmax=btmax, btmin=btmin, nbin=nbin, $
			arrow=arrow, color=fgcolor, /noinit, loc=arrowloc

		if kx2 ge 0 and ky2 ge 0 then  $
			plotvec, rdata(*,*,kx2),rdata(*,*,ky2), $
			xs=xs, ys=ys, dx=dx, dy=dy, $
			btmax=btmax, btmin=btmin, nbin=nbin, $
			arrow=arrow, color=color8.green, /noinit

		title3 = title3 + $
			strmid( 'VECTORS :' + strx(nv-1) + ', ' + stry(nv-1) + $
			'          ', 0, 20 ) + $
			'MAX,MIN=' + string(btmax, btmin, format='(2f6.0)')
	endif


;	font = 0	; hardware font
;	font = 3	; default font
;	xyouts, x1,y1, /device, font=font, title1, color=fgcolor
;	xyouts, x2,y2, /device, font=font, title2, color=fgcolor
;	xyouts, x3,y3, /device, font=font, title3, color=fgcolor
	xyouts, x1,y1, /device, title1, color=fgcolor, charsize=cs
	xyouts, x2,y2, /device, title2, color=fgcolor, charsize=cs
	xyouts, x3,y3, /device, title3, color=fgcolor, charsize=cs

	yn = ' '
	if keyword_set(oaodefault) then return

	if not keyword_set(noask) then begin
		print, format='("repeat? (y/n) > ",$)
		repeat read, yn until $
			 (yn eq 'y' or yn eq 'Y' or yn eq 'n' or yn eq 'N') 

		if ( yn eq 'y' or yn eq 'Y' ) then goto, top
	endif

	return
end

;  .run disp executes the following.
dispm, mparam,rdata
end
