;+
; NAME :	dispm3
; PURPOSE :
;	display magnetograms
;	arbitrary view angles and scaling/positioning
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	dispm3, mparam, rdata [,/whitebg] [,/noask]
;		[,/autolevel] [,/fixedlevel]
;		[,btmin=btmin] [,btmax=btmax] [,arrowloc=arrowloc]
;		[,meshl=meshl]
;
; INPUTS :
;     mparam : magnetogram parameters
;     rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	whitebg : white background (default is dark background)
;	noask :	if it is not set, you are asked to repeat plotting
;     /autolevel : contour levels are automatically adjusted to data (default)
;     /fixedlevel : contour levels are fixed as given by setlevel.pro
;     btmin : minimum value of BT to be plotted
;     btmax : value of BT corresponding to longest arrows
;     arrowloc : array of pointers where arrows will be plotted
;	meshl : initial mode of coordinate projection
; OUTPUTS :
;	creates magnetogram plot
; COMMON BLOCKS : 	gparam is created (as included in @gparam.pro)
; SIDE EFFECTS :	window #0 is created and stays there after exit
; RESTRICTIONS :	X-window environment is assumed
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 1994/10: contour level setting is moved to @setlevel
;	TS 2000/08: vector plot has been separated to plotvec.pro
;	TS 2000/09: setlevel converted to procedure
;          binning uses congrid so that halftone maps always fill the screen
;          case of dx.ne.dy taken care
;	TS 2000/10: automatic level adjustment of contours
;-
;------------------------------------------------------------------------
;

@sphere
@dispr3

pro cntr3, info,data, $
	device=device, normal=normal, $
	color= colors, style= styles

	ncont = n_elements(info)
	for i=1,ncont-1 do begin
		level=info(i).level
		color = colors(level)
		style = styles(level)
		npoints = info(i).n
		offset = info(i).offset
		pointer=[indgen(npoints),0]	; last 0 is to close the contour
		x = data(0,offset+pointer)
		y = data(1,offset+pointer)
; somehow must get rid of the rectangular frame...
		pline2, x,y, color=color,style=style, $
			device=device, normal=normal
	endfor

	return
end
		
pro proj, x,y,z, mesh=mesh
	if mesh eq 0 then begin
		z=0.0*z
	endif else $
	if abs(mesh) eq 1 then begin
		projt, x,y,z
	endif else $
	if mesh eq 2 then begin
		projs, x,y,z
	endif else $
	if mesh ge 3 then begin
		projr, x,y,z
	endif
	return
end

;  this subroutine is for contour plot
;  (xp,yp) are given in D-coordinates if mesh=0

pro pline2, xp,yp, color=color, thick=thick, style=style, $
	device=device, normal=normal
@gparam

	np = n_elements(xp)
	if np le 1 then return

	if mesh eq 0 then begin
		dtoc, xp,yp,0*xp, x,y,z
		proj, x,y,z, mesh=meshl
	endif else $
;  mesh=1 and meshl=1
	if mesh eq 1 then begin
		ttoc, xp,yp,0*xp, x,y,z
	endif else $
;  mesh=2 and meshl=2,3,4
	if mesh eq 2 then begin
		stoc, xp,alofy(yp),0*xp, x,y,z
	endif else $
;  mesh=3,4 and meshl=2,3,4
	if mesh eq 3 or mesh eq 4 then begin
		rtoc, xp,alofy(yp),0*xp, x,y,z
	endif
	rotpt, x,y,z, xr,yr,zr

	ixp = gfact*(xr-gxc) +xfc
	iyp = gfact*(yr-gyc) +yfc
	clipw = [xclipl, yclipb, xclipr, yclipt]
	if visibl(xr,yr,zr, loc=loc) then begin
		n=n_elements(loc)
		if n eq 1 then begin
			plots, ixp(loc),iyp(loc), clip=clipw, noclip=0, $
				device=device, normal=normal, $
				color=color,thick=thick, linestyle=style
		endif else begin
			break = where([loc,loc(n-1)]-[loc(0),loc] ne 1, count)
			for m=0,count-2 do begin
				plots, ixp(loc(break(m):break(m+1)-1)), $
				       iyp(loc(break(m):break(m+1)-1)), $
					clip=clipw, noclip=0, $
					device=device, normal=normal, $
					color=color,thick=thick, linestyle=style
			endfor
		endelse
	endif
	return
end

;  this subroutine is for halftone maps
;  (xp,yp) are given in D-coordinates

pro fill2, xp,yp, color=color
@gparam

	np = n_elements(xp)
	if np le 1 then return

	if mesh eq 0 then begin
		dtoc, xp,yp,0*xp, x,y,z
		proj, x,y,z, mesh=meshl
	endif else $
;  mesh=1 and meshl=1
	if mesh eq 1 then begin
		ttoc, xp,yp,0*xp, x,y,z
	endif else $
;  mesh=2 and meshl=2,3,4
	if mesh eq 2 then begin
		stoc, xp,alofy(yp),0*xp, x,y,z
	endif else $
;  mesh=3,4 and meshl=2,3,4
	if mesh eq 3 or mesh eq 4 then begin
		rtoc, xp,alofy(yp),0*xp, x,y,z
	endif
	rotpt, x,y,z, xr,yr,zr

	ixp = gfact*(xr-gxc) +xfc
	iyp = gfact*(yr-gyc) +yfc
	clipw = [xclipl, yclipb, xclipr, yclipt]
	if visibl(xr,yr,zr) then $
	polyfill, ixp,iyp, clip=clipw, noclip=0, $
		/device, color=color

	return
end

pro fill3, xp,yp,zp, color=color, thick=thick
@gparam

	np = n_elements(xp)
	if np le 1 then return

	rotpt, xp,yp,zp, xr,yr,zr

	ixp = gfact*(xr-gxc) +xfc
	iyp = gfact*(yr-gyc) +yfc
	clipw = [xclipl, yclipb, xclipr, yclipt]
	if visibl(xr,yr,zr) then $
		polyfill, ixp,iyp, clip=clipw, noclip=0, $
		/device, color=color
	return
end

;  this subroutine is for field line plot and for spherical grid
;  (xp,yp,zp) are in C-coordinates

pro pline3, xp,yp,zp, color=color, gmode=gmode, thick=thick, $
		device=device, normal=normal
@gparam

	if not keyword_set(gmode) then gmode=1

	np = n_elements(xp)
	if np le 1 then return

	if gmode eq 1 then $
		rotpt, xp,yp,zp, xr,yr,zr $
	else $
		pers, xp,yp,zp, xr,yr,zr

	ixp = gfact*(xr-gxc) +xfc
	iyp = gfact*(yr-gyc) +yfc
	clipw = [xclipl, yclipb, xclipr, yclipt]

	if visibl(xr,yr,zr, loc=loc) then begin
		n=n_elements(loc)
		if n eq 1 then begin
			plots, ixp(loc),iyp(loc), clip=clipw, noclip=0, $
				device=device, normal=normal, $
				color=color,thick=thick
		endif else begin
			break = where([loc,loc(n-1)]-[loc(0),loc] ne 1, count)
			for m=0,count-2 do begin
				plots, ixp(loc(break(m):break(m+1)-1)), $
				       iyp(loc(break(m):break(m+1)-1)), $
					clip=clipw, noclip=0, $
					device=device, normal=normal, $
					color=color,thick=thick
			endfor
		endelse
	endif
	return
end

;  this subroutine is for N,S,E,W marks
;  (xp,yp,zp) are in C-coordinates
pro pline5, xp,yp,zp, xp1,yp1, color=color, $
		device=device, normal=normal
@gparam

	np = n_elements(xp)
	rotptv, xp,yp,zp, x,y,z
	ixp = gfact*(x-gxc) +xfc
	iyp = gfact*(y-gyc) +yfc
	clipw = [xclipl, yclipb, xclipr, yclipt]
	if visibl(x,y,z, loc=loc) then $
		plots, ixp(loc),iyp(loc), clip=clipw, noclip=0, $
			device=device, normal=normal, $
			color=color,thick=thick

	xp1=ixp(0)
	yp1=iyp(0)
	return
end

pro disphalftone, mparam, b, $
	color=color, nop=nop, bwidth=bwidth
;  b is normalized to [0,1] (I) or [0,2] (V,B)
@gparam
;  assume 1.5mm is the size of elementary filling pattern

	xs=mparam.xs
	ys=mparam.ys
	dx=mparam.dx
	dy=mparam.dy
	nx=mparam.nx
	ny=mparam.ny

	ndx=fix(max([1.0, dx0/(0.15*devcm/gfact)+0.51]))
	ndy=fix(max([1.0, dy0/(0.15*devcm/gfact)+0.51]))
	ddx=dx/ndx
	ddy=dy/ndy

	bb = rebin(b, nx*ndx,ny*ndy)
	nx2 = nx*ndx -ndx+1
	ny2 = ny*ndy -ndy+1
	bb = bb(0:nx2-1, 0:ny2-1)

	xp = fltarr(nx2,ny2)
	yp = fltarr(nx2,ny2)
	for j=0,ny2-1 do xp(*,j) = xs + ddx*indgen(nx2)
	for i=0,nx2-1 do yp(i,*) = ys + ddy*indgen(ny2)

	if mesh eq 0 then begin
		dtoc, xp,yp,0*xp, x,y,z
		proj, x,y,z, mesh=meshl
	endif else $
;  mesh=1 and meshl=1
	if mesh eq 1 then begin
		ttoc, xp,yp,0*xp, x,y,z
	endif else $
;  mesh=2 and meshl=2,3,4
	if mesh eq 2 then begin
		stoc, xp,alofy(yp),0*xp, x,y,z
	endif else $
;  mesh=3,4 and meshl=2,3,4
	if mesh eq 3 or mesh eq 4 then begin
		rtoc, xp,alofy(yp),0*xp, x,y,z
	endif
	rotpt, x,y,z, xr,yr,zr

	ixp = gfact*(xr-gxc) +xfc
	iyp = gfact*(yr-gyc) +yfc

	xwdev = xclipr-xclipl+1
	ywdev = yclipt-yclipb+1
	buf = bytarr(xwdev, ywdev)

	for j=1,ny2-1 do $
	for i=1,nx2-1 do begin
		fillcolor = color+(bwidth-1)*bb(i,j)
		il = min(ixp(i-1:i,j-1:j)-xclipl,max=ir)
		jb = min(iyp(i-1:i,j-1:j)-yclipb,max=jt)
		if visibl(xr(i-1:i,j-1:j),yr(i-1:i,j-1:j),zr(i-1:i,j-1:j)) $
			and il ge 0 and ir le xwdev-1 $
			and jb ge 0 and jt le ywdev-1 then $
			buf(il:ir,jb:jt)=fillcolor	
	endfor
	tv,buf,xclipl,yclipb,order=0

	return
end

;  intensity map with dots
;  intlvl(0) < intensity             -- no dots
;  intlvl(1) < intensity < intlvl(0) -- one dot in 5x5 area
;  intlvl(2) < intensity < intlvl(1) -- 4  dots in 5x5 area
;  intlvl(3) < intensity < intlvl(2) -- 13 dots in 5x5 area
;              intensity < intlvl(3) -- ignore (maybe beyond the limb)

pro dispdot, mparam,rdata, color=color, $
	device=device, normal=normal

@gparam
	intlvl=[ 0.90, 0.80, 0.70, 0.05]

	idot = intarr(5,5,3)
	idot(2,2,0) = 1
	idot(*,*,1) = [ [0,0,0,0,0], $
			    [0,1,0,1,0], $
			    [0,0,0,0,0], $
			    [0,1,0,1,0], $
			    [0,0,0,0,0] ]
	idot(*,*,2) = [ [1,0,1,0,1], $
			    [0,1,0,1,0], $
			    [1,0,1,0,1], $
			    [0,1,0,1,0], $
			    [1,0,1,0,1] ]
	loc0 = where(idot(*,*,0) eq 1)
	loc1 = where(idot(*,*,1) eq 1)
	loc2 = where(idot(*,*,2) eq 1)
	idd0=5
	valmax=total(rdata)/n_elements(rdata)

	xs=mparam.xs
	ys=mparam.ys
	dx=mparam.dx
	dy=mparam.dy
	nx=mparam.nx
	ny=mparam.ny

;  assume 2mm is the size of elementary dot patterns

	ndx=fix(max([1.0, dx0/(0.2*devcm/gfact)+0.51]))
	ndy=fix(max([1.0, dy0/(0.2*devcm/gfact)+0.51]))
	ddx=dx/ndx
	ddy=dy/ndy
	ddxa = fltarr(idd0,idd0)
	ddya = fltarr(idd0,idd0)
	for j=0,idd0-1 do ddxa(*,j) = ddx*indgen(idd0)/float(idd0)
	for i=0,idd0-1 do ddya(i,*) = ddy*indgen(idd0)/float(idd0)

	val = rebin(rdata, nx*ndx,ny*ndy)/valmax
	for j=1,ny-1 do $
	for i=1,nx-1 do begin
		xp0 = xs + dx*(i-1)
		yp0 = ys + dy*(j-1)
		if val(i,j) le intlvl(0) and val(i,j) ge intlvl(3) then begin
			if val(i,j) lt intlvl(2) then begin
				loc=loc0
			endif else $
			if val(i,j) lt intlvl(1) then begin
				loc=loc1
			endif else begin
				loc=loc2
			endelse
			for m=0,n_elements(loc)-1 do begin
				xp = xp0 + ddxa(loc(m))		
				yp = yp0 + ddya(loc(m))		
				pline2, [xp,xp], [yp,yp], color=color, $
					device=device, normal=normal
;  arguments of pline2 should be D-coordinates
			endfor
		endif
	endfor
	return
end

; plot bars/arrows
pro dispbar, mparam, rdata, kx,ky,kz, $
	color=color, $
	device=device, normal=normal, $
	btmax=btmax, btmin=btmin, loc=loc
@gparam

	if n_elements(color) eq 0 then color=!p.color
	if not keyword_set(btmax) then btmax = max ( sqrt( bx^2 + by^2) )
	if n_elements(btmin) eq 0 then btmin= 0.1*btmax

	xs=mparam.xs
	ys=mparam.ys
	dx=mparam.dx
	dy=mparam.dy
	nx=mparam.nx
	ny=mparam.ny

;  arrsep is minimum arrow separation
;  grid size of arrow plotting is dx*ndxar, dy*ndyar
	ndxar=fix(arrsep/(dx0*gfact)) >1
	ndyar=fix(arrsep/(dy0*gfact)) >1

;  arrow size = 1mm
	asize=0.1*devcm/gfact*(dx/dx0)
	f=0.4
;  minimum length of bars = 0.5mm
	ww=0.05*devcm/gfact*(dx/dx0)
	dx1=dx*float(ndxar)
	dy1=dy*float(ndyar)
	dbmax=sqrt(dx1^2+dy1^2)

	bt=sqrt(rdata(*,*,kx)^2+rdata(*,*,ky)^2)
	db=dbmax*loglevel(bt,btmin,btmax)
	width=dbmax*loglevel(bt,btmax,10.0*btmax)

	xp = fltarr(nx,ny)
	yp = fltarr(nx,ny)
	for j=0,ny-1 do xp(*,j)=xs+dx*indgen(nx)
	for i=0,nx-1 do yp(i,*)=ys+dy*indgen(ny)

	if mesh eq 0 then begin
		dtoc, xp,yp,0*xp, x,y,z
		proj, x,y,z, mesh=meshl
	endif else $
;  mesh=1 and meshl=1
	if mesh eq 1 then begin
		ttoc, xp,yp,0*xp, x,y,z
	endif else $
;  mesh=2 and meshl=2,3,4
	if mesh eq 2 then begin
		stoc, xp,alofy(yp),0*xp, x,y,z
	endif else $
;  mesh=3,4 and meshl=2,3,4
	if mesh eq 3 or mesh eq 4 then begin
		rtoc, xp,alofy(yp),0*xp, x,y,z
	endif

	for i=1,nx-2,ndxar do $
	for j=1,ny-2,ndyar do begin
		if db(i,j) lt ww then goto, endloop
		ux=rdata(i,j,kx)/bt(i,j)
		uy=rdata(i,j,ky)/bt(i,j)
		dxx=ux*db(i,j)
		dyy=uy*db(i,j)
		if kz ge 0 then $
			dzz=db(i,j)*rdata(i,j,kz)/bt(i,j) $
		else  dzz=0.0
		vx=-uy
		vy= ux
		if arrow then begin

;  (ux,uy) and (vx,vy) are orthogonal
;  arguments of pline2 should be D-coordinates
			xp1 = x(i,j) - 0.5*dxx
			xp2 = x(i,j) + 0.5*dxx
			yp1 = y(i,j) - 0.5*dyy
			yp2 = y(i,j) + 0.5*dyy
			zp1 = z(i,j) - 0.5*dzz
			zp2 = z(i,j) + 0.5*dxx
			pline3, [xp1,xp2],[yp1,yp2],[zp1,zp2], $
					color=color, $
					device=device, normal=normal
			xp1=xp2 - (asize*ux + f*asize*vx)
			yp1=yp2 - (asize*uy + f*asize*vy)
			xp3=xp2 - (asize*ux - f*asize*vx)
			yp3=yp2 - (asize*uy - f*asize*vy)
			pline3, [xp1,xp2,xp3],[yp1,yp2,yp3],[zp2,zp2,zp2], $
					color=color, $
					device=device, normal=normal
		endif else begin
			xp1 = x(i,j) + (-0.5*dxx + 0.5*vx*width(i,j))
			xp2 = x(i,j) + ( 0.5*dxx + 0.5*vx*width(i,j))
			yp1 = y(i,j) + (-0.5*dyy + 0.5*vy*width(i,j))
			yp2 = y(i,j) + ( 0.5*dyy + 0.5*vy*width(i,j))
			zp1 = z(i,j) - 0.5*dzz
			zp2 = z(i,j) + 0.5*dzz
			if width(i,j) ge 1.0 then begin
				xp3 = x(i,j) + ( 0.5*dxx - 0.5*vx*width(i,j))
				xp4 = x(i,j) + (-0.5*dxx - 0.5*vx*width(i,j))
				yp3 = y(i,j) + ( 0.5*dyy - 0.5*vy*width(i,j))
				yp4 = y(i,j) + (-0.5*dyy - 0.5*vy*width(i,j))
				xp5=xp1
				yp5=yp1
				fill3 ,[xp1,xp2,xp3,xp4,xp5], $
				       [yp1,yp2,yp3,yp4,yp5], $
					[zp1,zp2,zp2,zp1,zp1], color=color, $
					device=device, normal=normal
			endif else pline3,[xp1,xp2],[yp1,yp2],[zp1,zp2], $
					device=device, normal=normal, $
					color=color
		endelse
endloop:
	endfor
	return
end


pro dispm3, mparam, rdata, $
	whitebg=whitebg, noask=noask, $
	autolevel=autolevel, fixedlevel=fixedlevel, $
	btmin = btmin, btmax = btmax, arrowloc=arrowloc, $
	meshl=meshll

	common blkd, coord_d
	common blkt, coord_t
	common blks, coord_s
	common blkr, coord_r
@gparam
;	common gparam, $
;		nz, nxy, nh, nc, nv, arrow, dots, frame, globe, lof, region, $
;		mesh, meshl, cl0eff, $
;		devcm, vmicm, vmi, chrcm, arrsep, $
;		xwsize, ywsize, xsize, ysize, yfsize, $
;		xfl,yfl,xfr,yfr, xfc,yfc, $
;		gxc, gyc, gfact, $
;		color8, $
;		rotang, xang, zang, eyed, $
;		xf, yf, xclipl, xclipr, yclipb, yclipt, 
;		rotr, rotv, rotx, rotz, $
;		dx0, dy0, rotdeg, zdeg, xdeg, scale, dxc1, dyc1

	if n_elements(meshll) eq 0 then meshl=2 else meshl=meshll
;		if field line file has been read in disp3.pro, meshl is taken
;		from the field line file and passed to this program

	if keyword_set(oaodefault) then fixedlevel=1
	if keyword_set(fixedlevel) then autolevel=0 else autolevel=1
		; default is autolevel as of 2000/10
	if not keyword_set(btmax) then btmax=1000.0
	if not keyword_set(btmin) then btmin=100.0

	eyed=0.0
	arcsec=0.07
	rsun =69.6
	ulen =10.0^9

	if strmid(mparam.tsys,0,1) ne ' ' and $
		strmid(mparam.dateo,0,1) ne ' ' and $
		strmid(mparam.stime,0,1) ne ' ' then begin
			sunparam, tsys=tsys, mparam.dateo, mparam.stime, $
				rsunc, pc, b0c, cl0c, doy, irotno
			pc = 3.141592*pc/180.0
			b0c = 3.141592*b0c/180.0
			cl0c = 3.141592*cl0c/180.0
			nochange = 0
			settocal = 1	; set to calculated value
			reset = 2		; set to 0
			setrsp = nochange
			setp = nochange
			setb0 = nochange
			setcl0 = reset

			if setrsp eq settocal then begin
				rsp=arcsec*rsunc
			endif else $
			if setrsp eq reset then begin
				rsp = arcsec*rsun
			endif
			
			if setp eq settocal then begin
				if mparam.prad eq mparam.ypos then begin
					mparam.prad=pc
					mparam.ypos=mparam.prad
				endif else mparam.prad =pc
			endif else $
			if setp eq reset then begin
				if mparam.prad eq mparam.ypos then begin
					mparam.prad=0.0
					mparam.ypos=0.0
				endif else mparam.prad =0.0
			endif

			if setb0 eq settocal then begin	
				mparam.b0rad=b0c
			endif else $
			if setb0 eq reset then begin
				mparam.b0deg=0.0
			endif

			if setcl0 eq settocal then begin
				mparam.cl0rad=cl0c
			endif else $
			if setcl0 eq reset and mparam.mesh ne 4 then begin
				mparam.cl0deg =0.0
			endif
	endif

	setdu, mparam
	settu, mparam
	setas, mparam
	setar, mparam
	xxsp = coord_d.xxsp

	mesho=0
	if mparam.mesh eq -1 then begin
		mparam.mesh=0
	endif
	mesh = mparam.mesh
	  
	if mesh eq 1 and meshl gt 1  or $
	   mesh gt 1 and (meshl eq 0 or meshl eq 1) then begin
		transm, mparam, mesho=meshl
	    print, 'subr.transm called'
	endif else $
	if meshl lt 0  then begin
		print, 'error: meshl<0',meshl
		return
	endif

	xs=mparam.xs
	ys=mparam.ys
	nx=mparam.nx
	ny=mparam.ny
	dx=mparam.dx
	dy=mparam.dy
	rsp=mparam.rsp

	scale=0.0
;  devcm : device unit corresponding to 1cm
;  vmicm : line spacing in cm
;  chrcm : character size in cm (only for plotm)
	devcm = 100/2.54	; assume 100dpi
	vmicm = 0.6
	chrcm = 0.25
	asepcm = 0.5
	vmi = vmicm*devcm
	chrsiz = chrcm*devcm
	arrsep = asepcm*devcm

	rotdeg=0.0
	zdeg=0.0
	xdeg=0.0
	dxc1=0.0
	dyc1=0.0
	dx0=dx
	dy0=dy
;  dx0 and dy0 are physical lengths, while dx and dy may be
;  spherical coordinates

	if mesh le 1  or  meshl lt 0 then begin
		dxy=min([abs(dx),abs(dy)])
		dxy0=dxy
	endif else $
	if mesh ge 2 and meshl ge 0 then begin
		dx0=dx*rsp
		dy0=alofy(dy)*rsp
		dxy0=min([abs(dx0),abs(dy0)])
		dxy=min([abs(dx),abs(alofy(dy))])
	endif
	dxy0=max([1.0,float(min([nx,ny]))/50.0])*dxy0/3.0
	dxy =max([1.0,float(min([nx,ny]))/50.0])*dxy/3.0

;  four corners of observed region

	xf=[xs,xs+dx*(nx-1),xs+dx*(nx-1),xs,xs]
	yf=[ys,ys,ys+dy*(ny-1),ys+dy*(ny-1),ys]
;  xf and yf are in D-coordinates if mesh=0

	if meshl lt 0 then begin
		xc4=xf
		yc4=yf
		zc4=xf*0.0
	endif else $
	if mesh eq 0 then begin
		dtoc, xf,yf,xf*0.0, xc4,yc4,zc4
		projs, xc4,yc4,zc4
	endif else $
	if mesh eq 1 then begin
		ttoc, xf,yf,xf*0.0, xc4,yc4,zc4
	endif else $
	if mesh ge 2 and meshl ge 0 then begin
		transf, mparam, mesho, $
			nx1,ny1,xs1,ys1,dx1,dy1,xe1,ye1
		xc4=[xs1,xe1,xe1,xs1]
		yc4=[ys1,ys1,ye1,ye1]
		dtoc, xc4, yc4, xc4*0.0, xc4,yc4,zc4
		projs, xc4,yc4,zc4
	endif
	xc40 = xc4
	yc40 = yc4
	zc40 = zc4

;  xc4--zc4 are in C-coordinates

	xwsize=600
	ywsize=700

	xfl = fix(0.075*xwsize)
	yfl = fix(0.075*xwsize)
	xfr= fix(0.925*xwsize)
	yfr= fix(0.925*xwsize)
	xsize = xfr-xfl
	yfsize = yfr-yfl
	ysize = ywsize
	xfc=0.5*(xfl+xfr)
	yfc=0.5*(yfl+yfr)

	gxc = 0.25*total(xc4(0:3))
	gyc = 0.25*total(yc4(0:3))

	gfact = min([xsize / (abs(xc4(1)-xc4(0)) $
                      + abs(xc4(3)-xc4(0)) ), $
              yfsize / (abs(yc4(1)-yc4(0))  $
                      + abs(yc4(3)-yc4(0)) ) ] )
	scale1=gfact/devcm

	xclipl = min(gfact*(xc4(0:3)-gxc) +xfc, max=xclipr)
	yclipb = min(gfact*(yc4(0:3)-gyc) +yfc, max=yclipt)

; print,'gfact,gxc,gyc,xfc,yfc=',gfact,gxc,gyc,xfc,yfc
; print,'mesh,meshl=',mesh,meshl

;  drawable area
;  (xfl,yfl)=lower left corner
;  (xfr,yfr)=upper right corner
;  (xfc,yfc)=center of window

;  clipping window
;  (xclipl,yclipb)=lower left corner
;  (xclipr,yclipt)=upper right corner

	window, 0, title='IDL 0 : MAGPACK2 display', xsize=xwsize, ysize=ywsize

; lower left and upper right corners of contour plot in device coordinates
	mapposition = [xclipl, yclipb, xclipr, yclipt]

; character size
	cs=1.0   ;  0.75

	setcolor, bandwidth, w_start, rb_start, g_start, color8, $
			whitebg = whitebg

	if keyword_set(whitebg) then begin
		fgcolor = color8.black 
		bgcolor = color8.white
	endif else begin
		fgcolor = color8.white
		bgcolor = color8.black
	endelse
	erase, bgcolor

	if ( mparam.kindm lt 0 ) then pol2b, mparam, rdata
	kindm = mparam.kindm
	mklist, mparam, nxy,nz,kxlist,kylist,kzlist,strx,stry,strz

	setlevel,mparam,rdata, $
		oaodefault=oaodefault, $
		title1,intlevels,dummylevels,bplevels,bmlevels,dplevels,dmlevels, $
		jplevels,jmlevels,fplevels,fmlevels, $
		bunit,dunit,iunit,junit,funit

	ki = mparam.status.i
	kz = kloc(mparam,/z)
top:
	clip=1
	region=0
	frame=0
	globe=0
	dots=0
	nh=0
	kh=-1
	nc=0
	kc=-1
	kx=-1
	ky=-1
	kx2=-1
	ky2=-1
	nv=0
	chrh = ' '
	chrc = ' '
	arrow=1
	title2 = ''
	title3 = ''

;	set plotting scale
	plot, [xclipl,xclipr],[yclipb,yclipt],/nodata, /noerase, $
		xstyle=5, ystyle=5, $
		position=[xclipl,yclipb, xclipr,yclipt], /device

;  rotation and view angles

	print,format='("enter rotation and view angles (rotr,rotz,rotx)",$)
	read, rotdeg, zdeg,xdeg
	print,format='("enter scale factor(=0.0 for reset), offset (dx,dy)",$)'
	read, scale,dxc1,dyc1

	if rotdeg ne 0 or zdeg ne 0 or xdeg ne 0 or scale ne 0.0 then clip=0
	setmat

	rotpt, xc40,yc40,zc40, xc4,yc4,zc4
	gxc = 0.25*total(xc4(0:3)) -dxc1
	gyc = 0.25*total(yc4(0:3)) -dyc1

	if scale gt 0.0 then begin
		gfact=devcm*scale
		scale1=scale
	endif else begin
		gfact = min([xsize / (abs(xc4(1)-xc4(0)) $
			+ abs(xc4(3)-xc4(0)) ), $
			yfsize / (abs(yc4(1)-yc4(0)) $
			+ abs(yc4(3)-yc4(0)) ) ] )
		if scale lt 0.0 then gfact=gfact*abs(scale)
		scale1=gfact/devcm
	endelse

; print,'gfact,gxc,gyc,xfc,yfc=',gfact,gxc,gyc,xfc,yfc
; print,'mesh,meshl=',mesh,meshl

; selection of items to be displayed
	
; location of observed area

	print,format="('obs.area    : 0=skip, 1=show',$)"
	repeat read, region until region eq 0 or region eq 1
	if region ne 0 then begin
		erase
		dispr3, mparam,rdata,ki,xc4,yc4, fgcolor,color8.yellow, /device
		return
      endif

	if nz gt 0 then begin

; halftone map

		print,format='("halftone map: 0=skip",$)'
		for i=1, nz do print,format='(", ",i1,"=",a5,$)', i,strz(i-1)
		repeat read,nh until nh ge 0 and nh le nz
		if nh ne 0 then begin
			kh = kzlist(nh-1)
			chrh = strmid( strz(nh-1), 0,1)
		endif

; contour map

		print,format='("contour map : 0=skip",$)'
		for i=1, nz do print,format='(", ",i1,"=",a5,$)', i,strz(i-1)
		repeat read,nc until nc ge 0 and nc le nz
		if nc ne 0 then begin
			kc = kzlist(nc-1)
			chrc = strmid( strz(nc-1), 0,1)
		endif
	endif

; vector map

	if nxy gt 0 then begin
		if nxy gt 1 then nxy1 = nxy+1 else nxy1=nxy
		if kloc(mparam,/px) ge 0 and mparam.alpha ne 0.0 then $
			print,'(BXP,BYP) is for alpha=',mparam.alpha
		print,format='("vector map  : 0=skip",$)'
		for i=1, nxy do print, $
		format='(", ",i1,"=(",a5,",",a5,")",$)', i,strx(i-1),stry(i-1)
		repeat read,nv until nv ge -nxy1 and nv le nxy1
		if nv lt 0 then begin
			nv = -nv
			arrow = 0
		endif
		if nv ne 0 and nv le nxy then begin
			kx = kxlist(nv-1)
			ky = kylist(nv-1)
		endif
		if nxy gt 1 and nv eq nxy+1 then begin
			kx = kxlist(1)
			ky = kylist(1)
			kx2 = kxlist(0)
			ky2 = kylist(0)
		endif
	endif	

; intensity in dots

	if ki ge 0 then begin
		print,format="('intensity in dots: 0=skip, 1=plot',$)"
		repeat read, dots until dots eq 0 or dots eq 1
	endif

; longitude/latitude circles

	if meshl ge 0 then begin
		print,format="('longitude/latitude circles: 0=skip, 1=plot',$)"
		repeat read, globe until globe eq 0 or globe eq 1
	endif

; border of observed region
	print,format="('border of observed region: 0=skip, 1=plot',$)"
	repeat read, frame until frame eq 0 or frame eq 1

	erase, bgcolor

;  scale indicator (longer than 3mm, shorter than 30mm)

	if meshl ge 0 then begin
		ixp=[ xfr-2.0*devcm, 0.0 ]
		iyp=[ yfr+0.5*devcm, 0.0 ]
		gauge=gfact/devcm*(rsp*1.0e9)/(rsun*ulen)

		if gauge lt 0.3 then begin
			gauge=10.0*gauge
			xyouts, ixp(0),iyp(0),'10**5 KM', color=fgcolor, /device
		endif else $
		if gauge gt 3.0 then begin
			gauge=gauge/10.0
			xyouts, ixp(0),iyp(0), '10**3 KM', color=fgcolor, /device
		endif else begin
			xyouts, ixp(0),iyp(0),'10**4 KM', color=fgcolor, /device
		endelse

		ixp(0)=ixp(0)-fix(0.2*devcm)
		iyp(0)=iyp(0)
		ixp(1)=ixp(0)-fix(gauge*devcm)
		iyp(1)=iyp(0)
		plots, ixp,iyp, thick=2, color=fgcolor, /device
		ixp(1)=ixp(0)
		iyp(1)=iyp(0)+fix(0.1*devcm)
		iyp(0)=iyp(0)-fix(0.1*devcm)
		plots, ixp,iyp, thick=2, color=fgcolor, /device
		ixp(0)=ixp(0)-fix(gauge*devcm)
		ixp(1)=ixp(0)
		plots, ixp,iyp, thick=2, color=fgcolor, /device
	endif

; halftone map

	if ( kh ge 0 ) then begin
		
		if ( chrh eq 'I' ) then begin
			intmax=max(rdata(*,*,kh))
			buf = loglevel(rdata(*,*,kh),0.3*intmax,intmax)
			disphalftone, mparam, buf, $
				color=w_start, nop=w_start, bwidth=0.7*bandwidth
		endif else $
		if chrh eq 'V' then begin
			vmax = max(abs(rdata(*,*,kh)))
			vmin = 0.01*vmax
			buf = loglevel(-rdata(*,*,kh),vmin,vmax) +1
			disphalftone, mparam, buf, $
				color=rb_start, nop=rb_start+bandwidth, bwidth=bandwidth
		endif else begin
			bmax = max(abs(rdata(*,*,kh)))
			bmin = 0.01*bmax
			buf = loglevel(rdata(*,*,kh),bmin,bmax) +1
			disphalftone, mparam, buf, $
				color=rb_start, nop=rb_start+bandwidth, bwidth=bandwidth
		endelse

		title3 = strmid('HALFTONE:'+strz(nh-1)+'                    ',0,20)
	endif 

;  intensity in dots

	if dots eq 1 and ki ge 0 then $
		dispdot, mparam, rdata(*,*,ki), $
			color=color8.yellow, /device

;  solar limb and longitude/latitude circles

	if globe eq 1 then begin
		lonlat=1
	endif else begin
		lonlat=0
	endelse
	limb=meshl ne 0
	
	sphere, xc4,yc4, dxy0,lonlat,limb,clip, $
		color=color8.yellow, /device

;  if limb=T, solar limb is drawn
;  if clip=T, long./lat. circles are only drawn inside the
;  observed region

; contour map

	nlevel =5
	if ( kc ge 0 ) then begin
		case chrc of
		'B':	begin
			plevels = bplevels & mlevels = bmlevels
			unit=bunit
			end
		'V':  begin
			plevels = dplevels & mlevels = dmlevels
			unit=dunit
			end
		'I':	begin
			plevels = intlevels & mlevels = dummylevels
			unit=iunit
			end
		'J':	begin
			plevels = jplevels & mlevels = jmlevels
			unit=junit
			end
		'F':	begin
			plevels = fplevels & mlevels = fmlevels
			unit=funit
			end
		else: begin
			plevels = fplevels & mlevels = fmlevels
			unit=funit
			end
		endcase
		if keyword_set(autolevel) and chrc ne 'I' then $
			lvl125,rdata(*,*,kc),nlevel=nlevel, $
			plevel=plevels,mlevel=mlevels

		sz = size(plevels)
		nlevels = fix(sz(1))
		plinestyle = intarr(nlevels)	; 0=solid
		mlinestyle = plinestyle +0		; 2=dashed (1=dotted -- no good)
		pcolors = intarr(nlevels)
		mcolors = pcolors
		plabels= intarr(nlevels)
		mlabels=plabels
		
		case chrc of
		'I':	begin
			pcolors(*) = color8.yellow
			mcolors(*) = color8.yellow ; dummy
			end
		'V':	begin
			pcolors(*) = color8.blue
			mcolors(*) = color8.red
			end
		else:	begin
			pcolors(*) = color8.red
			mcolors(*) = color8.blue
			end
		endcase

		title2 = strmid( $
			'CONTOURS:'+strz(nc-1) +'                    ',0,20) + $
  			'LEVELS='
		title2 = title2 + strlevel( plevels) +' '+ unit

		x = xs +dx*indgen(nx)
		y = ys +dy*indgen(ny)
		contour, rdata(*,*,kc), x,y, $
			path_info=cntrinfo, path_xy=cntrdata, /path_data_coords, $
			levels=[mlevels,plevels], $
			c_linestyle = [mlinestyle,plinestyle], $
			c_color = [mcolors, pcolors], $
			c_labels = [mlabels, plabels], $
			xstyle=5, ystyle=5, $
			color = fgcolor, $
			charsize=cs, $
			/noerase, $
			position = mapposition, /device

;	reset plotting scale
	plot, [xclipl,xclipr],[yclipb,yclipt],/nodata, /noerase, $
		xstyle=5, ystyle=5, $
		position=[xclipl,yclipb, xclipr,yclipt], /device

		cntr3, cntrinfo,cntrdata, $
			/device, $
			color= [mcolors, pcolors],style= [mlinestyle,plinestyle]

	endif

; vector map

	if ( kx ge 0 and ky ge 0 ) then begin

		kindmb = kindm mod 10
		if ( arrow gt 0 and kindmb gt 2 ) then arrow = 1 else arrow = 0
		if ( kx ne kloc(mparam,/px) and kindmb eq 6 ) then arrow =0
;		kindmb=6: data are pseudo B and Btp, and pseudo B is selected

		dispbar, mparam,rdata, kx,ky,kz, $
			btmax=btmax, btmin=btmin, $
			/device, $
			color=fgcolor, loc=arrowloc

		if kx2 ge 0 and ky2 ge 0 then  $
			dispbar, mparam, rdata, kx2,ky2,kz, $
			/device, $
			btmax=btmax, btmin=btmin, $
			color=color8.green

		title3 = title3 + $
			strmid( 'VECTORS :' + strx(nv-1) + ', ' + stry(nv-1) + $
			'          ', 0, 20 ) + $
			'MAX,MIN=' + string(btmax, btmin, format='(2f6.0)')
	endif

;  frame

;  arguments of pline2 should be D-coordinates
 	if frame eq 1 then begin
		for n=0,3 do begin
			alen=sqrt( (xf(n+1)-xf(n))^2 + (yf(n+1)-yf(n))^2 )
			np=fix(alen/dxy)
			xp=xf(n)+(xf(n+1)-xf(n))*indgen(np)/float(np-1)
			yp=yf(n)+(yf(n+1)-yf(n))*indgen(np)/float(np-1)
			pline2, xp,yp, color=fgcolor, thick=1, /device
		endfor
	endif

;  titles

;	font = 0	; hardware font
;	font = 3	; default font
	title4 = string(rotdeg,zdeg,xdeg,scale1, $
	format="('ROTR=',f6.1,4x,'(ROTZ,ROTX)=',2f6.1,4x,'SCALE FACTOR=',f8.4)")

	xyouts, xfl, ysize-1*vmi, /device, title1, color=fgcolor, charsize=cs
	xyouts, xfl, ysize-2*vmi, /device, title2, color=fgcolor, charsize=cs
	xyouts, xfl, ysize-3*vmi, /device, title3, color=fgcolor, charsize=cs
	xyouts, xfl, ysize-4*vmi, /device, title4, color=fgcolor, charsize=cs

	yn = ' '

	if not keyword_set(noask) then begin
		print, format='("repeat? (y/n) > ",$)
		repeat read, yn until $
			 (yn eq 'y' or yn eq 'n' or yn eq 'q') 

		if yn eq 'q' then stop
		if ( yn eq 'y' or yn eq 'Y' ) then goto, top
	endif

	return
end

;  .run disp executes the following.
dispm3, mparam,rdata
end
