;+
; NAME :	dispr3
; PURPOSE :
;	plot a sphere and indicate the observed region on it
;	arbitrary view angles and scaling/positioning
;
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	dispr3, mparam, rdata, ki, xc4,yc4, color1, color2
;		[,/device] [,/normal]
;
; INPUTS :
;     mparam : magnetogram parameters
;     rdata  : magnetogram data
;	ki : rdata(*,*,ki) holds intensity map
;	xc4,yc4 : coordinates of four corners of the map
;	color1 : color index for characters
;	color2 : color index for the globe
;
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	set /device when the output is to the X-window
;	set /normal when the output is to the postscript printer
; OUTPUTS :
;	plot a sphere and a rectangle indicating the observed region
; COMMON BLOCKS : 	common blkr is referenced
; SIDE EFFECTS : 	none
; RESTRICTIONS :	X-window environment is assumed
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/11/02
;-
;------------------------------------------------------------------------
;
pro dispr3, mparam, rdata, ki, xc4,yc4, color1,color2, $
	device=device, normal=normal

	common blkr, coord_r
@gparam

	nx=mparam.nx
	ny=mparam.ny
	xs=mparam.xs
	ys=mparam.ys
	dx=mparam.dx
	dy=mparam.dy
	rsp=mparam.rsp
	pdeg = mparam.pdeg
	b0deg = mparam.b0deg
	cl0deg = mparam.cl0deg
	title = mparam.title
	xxsp = coord_r.xxsp

;  data description

	cs = 0.75
	vmi = vmicm*devcm
	ixloc=xfl
	iyloc=ysize-2.0*vmi
	xyouts, ixloc,iyloc, color=color1, charsize=cs, title, $
		device=device, normal=normal

	iyloc=ysize-3.0*vmi
	xyouts, ixloc,iyloc, color=color1, charsize=cs, $
		device=device, normal=normal, $
		string(pdeg,b0deg,cl0deg,rsp, $
		format="('P=',f5.1,4x,'B0=',f4.1,4x,'CL0=',f6.1,4x,'RSP=',f6.2)")

	iyloc=ysize-4.0*vmi
	xyouts, ixloc,iyloc, color=color1, charsize=cs, $
		device=device, normal=normal, $
		string(nx,ny,xs,ys,dx,dy, $
		format="('(NX,NY)=',i3,2x,i3,4x,'(XS,YS)=',f6.2,2x,f6.2,4x,'(DX,DY)=',f6.2,2x,f6.2)")

;  save and change scaling parameters

	fsav=gfact
	gxcsav=gxc
	gycsav=gyc
	xfcsav=xfc
	yfcsav=yfc
	xfrsav=xfr
	yfrsav=yfr

	gxc=0.0
	gyc=0.0
	xfc=0.5*xsize
	yfc=0.5*xsize
	xfr=xsize
	yfr=xsize

;  markers

	rad=0.75*(0.5*xsize)
	gfact=rad/rsp
	setar, mparam
	d =0.1*devcm/gfact
	d1=0.3*devcm/gfact
	d2=0.6*devcm/gfact

	xpn=replicate(xxsp(0),3)
	xps=replicate(xxsp(0),3)
	xpe=replicate(xxsp(0),3)
	xpw=replicate(xxsp(0),3)
	ypn=replicate(xxsp(1),3)
	yps=replicate(xxsp(1),3)
	ype=replicate(xxsp(1),3)
	ypw=replicate(xxsp(1),3)
	zpn=replicate(xxsp(2),3)
	zps=replicate(xxsp(2),3)
	zpe=replicate(xxsp(2),3)
	zpw=replicate(xxsp(2),3)
;  N
	ypn(0)=xxsp(1)+rsp+d2
	ypn(1)=ypn(0)-d1
	xpn(2)=xxsp(0)-d
	ypn(2)=xxsp(1)+rsp+d2+d
	pline5, xpn(0:1),ypn(0:1),zpn(0:1),xp1,yp1, color=color1, $
		device=device, normal=normal
	pline5, xpn(2),ypn(2),zpn(2),xp1,yp1, color=color1, $
		device=device, normal=normal
	xyouts,xp1,yp1,'N', $
		device=device, normal=normal

;  S
	yps(0)=xxsp(1)-rsp-d2
	yps(1)=yps(0)+d1
	xps(2)=xxsp(0)-d
	yps(2)=xxsp(1)-rsp-d2-4.0*d
	pline5, xps(0:1),yps(0:1),zps(0:1),xp1,yp1, color=color1, $
		device=device, normal=normal
	pline5, xps(2),yps(2),zps(2),xp1,yp1, color=color1, $
		device=device, normal=normal
	xyouts, xp1,yp1,'S', $
		device=device, normal=normal
;  E
	xpe(0)=xxsp(0)-rsp-d2
	xpe(1)=xpe(0)+d1
	xpe(2)=xxsp(0)-rsp-d2-3.0*d
	ype(2)=xxsp(1)-d
	pline5, xpe(0:1),ype(0:1),zpe(0:1),xp1,yp1, color=color1, $
		device=device, normal=normal
	pline5, xpe(2),ype(2),zpe(2),xp1,yp1, color=color1, $
		device=device, normal=normal
	xyouts, xp1,yp1,'E', $
		device=device, normal=normal
;  W
	xpw(0)=xxsp(0)+rsp+d2
	xpw(1)=xpw(0)-d1
	xpw(2)=xxsp(0)+rsp+d2+d
	ypw(2)=xxsp(1)-d
	pline5, xpw(0:1),ypw(0:1),zpw(0:1),xp1,yp1, color=color1, $
		device=device, normal=normal
	pline5, xpw(2),ypw(2),zpw(2),xp1,yp1, color=color1, $
		device=device, normal=normal
	xyouts, xp1,yp1,'W', $
		device=device, normal=normal

;  sphere

	lonlat=1
	globe=1
	clip=0
	dl=0.02*rsp
	sphere, xc4,yc4, dl, lonlat,globe,clip, color=color2, $
		device=device, normal=normal

;  observed area

	xf=[xs,xs+dx*(nx-1), xs+dx*(nx-1), xs, xs]
	yf=[ys,ys, ys+dy*(ny-1), ys+dy*(ny-1), ys]
;  arguments of pline2 should be D-coordinates

	for n=0,3 do begin
		alen=sqrt( (xf(n+1)-xf(n))^2 + (yf(n+1)-yf(n))^2 )
		np=fix(alen/(dl*(dx/dx0)))
		xp=xf(n)+(xf(n+1)-xf(n))*indgen(np)/float(np-1)
		yp=yf(n)+(yf(n+1)-yf(n))*indgen(np)/float(np-1)
		pline2, xp,yp, color=color1, $
			device=device, normal=normal
	endfor

;  intensity map

	if ki ge 0 then begin
		aimin=0.7
		for j=0,ny-1 do $
		for i=0,nx-1 do begin
		if rdata(i,j,ki) gt aimin then goto, loopend
			xp1=xs+dx*i
			yp1=ys+dy*j
			pline2, [xp1,xp1], [yp1,yp1], color=color2, $
				device=device, normal=normal
;  arguments of pline2 should be D-coordinates
loopend:
		endfor
	endif

;  restore normal scaling

	gfact=fsav
	gxc=gxcsav
	gyc=gycsav
	xfc=xfcsav
	yfc=yfcsav
	xfr=xfrsav
	yfr=yfrsav

	return
	end
