;+
; NAME :	divb1
; PURPOSE :
;	calculate Bn * div B from (BX,BY) (planar geometry)
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	divb1, mparam, rdata [,divb] [,divba]
;		[,/faceon] [,smooth=smooth]
;		[,method=method] [,nosave=nosave] [,dbldz=dbldz]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	/faceon : if set, assume the region is at disk center
;	smooth : if /smooth is set, apply 3x3 smoothing filter to (BX,BY)
;		     for smooth >1, 2*smooth+1 is the size of the smoothing filter
;	nosave: if set, grad is not stored in rdata
;	method : 1=usual differentiation, 2=parity insensitive differentiation
;	dbldz : value of d(BL)/dz (z: along line of sight)
;	this procedure returns Bn*divB, which should be 0 if dbldz is correct
; OUTPUTS :
;	divb : Bn * div B
;	divba : same as divb but for -BT
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	mparam will be modified
; RESTRICTIONS :	none
; PROCEDURE :
;	use spline fitting to obtain derivatives of BX and BY
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/10/16
;-
;------------------------------------------------------------------------
;

pro divb1, mparam, rdata, divb, divba, faceon=faceon, smooth=smooth, $
	method=method, nosave=nosave, dbldz=dbldz

	if not keyword_set(dbldz) then dbldz=0.0
;	dbldz = d(BL)/dz (z: along line of sight)
;	this procedure returns Bn*divB, which should be 0 if dbldz is correct

	common blkt, coord_t

	kindm = mparam.kindm

	if kindm lt 0 then pol2b, mparam, rdata

	kindmb = kindm mod 10

	if kindmb eq 1 or kindmb eq 5 then begin
		print, '(proc.divb1) error: magnetogram is not vector magnetogram'
		return
	endif

	if kindmb eq 2 or kindmb eq 6 then begin
		vector1, mparam, rdata, faceon=faceon
		kindmb = mparam.kindm mod 10
	endif

	nx = mparam.nx
	ny = mparam.ny
	dx = mparam.dx
	dy = mparam.dy

	pi = 3.141592

	kx = kloc( mparam, /x)
	ky = kloc( mparam, /y)
	kz = kloc( mparam, /z)
	if kx eq -1 or ky eq -1 or kz eq -1 then begin
		print,'(proc.curlb1) error:kx,ky,kz=',kx,ky,kz
		return
	endif
	if keyword_set(faceon) then begin
		ainv= [ [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0] ] 
	endif else begin
		setdu, mparam
		settu, mparam
		amat = coord_t.atmat
		ainv = coord_t.atinv
	endelse

	rx = amat(0,2)
	ry = amat(1,2)
	rz = amat(2,2)
	rx2 = rx*rx
	ry2 = ry*ry
	rz2 = rz*rz
	rz21 = 1-rz2

	bx = rdata(*,*,kx)
	by = rdata(*,*,ky)
	bz = rdata(*,*,kz)
	if keyword_set(smooth) then begin
		bx = smooth(bx, 2*smooth+1)
		by = smooth(by, 2*smooth+1)
		bz = smooth(bz, 2*smooth+1)
	endif

if not keyword_set(method) then method=1
	
if method eq 1 then begin
	dBxdx = surface3(bx,dxydata=[dx,dy],ndiv=1,diffx=1, $
		solution=solution,/zeroderiv)
	dBxdy = surface3(bx,dxydata=[dx,dy],ndiv=1,diffy=1, $
		known=solution,/zeroderiv)
	dBydx = surface3(by,dxydata=[dx,dy],ndiv=1,diffx=1, $
		solution=solution,/zeroderiv)
	dBydy = surface3(by,dxydata=[dx,dy],ndiv=1,diffy=1, $
		known=solution,/zeroderiv)

endif else begin
	bxy=bx*by
	bxy2=bx*bx-by*by
	bt2 = (bx*bx + by*by) > 1.0e-4

	dbxydx = surface3(bxy,dxydata=[dx,dy],ndiv=1,diffx=1, $
		solution=solution,/zeroderiv)
	dbxydy = surface3(bxy,dxydata=[dx,dy],ndiv=1,diffy=1, $
		known=solution,/zeroderiv)
	dbxy2dx =surface3(bxy2,dxydata=[dx,dy],ndiv=1,diffx=1, $
		solution=solution,/zeroderiv)
	dbxy2dy =surface3(bxy2,dxydata=[dx,dy],ndiv=1,diffy=1, $
		known=solution,/zeroderiv)

	dBxdx = ( 0.5*bx*dbxy2dx + by*dbxydx)/bt2
	dBydy = (-0.5*by*dbxy2dy + bx*dbxydy)/bt2
	dBydx = (-0.5*by*dbxy2dx + bx*dbxydx)/bt2
	dBxdy = ( 0.5*bx*dbxy2dy + by*dbxydy)/bt2
endelse
	dBzdx = surface3(bz,dxydata=[dx,dy],ndiv=1,diffx=1, $
		solution=solution,/zeroderiv)
	dBzdy = surface3(bz,dxydata=[dx,dy],ndiv=1,diffy=1, $
		known=solution,/zeroderiv)

	nb  =  rx*bx +ry*by +rz*bz
	nba = -rx*bx -ry*by +rz*bz
	nxb = rx*by - ry*bx
	
	divb =  rz2*nb*(dBxdx + dBydy) $
		+ rz*dBzdx*( nb*rx - ry*nxb ) $
		+ rz*dBzdy*( nb*ry + rx*nxb ) $
		+ rz2*nxb*(dBydx - dBxdy) + dbldz*nb
;	 	= Bn * div B

;	divb for other sign of Bt
	divba = -rz2*nba*(dBxdx + dBydy) $
		+ rz*dBzdx*( nba*rx + ry*nxb ) $
		+ rz*dBzdy*( nba*ry - rx*nxb ) $
		+ rz2*nxb*(dBydx - dBxdy) + dbldz*nba

	if not keyword_set(nosave) then begin
		k = kloc( mparam, /bdiv)
		if k eq -1 then begin
			k = newk( mparam, rdata )
			mparam.nframe = mparam.nframe +1
		endif	

		rdata(*,*,k) = divb

		kindm3 = (mparam.kindm/100 ) * 100
		if keyword_set(faceon) then begin
			mparam.status.divb = k
			mparam.kindm = kindm3 + 10L + kindmb
		endif else begin
			mparam.status.divb = k
			mparam.kindm = kindm3 + 20L + kindmb
		endelse
	endif

	return
end

