;+
; NAME :	eqtime
; PURPOSE :
;	calculate the equation of time
;	( = hour angle of the true sun
;	  - hour angle of the local mean sun)
; CATEGORY :
; 	practical astronomy
; CALLING SEQUENCE :
;	eqt = eqtime(date, time, tsys=tsys)
; INPUTS :
;	date given as 'yy/mm/dd' or 'yyyymmdd'
;       time = 'hh:mm:ss' or
;              a floating point number expressing the time in hour
; OPTIONAL INPUT PARAMETERS : 
;	tsys : timezone. If not specified, 'UT' is assumed.
; KEYWORD PARAMETERS :
; OUTPUTS :
;	eqt = equation of time in unit of hour
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
; RESTRICTIONS :
;	accuracy is about 2 seconds
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	based on Almanac for Computors (U.S.Naval Observatory)
;	originally coded by K.Shibasaki 10/28/83 as koyomi/zs for OAO Melcom 70
;	T.Sakurai, 1993
;	TS 97/12/22 parameters updated
;	TS 99/01/04 parameters updated, format of year can be 'yyyymmdd'
;	TS 99/12/04 parameters updated
;	made separated from stransit.pro
;	TS 00/11/14 parameters updated
;	TS 01/12/22 parameters updated
;-
;====================================================================

function eqtime, date, time, tsys=tsys

	rd=3.14159265358979d0/180.0d0

	if n_params() eq 0 then begin
	  date = ' '
	  print, 'enter date (yy/mm/dd or yyyymmdd)'
	  read, date
	  tsys='UT'
	  print, 'enter time zone (UT, JST, etc.)'
	  read, tsys
	  time = ' '
	  print, 'enter time (hh:mm:ss)'
	  read, time
	endif

	if n_elements(tsys) eq 0 then tsys = 'ut'

	if (strmid(date,2,1) eq '/') then begin
		year	= fix(strmid(date,0,2))
		if (year le 50) then year=year+2000 else year=year+1900
		month	= fix(strmid(date,3,2))
		day	= fix(strmid(date,6,2))
	endif else begin
		year	= fix(strmid(date,0,4))
		month	= fix(strmid(date,4,2))
		day	= fix(strmid(date,6,2))
	endelse

	it3=275*month/9
	idoy1=(month+9)/12*(1+((year mod 4)+2)/3)
	doy =float(it3-idoy1+day-30) + uthour(tsys, time)/24.0

	if (year eq 2002) then begin
		doy=doy+365.
		year = year-1
	endif

	case (year) of
	1982: begin
		x1=3.307
		x2=9.332
		x3=37.329
		x4=0.91747
		end
	1983: begin
		x1=3.562
		x2=9.094
		x3=36.377
		x4=0.91747
		end
	1984: begin
		x1=3.819
		x2=8.855
		x3=35.421
		x4=0.91747
		end
	1985: begin
		x1=3.508
		x2=9.598
		x3=38.394
		x4=0.91747
		end
	1986: begin
		x1=3.761
		x2=9.363
		x3=37.454
		x4=0.91747
		end
	1987: begin
		x1=4.020
		x2=9.122
		x3=36.486
		x4=0.91747
		end
	1988: begin
		x1=4.274
		x2=8.887
		x3=35.546
		x4=0.91747
		end
	1989: begin
		x1=3.539
		x2=9.639
		x3=38.557
		x4=0.91747
		end
	1990: begin
		x1=3.798
		x2=9.397
		x3=37.589
		x4=0.91747
		end
	1991: begin
		x1=4.052
		x2=9.161
		x3=36.645
		x4=0.91748
		end
	1992: begin
		x2=8.92889
		x3=35.7187
		x4=0.917505
		end
	1993: begin
		x1=3.49347
		x2=9.67955
		x3=38.71657
		x4=0.917486
		end
	1994: begin
		x1=3.741556
		x2=9.447933
		x3=37.801375
		x4=0.917473
		end
	1995: begin
		x1=4.083159
		x2=9.209114
		x3=36.831114
		x4=0.917465
		end
	1996: begin
		x1=4.083159
		x2=9.209114
		x3=36.831114
		x4=0.917465
		end
	1997: begin
		x1=3.559273
		x2=9.693544
		x3=38.784848
		x4=0.917486
		end
	1998: begin
		x1=3.973954
		x2=9.462792
		x3=37.848180
		x4=0.917501
		end
	1999: begin
		x1=4.212082
		x2=9.235435
		x3=36.953773
		x4=0.917515
		end
	2000: begin
		x1=4.392457
		x2=8.990560
		x3=35.974052
		x4=0.917499
		end
	2001: begin
		x1=3.638365
		x2=9.729686
		x3=38.924743
		x4=0.917448
		end
	2002: begin
		x1=3.872599
		x2=9.486818
		x3=37.964154
		x4=0.917455
		end
	else: begin
		print, '(func.eqtime) no ephemeris parameters available',year
		return, 0
		end
	endcase

      th1=1.915*sin((0.9856*doy-x1)*rd)
      th2=0.020*sin((1.9712*doy-2.0*x1)*rd)
      if(year le 1984) then $
        th3=0. $
      else $
        th3=.014*cos((.9856*doy-x1)*rd)

      th =(x2+0.98561*doy+th1+th2+th3)*rd
      atth=atan(tan(th)/x4)/rd
	while(1) do begin
		sa1=abs(atth-th/rd)
		sa2=abs(atth+180.-th/rd)
		if(sa1 lt sa2) then goto, escape
		atth=atth+180.
	endwhile
escape:

      eqt=(x3+3.94244*doy-4.*atth)/60.

	if n_params() eq 0 then begin
	  if eqt ge 0 then sig='+' else sig='-'
	  min=fix(60*abs(eqt))
	  sec=fix(3600*abs(eqt)-60*min)
	  tsysu=' '+strtrim(strupcase(tsys),2)
	  print, '(func.eqtime) date=',date,'  time=',time, tsysu
	  print, 'equation of time = ', $
		string(format='(a1,i3.2," min ",i2.2," sec")', $
		sig,min,sec)
	endif
	return, eqt
end
