;+
; NAME :	lff1
; PURPOSE :
;	calculate linear (constant-alpha) force-free field 
;	based on the longitudinal field (BL) in rdata (planar geometry)
; CATEGORY :
;	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	lff1, mparam, rdata [,bxp, byp] 
;		[,alpha=alpha] [,maxalpha=maxalpha] [,/getmaxalpha]
;		[,/faceon] [,/tight] [,/nosave]
;		[,h=h] [,bzp=bzp]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	alpha: force-free parameter
;	maxalpha : maximum allowable value of alpha
;	/getmaxalpha : if set, the value of maxalpha is given with no further 
;				computation of (bpx,bpy)
;	/faceon : if set, assume that given magnetogram is for BN, regardless
;				of the actual position on the solar disk
;	/tight : if set, FFT is made on the area with the same size as the 
;	         input magnetogram. if not set, an area twice as large as
;	         the input magnetogram is used for FFT
;	/nosave : if set, bxp and byp are not stored in rdata
;	h : height above the surface (not along the line of sight,
;	    but perpendicular to the boundary plane)
;	bzp : line-of-sight component of current-free field
;	      if h=0, this is identical to the input BL
; OUTPUTS :
;	bxp, byp : computed force-free transverse fields
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	mparam will be modified
; RESTRICTIONS :	none
; PROCEDURE :
;	use FFT to calculate linear force-free fields
; MODIFICATION HISTORY :
;	T.Sakurai, 2000 July
;	TS 2000 Oct. /nosave added
;	TS 2001/01/31: h and bzp parameters added
;-
;----------------------------------------------------------------------
;

pro setbtc, kwx,kwy, alx,aly, cx,cy, amat, alpha=alpha, $
	h=h,cfactor=cf
;	calculate complex coefficients (cx,cy) to be multiplied
;	on the Fourier transform of BL
;	kwx,kwy = integer indices
;	alx = x-width /2*pi
;	amat = matrix of coordinate transform between data and tangential
;	coordinates
;	alpha = force-free parameter
;	h = height above the boundary plane
;	cfactor = exp(i*h*ckz)
;	
	ci = complex (0.0, 1.0)
	akwx= kwx/alx
	akwy= kwy/aly
	akx=  akwx*amat(0,0)
	aky=  akwx*amat(0,1) + akwy*amat(1,1)

	loc = where(akx*akx + aky*aky - alpha*alpha lt 0.0 $
			and abs(kwx)+abs(kwy) ne 0, count)
	if count gt 0 then begin
		print,'(proc.setbtc) alpha exceeds max value'
	endif
	akz=  sqrt( (akx*akx + aky*aky - alpha*alpha) > 0)

	k0 = where(kwx eq 0 and kwy eq 0, count0)
	if count0 gt 0 then begin
		akz(k0) = 1
;		this is an artificial value, only to avoid zero divition
	endif

	ckz=  ci*akz - akwx*amat(0,2) - akwy*amat(1,2)
	ckxy= alpha*ci* (akwx*amat(1,2) - akwy*amat(0,2)) + alpha*alpha*amat(2,2)

	cx= ( ( akwx + ckz*amat(0,2) )*ci*akz - $
			alpha* (ci*akwy*amat(2,2) + alpha*amat(0,2) ) ) $
		/( ckz*amat(2,2)*ci*akz - ckxy )

	cy= ( (  akwy + ckz*amat(1,2) )*ci*akz + $
			alpha* (ci*akwx*amat(2,2) - alpha*amat(1,2) ) ) $
		/( ckz*amat(2,2)*ci*akz - ckxy )

	if keyword_set(h) then begin
		cf=exp(ci*ckz*h)
		cx=cx*cf
		cy=cy*cf
	endif else cf=1.0

	return
	end

;
pro lff1, mparam, rdata, bxp, byp, nosave=nosave, $
	alpha=alpha, maxalpha=maxalpha, $
	getmaxalpha=getmaxalpha, faceon = faceon, tight=tight, $
	h=h, bzp=bzp

	common blkt, coord_t

	if not keyword_set(alpha) then alpha=0.0
	if not keyword_set(h) then h=0.0

	nx = mparam.nx
	ny = mparam.ny
	dx = mparam.dx
	dy = mparam.dy

	pi = 3.141592

	if keyword_set(tight) then begin
		nxh = nx/2
		nyh = ny/2
		nx2 = nx
		ny2 = ny
	endif else begin
		nxh = nx
		nyh = ny
		nx2 = 2*nx
		ny2 = 2*ny
	endelse

	alx = dx*nx2/(2.0*pi)
	aly = dy*ny2/(2.0*pi)

	kz = kloc( mparam, /z)

        if kz eq -1 then begin
                print,'(proc.lff1) error:kz=',kz
                return
        endif

	if keyword_set(faceon) then $
		amat= [ [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0] ] $
	else begin
		setdu, mparam
		settu, mparam
		amat = coord_t.atmat
	endelse

	kwx = intarr(nx2,ny2)
	kwy = intarr(nx2,ny2)
	for j=0, ny2-1 do kwx(*,j) = [indgen(nxh),nxh,-nxh+1+indgen(nxh-1)]
	for i=0, nx2-1 do kwy(i,*) = [indgen(nyh),nyh,-nyh+1+indgen(nyh-1)]

	akwx= kwx/alx
	akwy= kwy/aly
	akx=  akwx*amat(0,0)
	aky=  akwx*amat(0,1) + akwy*amat(1,1)
	kne0 = where(kwx ne 0 or kwy ne 0)
	maxalpha = min(sqrt(akx(kne0)^2 + aky(kne0)^2))

	if keyword_set(getmaxalpha) then return

	if abs(alpha) gt maxalpha then begin
		print,'(proc.lff1) alpha gt alphamax ',alpha,maxalpha
		return
	endif

	rtemp = fltarr(nx2,ny2)
	rtemp( 0:(nx-1), 0:(ny-1) ) = rdata(*,*,kz)
	cftbz = fft( rtemp, -1)
	cftbz(0,0) = 0
; DC component is not allowed for fff
	cz=cftbz

	setbtc, kwx,kwy, alx,aly, cx, cy, amat, $
		alpha=alpha, h=h, cfactor=cf
	cz(*,*)=cf

	setbtc, kwx(*,nyh), kwy(*,nyh), alx,aly, cx1, cy1, amat, $
		alpha=alpha, h=h, cfactor=cf1
	setbtc, kwx(*,nyh),-kwy(*,nyh), alx,aly, cx2, cy2, amat, $
		alpha=alpha, h=h, cfactor=cf2
	cx(*, nyh) = 0.5*(cx1+cx2)
	cy(*, nyh) = 0.5*(cy1+cy2)
	cz(*, nyh) = 0.5*(cf1+cf2)

	setbtc,  kwx(nxh,*),kwy(nxh,*), alx,aly, cx1, cy1, amat, $
		alpha=alpha, h=h, cfactor=cf1
	setbtc, -kwx(nxh,*),kwy(nxh,*), alx,aly, cx2, cy2, amat, $
		alpha=alpha, h=h, cfactor=cf2
	cx(nxh, *) = 0.5*(cx1+cx2)
	cy(nxh, *) = 0.5*(cy1+cy2)
	cz(nxh, *) = 0.5*(cf1+cf2)

	setbtc,  kwx(nxh,nyh), kwy(nxh,nyh), alx,aly, cx1, cy1, amat, $
		alpha=alpha, h=h, cfactor=cf1
	setbtc, -kwx(nxh,nyh), kwy(nxh,nyh), alx,aly, cx2, cy2, amat, $
		alpha=alpha, h=h, cfactor=cf2
	setbtc,  kwx(nxh,nyh),-kwy(nxh,nyh), alx,aly, cx3, cy3, amat, $
		alpha=alpha, h=h, cfactor=cf3
	setbtc, -kwx(nxh,nyh),-kwy(nxh,nyh), alx,aly, cx4, cy4, amat, $
		alpha=alpha, h=h, cfactor=cf4
	cx(nxh, nyh) = 0.25*(cx1+cx2+cx3+cx4)
	cy(nxh, nyh) = 0.25*(cy1+cy2+cy3+cy4)
	cz(nxh, nyh) = 0.25*(cf1+cf2+cf3+cf4)

	cx(0,0) = 0.0
	cy(0,0) = 0.0
	cz(0,0) = 0.0

	cx = cx*cftbz
	cy = cy*cftbz
	cz = cz*cftbz

	cx = fft( cx, 1)
	bxp = float( cx( 0:(nx-1), 0:(ny-1) ) )

	cy = fft( cy, 1)
	byp = float( cy( 0:(nx-1), 0:(ny-1) ) )

	cz = fft( cz, 1)
	bzp = float( cz( 0:(nx-1), 0:(ny-1) ) )

      kindm2 = (mparam.kindm/10) *10

      kxp = kloc( mparam, /px )
      kyp = kloc( mparam, /py )
      if not keyword_set(nosave) then begin
            mparam.alpha = alpha
            if kxp eq -1 then begin
                  kxp = newk( mparam, rdata )
                  mparam.status.bxp = kxp
            endif
            if kyp eq -1 then begin
                  kyp = newk( mparam, rdata )
                  mparam.status.byp = kyp
            endif
            rdata(*,*,kxp) = bxp
            rdata(*,*,kyp) = byp
            mparam.status.blp = kz
            mparam.nframe = mparam.nframe +3
            mparam.kindm = kindm2 + 7L
      endif else begin
            if kxp ge 0 and kyp ge 0 then mparam.kindm = kindm + 7L $
            else mparam.kindm = kindm2 +3L
      endelse

	return
end
