;+
; NAME :	plotm
; PURPOSE :
;	plot magnetograms on a line-drawing device
;	for printer output from disp.pro
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	plotm, mparam, rdata [,/color]
;		[,/autolevel] [,/fixedlevel]
;		[,btmin=btmin] [,btmax=btmax] [,arrowloc=arrowloc]
;		[arrowloc=arrowloc]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	color : if set, plot output is made in color
;	/autolevel : contour levels are automatically adjusted to data (default)
;	/fixedlevel : contour levels are fixed as given by setlevel.pro
;	btmin : minimum value of BT to be plotted
;	btmax : value of BT corresponding to longest arrows
;	arrowloc : array of pointers where arrows will be plotted
;     papersize : horizontal width of the paper in cm
; OUTPUTS :
;	creates magnetogram plot
; COMMON BLOCKS : 	gparam is referenced
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1993
;	TS 1994/10: contour level setting was moved to @setlevel
;	TS 2000/08: vector plot has been separated to plotvec.pro
;	TS 2000/10: automatic level adjustment of contours,
;	            arrowloc added
;-
;------------------------------------------------------------------------
;
pro plotm, mparam, rdata, color=color, $
	autolevel=autolevel, fixedlevel=fixedlevel, $
	papersize=papersize, $
	btmin = btmin, btmax = btmax, arrowloc=arrowloc

@gparam

	if mparam.mesh lt 2 then km = ' (10!E4!Nkm)' else km=''

	if keyword_set(fixedlevel) then autolevel=0 else autolevel=1
		; default is autolevel as of 2000/10
	if not keyword_set(btmax) then btmax=1000.0
	if not keyword_set(btmin) then btmin=100.0

	if not keyword_set( color ) then begin
		fgcolor = !p.color
	endif else begin
		setcolor, bandwidth, w_start, rb_start, g_start, color8
		fgcolor = color8.white
	endelse

	xs=mparam.xs
	ys=mparam.ys
	nx=mparam.nx
	ny=mparam.ny
	dx=mparam.dx
	dy=mparam.dy
	xe = xs + dx *(nx-1)
	ye = ys + dy *(ny-1)

	if ( mparam.kindm lt 0 ) then pol2b, mparam, rdata
	kindm = mparam.kindm
	mklist, mparam, nxy,nz,kxlist,kylist,kzlist,strx,stry,strz

	x = xs + indgen(nx)*dx
	y = ys + indgen(ny)*dy

	setlevel,mparam,rdata, $
    	title1,intlevels,dummylevels,bplevels,bmlevels,dplevels,dmlevels, $
		jplevels,jmlevels,fplevels,fmlevels, $
		bunit,dunit,iunit,junit,funit

	kc=-1
	kx=-1
	ky=-1
	kx2=-1
	ky2=-1
	chrh = ' '
	chrc = ' '

	if ( nc ne 0 ) then begin
		kc = kzlist(nc-1)
		chrc = strmid( strz(nc-1), 0,1)
	endif
	if ( nv ne 0 and nv le nxy ) then begin
		kx = kxlist(nv-1)
		ky = kylist(nv-1)
	endif
	if ( nxy gt 1 and nv eq nxy+1 ) then begin
		kx = kxlist(1)
		ky = kylist(1)
		kx2 = kxlist(0)
		ky2 = kylist(0)
	endif

	title2 = ''
	title3 = ''
	
      xfl = 0.075
      yfl = 0.075
      xfr= 0.925
      yfr= 0.925
      xsize = xfr-xfl
      yfsize = yfr-yfl
      ysize = 1.3

; paper is assumed to be of square shape
; xfl,yfl = lower left corner of plot window in normalized coordinates
; xfr,yfr = upper right corner of plot window in normalized coordinates
; xsize,ysize = size of plot window in normalized coordinates

; determine magnification factor
      nx1 = fix(max([nx,nx*(dx/dy)]))
      ny1 = fix(max([ny,ny*(dy/dx)]))
      mag = min( [ xsize/float(nx1), ysize/float(ny1) ] )

; lower left and upper right corners of contour plot in normalized coordinates
	mapposition = [xfl, yfl, $
	               xfl+mag*(nx1-1), yfl+mag*(ny1-1) ]
; locations of title strings
	h = 0.04*ysize
	x3 = xfl	& y3 = yfl + mag*ny1 +h
	x2 = x3     & y2 = y3 + h
	x1 = x3      & y1 = y2 + h

; character size
	cs=0.75

	erase

; contour map

	nlevel=5
	if ( kc ge 0 ) then begin
		case chrc of
		'B':	begin
			plevels = bplevels & mlevels = bmlevels
			unit=bunit
			end
		'V':  begin
			plevels = dplevels & mlevels = dmlevels
			unit=dunit
			end
		'I':	begin
			plevels = intlevels & mlevels = dummylevels
			unit=iunit
			end
		'J':	begin
			plevels = jplevels & mlevels = jmlevels
			unit=junit
			end
		'F':	begin
			plevels = fplevels & mlevels = fmlevels
			unit=funit
			end
		else:	begin
			plevels = fplevels & mlevels = fmlevels
			unit=funit
			end
		endcase
		if keyword_set(autolevel) and chrc ne 'I' then $
			lvl125,rdata(*,*,kc),nlevel=nlevel, $
			plevel=plevels,mlevel=mlevels


		sz = size(plevels)
		nlevels = fix(sz(1))
		plinestyle = intarr(nlevels)	; 0=solid
		mlinestyle = plinestyle +2		; 2=dashed (1=dotted -- no good)
		pcolors = intarr(nlevels)
		mcolors = pcolors
		plabels= intarr(nlevels)
		mlabels=plabels
		pthick = intarr(nlevels)+1

		if not keyword_set( color ) then begin
			pcolors(*) = fgcolor
			mcolors(*) = fgcolor
			mthick = intarr(nlevels)+1
		endif else begin
			mthick = intarr(nlevels)+2
			case chrc of
			'I':	begin
				pcolors(*) = color8.yellow
				mcolors(*) = color8.yellow ; dummy
				end
			'V':	begin
				pcolors(*) = color8.blue
				mcolors(*) = color8.red
				end
			else:	begin
				pcolors(*) = color8.red
				mcolors(*) = color8.blue
				end
			endcase
		endelse

		title2 = strmid( $
			'CONTOURS:'+strz(nc-1) +'                    ',0,20) + $
  			'LEVELS='
		title2 = title2 + strlevel( plevels) +' '+ unit

		contour, rdata(*,*,kc), x,y, $
		/noerase, $
		levels=[mlevels,plevels], $
		c_linestyle = [mlinestyle,plinestyle], $
		c_color = [mcolors, pcolors], $
		c_thick = [mthick, pthick], $
		c_labels = [mlabels, plabels], $
		xstyle=1, ystyle=1, /follow, $
		color = fgcolor, $
            charsize=cs, $
		position = mapposition, /normal, $
		xtitle='!6X'+km, ytitle='!6Y'+km
	endif

	if kc lt 0 then begin
		contour, bytarr(nx,ny), x,y, $
		/noerase, /nodata, $
		xstyle=1, ystyle=1, $
		color = fgcolor, $
            charsize=cs, $
		position = mapposition, /normal, $
		xtitle='!6X'+km, ytitle='!6Y'+km
	endif

; vector map
	if ( kx ge 0 and ky ge 0 ) then begin

		kindmb = kindm mod 10
		if ( arrow gt 0 and kindmb gt 2 ) then arrow = 1 else arrow = 0
		if ( kx ne kloc(mparam,/px) and kindmb eq 6 ) then arrow =0
;		kindmb=6: data are pseudo B and Btp, and pseudo B is selected

		nbin = max([nx,ny])/50 +1

		plotvec, rdata(*,*,kx),rdata(*,*,ky), $
			xs=xs, ys=ys, dx=dx, dy=dy, $
			btmax=btmax, btmin=btmin, nbin=nbin, $
			arrow=arrow, color=fgcolor, /noinit, loc=arrowloc

		if kx2 ge 0 and ky2 ge 0 then  $
			plotvec, rdata(*,*,kx2),rdata(*,*,ky2), $
			xs=xs, ys=ys, dx=dx, dy=dy, $
			btmax=btmax, rbtmin=rbtmin, nbin=nbin, $
			arrow=arrow, color=fgcolor, /noinit

		title3 = title3 + $
			strmid( 'VECTORS :' + strx(nv-1) + ', ' + stry(nv-1) + $
			'          ', 0, 20 ) + $
			'MAX,MIN=' + string(btmax, btmin, format='(2f6.0)')
	endif


;	font = 0	; hardware font
	xyouts, x1,y1, /normal, title1, color=fgcolor, charsize=cs
	xyouts, x2,y2, /normal, title2, color=fgcolor, charsize=cs
	xyouts, x3,y3, /normal, title3, color=fgcolor, charsize=cs

	return
end

