;+
; NAME :	plotm3
; PURPOSE :
;	plot magnetograms on a line-drawing device
;	for printer output from disp3.pro
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	plotm3, mparam, rdata [,/color]
;		[,/autolevel] [,/fixedlevel]
;		[,papersize=papersize]
;		[,btmin=btmin] [,btmax=btmax] [,arrowloc=arrowloc]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	color : if set, plot output is made in color
;	/autolevel : contour levels are automatically adjusted to data (default)
;	/fixedlevel : contour levels are fixed as given by setlevel.pro
;	btmin : minimum value of BT to be plotted
;	btmax : value of BT corresponding to longest arrows
;	arrowloc : array of pointers where arrows will be plotted
;	papersize : horizontal width of the paper in cm
; OUTPUTS :
;	creates magnetogram plot
; COMMON BLOCKS : 	gparam is referenced as include file @gparam
;				common blkd is referenced
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/11/02
;-
;------------------------------------------------------------------------
;
pro plotm3, mparam, rdata, color=color, $
	autolevel=autolevel, fixedlevel=fixedlevel, $
	papersize=papersize, $
	btmin = btmin, btmax = btmax, arrowloc=arrowloc

	common blkd, coord_d
@gparam

	if keyword_set(fixedlevel) then autolevel=0 else autolevel=1
		; default is autolevel as of 2000/10
	if not keyword_set(btmax) then btmax=1000.0
	if not keyword_set(btmin) then btmin=100.0

	if not keyword_set( color ) then begin
		fgcolor = !p.color
		yellow = fgcolor
		green = fgcolor
	endif else begin
		setcolor, bandwidth, w_start, rb_start, g_start, color8
		fgcolor = color8.white
		yellow = color8.yellow
		green = color8.green
	endelse

	arcsec=0.07
	rsun =69.6
	ulen =10.0^9
	xxsp = coord_d.xxsp

	xs=mparam.xs
	ys=mparam.ys
	nx=mparam.nx
	ny=mparam.ny
	dx=mparam.dx
	dy=mparam.dy
	rsp=mparam.rsp

;  devcm : device unit corresponding to 1cm
;  vmicm : line spacing in cm
;  chrcm : character size in cm (only for plotm)
	devcm = 1.0/float(papersize)
	vmicm = 0.6
	chrcm = 0.25
	asepcm = 0.5
	vmi = vmicm*devcm
	chrsiz = chrcm*devcm
	arrsep = asepcm*devcm

;  dx0 and dy0 are physical lengths, while dx and dy may be
;  spherical coordinates

	if mesh le 1  or  meshl lt 0 then begin
		dxy=min([abs(dx),abs(dy)])
		dxy0=dxy
	endif else $
	if mesh ge 2 and meshl ge 0 then begin
		dx0=dx*rsp
		dy0=alofy(dy)*rsp
		dxy0=min([abs(dx0),abs(dy0)])
		dxy=min([abs(dx),abs(alofy(dy))])
	endif
	dxy0=max([1.0,float(min([nx,ny]))/50.0])*dxy0/3.0
	dxy =max([1.0,float(min([nx,ny]))/50.0])*dxy/3.0

	kindm = mparam.kindm
	mklist, mparam, nxy,nz,kxlist,kylist,kzlist,strx,stry,strz

	setlevel,mparam,rdata, $
    	title1,intlevels,dummylevels,bplevels,bmlevels,dplevels,dmlevels, $
		jplevels,jmlevels,fplevels,fmlevels, $
		bunit,dunit,iunit,junit,funit

;  four corners of observed region
	xf=[xs,xs+dx*(nx-1),xs+dx*(nx-1),xs,xs]
	yf=[ys,ys,ys+dy*(ny-1),ys+dy*(ny-1),ys]
;  xf and yf are in D-coordinates if mesh=0

	if meshl lt 0 then begin
		xc4=xf
		yc4=yf
		zc4=xf*0.0
	endif else $
	if mesh eq 0 then begin
		dtoc, xf,yf,xf*0.0, xc4,yc4,zc4
		projs, xc4,yc4,zc4
	endif else $
	if mesh eq 1 then begin
		ttoc, xf,yf,xf*0.0, xc4,yc4,zc4
	endif else $
	if mesh ge 2 and meshl ge 0 then begin
		transf, mparam, mesho, $
		nx1,ny1,xs1,ys1,dx1,dy1,xe1,ye1
		xc4=[xs1,xe1,xe1,xs1]
		yc4=[ys1,ys1,ye1,ye1]
		dtoc, xc4, yc4, xc4*0.0, xc4,yc4,zc4
	endif

	xfl = 0.075
	yfl = 0.075
	xfr= 0.925
	yfr= 0.925
	xsize = xfr-xfl
	yfsize = yfr-yfl
	ysize = 1.3
	xfc=0.5*(xfl+xfr)
	yfc=0.5*(yfl+yfr)

	gxc = 0.25*total(xc4(0:3))
	gyc = 0.25*total(yc4(0:3))

	gfact = min([xsize / (abs(xc4(1)-xc4(0)) $
		+ abs(xc4(3)-xc4(0)) ), $
	yfsize / (abs(yc4(1)-yc4(0))  $
		+ abs(yc4(3)-yc4(0)) ) ] )

        xclipl = min(gfact*(xc4(0:3)-gxc) +xfc, max=xclipr)
        yclipb = min(gfact*(yc4(0:3)-gyc) +yfc, max=yclipt)

	rotpt, xc4,yc4,zc4, xc4,yc4,zc4

	gxc = 0.25*total(xc4(0:3)) -dxc1
	gyc = 0.25*total(yc4(0:3)) -dyc1

	if scale gt 0.0 then begin
		gfact=devcm*scale
		scale1=scale
	endif else begin
		gfact = min([xsize / (abs(xc4(1)-xc4(0)) $
			+ abs(xc4(3)-xc4(0)) ), $
			yfsize / (abs(yc4(1)-yc4(0)) $
			+ abs(yc4(3)-yc4(0)) ) ] )
		if scale lt 0.0 then gfact=gfact*abs(scale)
		scale1=gfact/devcm
	endelse

; lower left and upper right corners of contour plot in normalized coordinates
	mapposition = [xclipl, yclipb, xclipr, yclipt]

; character size
	cs=0.75

;     set plotting scale
	plot, [xclipl,xclipr],[yclipb,yclipt], /nodata, /noerase, $
		xstyle=5, ystyle=5, $
		position=[xclipl,yclipb, xclipr,yclipt], /normal

	ki = mparam.status.i
	kz = kloc(mparam,/z)

	clip=1
	kc=-1
	kx=-1
	ky=-1
	kx2=-1
	ky2=-1
	chrh = ' '
	chrc = ' '

	if region ne 0 then begin
		dispr3, mparam,rdata,ki,xc4,yc4, fgcolor, yellow, /normal
		return
	endif

	if nc ne 0 then begin
		kc = kzlist(nc-1)
		chrc = strmid( strz(nc-1), 0,1)
	endif
	if nv ne 0 and nv le nxy then begin
		kx = kxlist(nv-1)
		ky = kylist(nv-1)
	endif
	if nxy gt 1 and nv eq nxy+1 then begin
		kx = kxlist(1)
		ky = kylist(1)
		kx2 = kxlist(0)
		ky2 = kylist(0)
	endif

	title2 = ''
	title3 = ''
	
	erase
	if rotdeg ne 0 or zdeg ne 0 or xdeg ne 0 or scale ne 0.0 then clip=0


;  scale indicator (longer than 3mm, shorter than 30mm)

	if meshl ge 0 then begin
		ixp=[ xfr-2.0*devcm, 0.0 ]
		iyp=[ yfr+0.5*devcm, 0.0 ]
		gauge=gfact/devcm*(rsp*1.0e9)/(rsun*ulen)
		if gauge lt 0.3 then begin
			gauge=10.0*gauge
			xyouts, ixp(0),iyp(0),'10**5 KM', color=fgcolor, /normal
		endif else $
		if gauge gt 3.0 then begin
			gauge=gauge/10.0
			xyouts, ixp(0),iyp(0), '10**3 KM', color=fgcolor, /normal
		endif else begin
			xyouts, ixp(0),iyp(0),'10**4 KM', color=fgcolor, /normal
		endelse
		ixp(0)=ixp(0)- 0.2*devcm
		iyp(0)=iyp(0)
		ixp(1)=ixp(0)- gauge*devcm
		iyp(1)=iyp(0)
		plots, ixp,iyp, thick=2, color=fgcolor, /normal
		ixp(1)=ixp(0)
		iyp(1)=iyp(0)+ 0.1*devcm
		iyp(0)=iyp(0)- 0.1*devcm
		plots, ixp,iyp, thick=2, color=fgcolor, /normal
		ixp(0)=ixp(0)- gauge*devcm
		ixp(1)=ixp(0)
		plots, ixp,iyp, thick=2, color=fgcolor, /normal
	endif

;  intensity in dots

	if dots eq 1 and ki ge 0 then $
		dispdot, mparam, rdata(*,*,ki), color=yellow, /normal

;  solar limb and longitude/latitude circles

	if globe eq 1 then begin
		lonlat=1
	endif else begin
		lonlat=0
	endelse
	limb=meshl ne 0

	sphere, xc4,yc4, dxy0,lonlat,limb,clip, $
		color=yellow, /normal

;  if limb=T, solar limb is drawn
;  if clip=T, long./lat. circles are only drawn inside the
;  observed region

; contour map

	nlevel=5
	if kc ge 0 then begin
		case chrc of
		'B':	begin
			plevels = bplevels & mlevels = bmlevels
			unit=bunit
			end
		'V':  begin
			plevels = dplevels & mlevels = dmlevels
			unit=dunit
			end
		'I':	begin
			plevels = intlevels & mlevels = dummylevels
			unit=iunit
			end
		'J':	begin
			plevels = jplevels & mlevels = jmlevels
			unit=junit
			end
		'F':	begin
			plevels = fplevels & mlevels = fmlevels
			unit=funit
			end
		else:	begin
			plevels = fplevels & mlevels = fmlevels
			unit=funit
			end
		endcase
		if keyword_set(autolevel) and chrc ne 'I' then $
			lvl125,rdata(*,*,kc),nlevel=nlevel, $
			plevel=plevels,mlevel=mlevels


		sz = size(plevels)
		nlevels = fix(sz(1))
		plinestyle = intarr(nlevels)	; 0=solid
		mlinestyle = plinestyle +2		; 2=dashed (1=dotted -- no good)
		pcolors = intarr(nlevels)
		mcolors = pcolors
		plabels= intarr(nlevels)
		mlabels=plabels
		pthick = intarr(nlevels)+1

		if not keyword_set( color ) then begin
			pcolors(*) = fgcolor
			mcolors(*) = fgcolor
			mthick = intarr(nlevels)+1
		endif else begin
			mthick = intarr(nlevels)+2
			case chrc of
			'I':	begin
				pcolors(*) = color8.yellow
				mcolors(*) = color8.yellow ; dummy
				end
			'V':	begin
				pcolors(*) = color8.blue
				mcolors(*) = color8.red
				end
			else:	begin
				pcolors(*) = color8.red
				mcolors(*) = color8.blue
				end
			endcase
		endelse

		title2 = strmid( $
			'CONTOURS:'+strz(nc-1) +'                    ',0,20) + $
  			'LEVELS='
		title2 = title2 + strlevel( plevels) +' '+ unit

		x = xs +dx*indgen(nx)
		y = ys +dy*indgen(ny)

		contour, rdata(*,*,kc), x,y, $
			path_info=cntrinfo, path_xy=cntrdata, /path_data_coords, $
		/noerase, $
		levels=[mlevels,plevels], $
		c_linestyle = [mlinestyle,plinestyle], $
		c_color = [mcolors, pcolors], $
		c_thick = [mthick, pthick], $
		c_labels = [mlabels, plabels], $
		xstyle=5, ystyle=5, $
		color = fgcolor, $
            charsize=cs, $
		position = mapposition, /normal

;	reset plotting scale
		plot, [xclipl,xclipr],[yclipb,yclipt],/nodata, /noerase, $
			xstyle=5, ystyle=5, $
			position=[xclipl,yclipb, xclipr,yclipt], /normal

		cntr3, cntrinfo,cntrdata, $
			/normal, $
			color= [mcolors, pcolors],style= [mlinestyle,plinestyle]
	endif

; vector map

	if kx ge 0 and ky ge 0 then begin

		kindmb = kindm mod 10
		if ( arrow gt 0 and kindmb gt 2 ) then arrow = 1 else arrow = 0
		if ( kx ne kloc(mparam,/px) and kindmb eq 6 ) then arrow =0
;		kindmb=6: data are pseudo B and Btp, and pseudo B is selected

		dispbar, mparam,rdata, kx,ky,kz, $
			btmax=btmax, btmin=btmin, $
			/normal, $
			color=fgcolor, loc=arrowloc

		if kx2 ge 0 and ky2 ge 0 then  $
			dispbar, mparam,rdata, kx2,ky2,kz, $
				btmax=btmax, btmin=btmin, $
				/normal, $
				color=color8.green

		title3 = title3 + $
			strmid( 'VECTORS :' + strx(nv-1) + ', ' + stry(nv-1) + $
			'          ', 0, 20 ) + $
			'MAX,MIN=' + string(btmax, btmin, format='(2f6.0)')
	endif

;  frame

;  arguments of pline2 should be D-coordinates
	if frame eq 1 then begin
		for n=0,3 do begin
			alen=sqrt( (xf(n+1)-xf(n))^2 + (yf(n+1)-yf(n))^2 )
			np=fix(alen/dxy)
			xp=xf(n)+(xf(n+1)-xf(n))*indgen(np)/float(np-1)
			yp=yf(n)+(yf(n+1)-yf(n))*indgen(np)/float(np-1)
			pline2, xp,yp, color=fgcolor, thick=1, /normal
		endfor
	endif

	title4 = string(rotdeg,zdeg,xdeg,scale1, $
	format="('ROTR=',f6.1,4x,'(ROTZ,ROTX)=',2f6.1,4x,'SCALE FACTOR=',f8.4)")

	xyouts, xfl, ysize-1*vmi, /normal, title1, color=fgcolor, charsize=cs
	xyouts, xfl, ysize-2*vmi, /normal, title2, color=fgcolor, charsize=cs
	xyouts, xfl, ysize-3*vmi, /normal, title3, color=fgcolor, charsize=cs
	xyouts, xfl, ysize-4*vmi, /normal, title4, color=fgcolor, charsize=cs

	return
end

