;+
; NAME :	plotvec
; PURPOSE :
;	display transverse field vectors
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	plotvec, bx,by
;		[,xs=xs] [,ys=ys] [,dx=dx] [,dy=dy]
;		[,arrow=arrow] [,/color] [,nbin=nbin]
;		[,/noinit] [,btmax=btmax] [,btmin=btmin] [,loc=loc]
; INPUTS :
;	bx,by : transverse field vectors
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	xs,ys : lower left corder of magnetogram, default=(0,0)
;	dx,dy : mesh spacing, default=(1,1)
;	arrow : if arrow=0, bars instead of arrows are plotted
;	        arrow=1 makes arrows of standard lengths, 
;	        arrow>1 makes correspondingly longer arrows
;	nbin : binning to be applied to (bx,by), default is to plot
;	       less than 100 arrows in x,y-directions
;	/noinit : use scaling established by previous call to contour procedure
;	btmin : minimum value of BT to be plotted
;	btmax : value of BT corresponding to longest arrows
;	loc : array of pointers where arrows will be plotted
;
; OUTPUTS :
;	creates vector plot
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	X-window environment is assumed
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/08 : separated from dispm.pro
;-
;------------------------------------------------------------------------

pro plotvec, bx,by, $
	xs=xs, ys=ys, dx=dx, dy=dy, $
	arrow=arrow, color=color, nbin=nbin, $
	noinit=noinit, btmax=btmax, btmin=btmin, loc=loc

	if not keyword_set(arrow) then arrow=0
	if n_elements(color) eq 0 then color=!p.color
	if not keyword_set(btmax) then btmax = max ( sqrt( bx^2 + by^2) )
	if n_elements(btmin) eq 0 then btmin= 0.1*btmax

	sz =size(bx)
	nx = sz(1)
	ny = sz(2)
	if not keyword_set(xs) then xs=0.0
	if not keyword_set(ys) then ys=0.0
	if not keyword_set(dx) then dx=1.0
	if not keyword_set(dy) then dy=1.0

	if not keyword_set(nbin) then  nbin = max([nx,ny])/50 +1

; set plotting scale
	if not keyword_set(noinit) then begin
		x = xs + dx*indgen(nx)
		y = ys + dy*indgen(ny)
		contour, bx, x,y, /noerase, /nodata, /isotropic, $
			xstyle=1, ystyle=1
	endif

; vector map

	xb=fltarr(2)
	yb=xb
	xa1=xb
	xa2=xb
	ya1=xb
	ya2=xb
	ds0 = 0.5* nbin * sqrt( dx*dx + dy*dy )
		; max half-size of bars and arrows
	dsa = 0.5* ds0
		; max half-size of arrow heads
	dsm = 0.1* ds0
		; minimum half-size of bars and arrows
	fa = 0.3

	if n_elements(loc) gt 0 then begin
		if loc(0) lt 0 then return
		for k=0, n_elements(loc)-1 do begin
			i = loc(k) mod nx
			j = loc(k) / nx
			bt = sqrt(bx(i,j)^2 +by(i,j)^2)
			if ( bt eq 0.0) then goto, endloop1
			ds = 0.9*dsm + (ds0-dsm) * $
				loglevel( bt, btmin, btmax) *(arrow>1)
			xb(0) = xs + dx*i + ds*bx(i,j)/bt
			yb(0) = ys + dy*j + ds*by(i,j)/bt
			xb(1) = xs + dx*i - ds*bx(i,j)/bt
			yb(1) = ys + dy*j - ds*by(i,j)/bt
			if arrow eq 0 then begin
				if (ds lt dsm) then goto, endloop1
				plots, xb, yb, $
				color = color, thick=2
			endif else begin
				if (ds lt dsm) then goto, endloop1
				plots, xb, yb, $
				color = color, thick=1
				xa1(0)=xb(0) & xa2(0)=xb(0)
				ya1(0)=yb(0) & ya2(0)=yb(0)
				xa1(1)=xa1(0)-dsa/bt*(bx(i,j)+fa*by(i,j))
				xa2(1)=xa2(0)-dsa/bt*(bx(i,j)-fa*by(i,j))
				ya1(1)=ya1(0)-dsa/bt*(by(i,j)-fa*bx(i,j))
				ya2(1)=ya2(0)-dsa/bt*(by(i,j)+fa*bx(i,j))
				plots, xa1, ya1, $
					color = color, thick=1
				plots, xa2, ya2, $
					color = color, thick=1
			endelse
endloop1:
		endfor

	endif else begin
		for j = 0, ny-1, nbin do begin
		for i = 0, nx-1, nbin do begin
			bt = sqrt(bx(i,j)^2 +by(i,j)^2)
			if ( bt eq 0.0) then goto, endloop2
			ds = 0.9*dsm + (ds0-dsm) * $
				loglevel( bt, btmin, btmax) *(arrow>1)
			xb(0) = xs + dx*i + ds*bx(i,j)/bt
			yb(0) = ys + dy*j + ds*by(i,j)/bt
			xb(1) = xs + dx*i - ds*bx(i,j)/bt
			yb(1) = ys + dy*j - ds*by(i,j)/bt
			if arrow eq 0 then begin
				if (ds lt dsm) then goto, endloop2
				plots, xb, yb, $
				color = color, thick=2
			endif else begin
				if (ds lt dsm) then goto, endloop2
				plots, xb, yb, $
				color = color, thick=1
				xa1(0)=xb(0) & xa2(0)=xb(0)
				ya1(0)=yb(0) & ya2(0)=yb(0)
				xa1(1)=xa1(0)-dsa/bt*(bx(i,j)+fa*by(i,j))
				xa2(1)=xa2(0)-dsa/bt*(bx(i,j)-fa*by(i,j))
				ya1(1)=ya1(0)-dsa/bt*(by(i,j)-fa*bx(i,j))
				ya2(1)=ya2(0)-dsa/bt*(by(i,j)+fa*bx(i,j))
				plots, xa1, ya1, $
					color = color, thick=1
				plots, xa2, ya2, $
					color = color, thick=1
			endelse
endloop2:
		endfor
		endfor
	endelse
return
end

