;+
; NAME :	rdfits
; PURPOSE :
;	read magnetogram file in FITS format
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	rdfits, mparam, rdata [,filename=fitsfile]
;		[,header=header] [,/list=list] [,/nolist] [,mode=mode]
; INPUTS :
;	none
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	filename : FITS magnetogram file name
;	header : a string array containing header data
;	mode : indicator of detected site-specific header records
;		0 = none, 1=magpack2, 2=Mees/Hawaii, 3=BBSO, 4=KPNO
;		5=KPNO(synoptic), 6=SOHO/MDI, 7=Mt.Wilson
;	/list : if set, parameter values in the header are printed
;	/nolist: if set, messages are not printed
; OUTPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
;	if filename is not specified, it should be typed in from keyboard
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/09
;	TS 2000/10/06 floating point data (nbytem=-4) added
;-
;------------------------------------------------------------------------
;

;  mapping from sine(latitude) to arcsin(latitude)
pro synop,mparam,rdata

	nx = mparam.nx
	nx0 = nx
	ny = mparam.ny
	ny0 = ny
	xs = mparam.xs
	xs0 = xs
	ys = mparam.ys
	ys0 = ys
	dx = mparam.dx
	dx0 = dx
	dy = mparam.dy
	dy0 = dy
	rdata1 = rdata

      halfpi=1.5707963

	if ys0 le -1.0 then begin
		ys=-halfpi
	endif else if ys0 ge 1.0 then begin
		ys=halfpi
	endif else begin
		ys=yofal(asin(ys0))
	endelse 
	ye0=ys0+dy0*(ny0-1)
	if ye0 le -1.0 then begin
		ye=-halfpi
	endif else if ye0 ge 1.0 then begin
		ye=halfpi
	endif else begin
		ye=yofal(asin(ye0))
	endelse
	dy=yofal(dy0)
	ny=(ye-ys)/dy+1
	dy=(ye-ys)/(ny-1)

	ss = size(rdata1)
	nz = ss(3)
	rdata=fltarr(nx,ny,nz)

	for k=0,nz-1 do begin
		for i=0, nx-1 do begin
 			buf = reform(rdata1(i,*,k))
			y = ys + dy*indgen(ny)
			sinl = sin(alofy(y))
			dj=(sinl-ys0)/dy0
			j1 = ( fix(dj) > 0 ) < (ny0-2)
	  		j2 = j1+1
			dj = dj -j1
;  sinl = ys0 + dy0*j
			rdata(i,*,k)=(1.0-dj)*buf(j1)+dj*buf(j2)
		endfor
	endfor
	mparam.nx = nx
	mparam.ny = ny
	mparam.xs = xs
	mparam.ys = ys
	mparam.dx = dx
	mparam.dy = dy
	return
end

pro rdfits, mparam,rdata, filename=filename, header=header, $
	list=list, nolist=nolist, mode=mode

	if n_elements(filename) eq 0 then begin
		filename=''
		read,'enter FITS file name ',filename
	endif
	openr,lun,filename,/get_lun
	header1=strarr(36)
	for i=0,35 do header1(i)=string(' ',format='(a80)')

	count=0
	while not eof(lun) do begin
		readu,lun,header1
		if count gt 0 then header=[header,header1] else header=header1
		for i=0,35 do begin
			if strcmp(header1(i),'END     ',8) then goto, enddetect
		endfor
		count=count+1
	endwhile
enddetect:
	dechdr,header,mparam, speca=speca,factor=factor,zlevel=zlevel, $
		mode=mode,skylvl=skylvl,  $
		list=list

	nx = long(mparam.nx)
	ny = long(mparam.ny)
	nbytem = mparam.nbytem
	nframe = mparam.nframe
      maxdim = max([ min([8,8*512L*512L/(nx*ny)]),nframe])
	rdata = fltarr(nx,ny,maxdim)
	nframe=mparam.nframe

	specs = defspecs(nspec)
	uflval, ufl,iufl2,iufl4

	for k=0, nframe-1 do begin
		if nbytem eq 1 then a=bytarr(nx,ny)
		if nbytem eq 2 then a=intarr(nx,ny)
		if nbytem eq 4 then a=lonarr(nx,ny)
		if nbytem eq -4 then a=fltarr(nx,ny)
		readu, lun, a
		if ( myint() lt 0) then byteorder,a
		sky = where(a eq skylvl,count)
		mult = a*0 +1
		add  = a*0
		if count gt 0 then begin
			mult(sky)=0
			add(sky) = ufl
		endif
		rdata(*,*,k) = mult*(factor(k)*a + zlevel(k)) +add
		for i=0, nspec-1 do begin
			if strcmp(speca(k),specs(i)) then begin
				mparam.status.(i) = k
				mparam.factors.(i) = factor(k)
			endif
		endfor
		if not keyword_set(nolist) then $
			print, k, '-th record is ', speca(k)
;	for Mees magnetogram: temporarily-stored BT,AZ will later be
;	converted to BX,BY
		if strcmp(speca(k),'TB',2) then begin
			mparam.status.bx = k
			mparam.factors.bx = factor(k)
		endif
		if strcmp(speca(k),'AZ',2) then begin
			mparam.status.by = k
			mparam.factors.by = mparam.factors.bx
		endif
	endfor

	close,lun
	free_lun, lun
	nbytem = abs(nbytem)

	modestr=['generic','magpack2','Mees/Hawaii','BBSO','KPNO', $
			'KPNO(synoptic)','SOHO/MDI','Mt.Wilson']

	if keyword_set(list) then begin
		print,'(proc.rdfits) mode=',mode,' ('+modestr(mode)+')'
		print,'parameters in ',filename
		printmp, mparam 
	endif

;  Mees: converion from (Bt,AZ) to (BX,NY)

	if mode eq 2 then begin
		kx = kloc(mparam,/x)
		ky = kloc(mparam,/y)
	  	bt=rdata(*,*,kx)
	  	az=rdata(*,*,ky)
	  	rdata(*,*,kx)=-bt*sin(az)
	  	rdata(*,*,ky)= bt*cos(az)
		print,'conversion from (bt,az) to (bx,by) done.'
	endif

;  KPNO synoptic data: re-gridding from sin(lat) to arcsin

	if mode eq 5 then begin
		synop,mparam,rdata
	  	print,'re-gridding of synoptic data done.'
	endif
	return
end

