;+
; NAME :	sunparam
; PURPOSE :
;	calculate solar parameters 
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	sunparam, date, time, rsun, p, b0, cl0, doy, carrot, tsys=tsys
; INPUTS :
;	date = 'yy/mm/dd' or 'yyyymmdd', time = 'hh:mm:ss'
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	tsys = 'GMT+hh', 'JST',,,etc.,  which indicates a timezone
;	        if not specified, assume tsys='UT'
; OUTPUTS :
;     rsun=  semidiameter in arcsec
;     p   =  position angle of sun's pole in deg.
;     b0  =  heliographic latitude of disk center in deg.
;     cl0 =  Carrington longitude of the meridian in deg.
;     doy =  day of year
;     carrot = Carrington rotation number
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	originally coded by K.Shibasaki 10/28/83 as koyomi/zs for OAO Melcom 70
;	T.Sakurai, 1992
;	           1993.2 tsys is added to argument list
;		   1999.1 format of year can be 'yyyymmdd'
;		   1999.5 uttime function is separated from this routine
;		          tsys moved from argument to keyword
;		   2000.1 rsun,p,b0,cl0,doy can be obtained as keywords
;	           2000.6 Carrington rotation number
;-
;------------------------------------------------------------------------
;
pro sunparam, date, time, rsun, p, b0, cl0, doy, carrot, tsys=tsys, $
	rsun=rsun1, p=p1, b0=b01, cl0=cl01, doy=doy1, carrot=carrot1

	rd=3.14159265358979d0/180.0d0

        if n_params() eq 0 then begin
          tsys=' '
          print,'enter timezone (GMT+hh, JST, etc.)'
          read, tsys
	  if strlen(tsys) eq 0 then tsys='ut' 
          date = ' '
rep: 	  print, 'enter date (yy/mm/dd or yyyymmdd)'
          read, date
	  if strlen(date) eq 0 then goto, rep
          time = ' '
          print, 'enter time (hh:mm:ss)'
          read, time
	  if strlen(time) eq 0 then time='00:00:00'
        endif

	if not keyword_set(tsys) then tsys='ut'

	if (strmid(date,2,1) eq '/') then begin
		year  = fix(strmid(date,0,2))
		if (year le 50) then year=year+2000 else year=year+1900
		month = fix(strmid(date,3,2))
		day   = fix(strmid(date,6,2))
	endif else begin
		year  = fix(strmid(date,0,4))
		month = fix(strmid(date,4,2))
		day   = fix(strmid(date,6,2))
	endelse

	ut = uthour(tsys,time)

	it1=367L*(year-1900)
	it2=((month+9)/12+year)*7/4
	it3=275L*month/9
	t1=double(it1-it2+it3+day) +ut/24.0d0
; new t1 = -3294 at 0 UT, 1900/01/01 (JD 2415020.5) = JD -2418314.5
; old t1 = JD -2418313.5 ?
	t =(t1+3293.5d0)/36525.0d0
; new t = (JD - 2415021)/36525 = Julian century since 1900/01/01 12 UT)
; old t = (JD -2415020)/36525
	tt=t1+20093.5d0
; new tt = JD - 2398221, JD2398221 = 12 UT, 1854/01/02 (?)
; old tt = JD -2398220
	gml=(279.691d0 + 36000.769d0*t)*rd
	g =(358.476d0 + 35999.050d0*t)*rd
	tgl=gml+(1.919d0-0.0048d0*t)*rd*sin(g)+0.020d0*rd*sin(2.0d0*g)
	rlg=0.00003d0+(0.000018d0*t-0.00727d0)*cos(g)-0.00009d0*cos(2.0d0*g)
	dist = float(10.0d0^(rlg)) ;     dist   distance in au
	rsun = float(961.18d0/dist)
	e =23.441d0*rd
	ai =7.250d0*rd
	omg=(73.739167d0+0.013958d0*tt/365.25d0)*rd
	b0 =asin(sin(tgl-omg)*sin(ai))
	cbclm=-cos(tgl-omg)
	cbslm=-sin(tgl-omg)*cos(ai)
	xlm=atan(cbslm,cbclm)
	itgl=long( ( (tgl-omg)/rd ) /360.0d0)
	cl0=(360.0d0-360.0d0/25.38d0*tt)*rd+xlm+itgl*360.0d0*rd
	p =-atan(cos(tgl)*tan(e))-atan(cos(tgl-omg)*tan(ai))
	b0=float(b0/rd)
	cl0=cl0/rd
	lc=long(cl0/360.0d0)
	cl0=float(cl0-360.0d0*(lc-1))
	carrot=-lc-42
	p = float(p/rd)
	idoy1=(month+9)/12*(1+((year mod 4)+2)/3)
	doy =float(it3-idoy1+day-30 +ut/24.0d0)

	if n_params() eq 0 then begin
	  tsysu=strtrim(strupcase(tsys),2)
	  print, date,', ',time,' ',tsysu
	  print, 'distance to the sun  =',dist,' AU'
	  print, 'solar apparent radius=',rsun,' arcsec'
	  print, 'p,b0,cl0=',p,b0,cl0,' degrees'
	  print, 'doy=',doy,'   Carrington rotation number=',carrot
	endif

	rsun1=rsun
	p1=p
	b01=b0
	cl01=cl0
	doy1=doy
	carrot1=carrot
	return
end
