;+
; NAME :	sunpos
; PURPOSE :
;	calculate the right ascension and declination of the sun
;	at a specified date and time
; CATEGORY :
; 	practical astronomy
; CALLING SEQUENCE :
;	sunpos, date, time, ra, dec, tsys = tsys, ecllon=ecllon
; INPUTS :
;	date given as 'yy/mm/dd' or 'yyyymmdd'
;       time = 'hh:mm:ss' or
;              a floating point number expressing the time in hour
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
; OUTPUTS :
;	ra = right ascension in degrees (not in hour)
;	dec = delclination in degrees
;	ecllon = ecliptic longitude in degrees
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
; RESTRICTIONS :
;	accuracy is about 1 minute of arc
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	based on Almanac for Computors (U.S.Naval Observatory)
;	T.Sakurai, 1999 May
;-
;====================================================================

function strtime, time, sec=sec
	hour=fix(abs(time))
	min=fix(60.0*abs(time)-60*hour)
	s =fix(3600.0*abs(time)-3600.0*hour-60.0*min)
	if time lt 0.0 then hour=-hour
	str = string(format='(i3,"h ",i2,"m ")', hour,min)
	if keyword_set(sec) then str=str+string(format='(i2,"s")',s)
	return, str
end

function strang, angle, sec=sec
	deg=fix(abs(angle))
	min=fix(60.0*abs(angle)-60*deg)
	s = fix(3600.0*abs(angle)-3600.0*deg-60.0*min)
	if angle lt 0.0 then deg=-deg
	str = string(format='(i4,"deg ",i2,"min")', deg,min)
	if keyword_set(sec) then str=str+string(format='(i2,"sec")',s)
	return, str
end

pro sunpos, date, time, ra, dec, tsys=tsys, ecllon=ecllon

	rd=3.14159265358979d0/180.0d0

	if n_params() eq 0 then begin
	  date = ' '
	  print, 'enter date (yy/mm/dd or yyyymmdd)'
	  read, date
	  tsys = ' '
	  print, 'enter time zone (UT, JST, etc.)'
	  read, tsys
	  time = ' '
	  print, 'enter time (hh:mm:ss)'
	  read, time
	endif

	if (strmid(date,2,1) eq '/') then begin
		year	= fix(strmid(date,0,2))
		if (year le 50) then year=year+2000 else year=year+1900
		month	= fix(strmid(date,3,2))
		day	= fix(strmid(date,6,2))
	endif else begin
		year	= fix(strmid(date,0,4))
		month	= fix(strmid(date,4,2))
		day	= fix(strmid(date,6,2))
	endelse

	ut = uthour(tsys, time)
	eqt= eqtime(date, ut, tsys='ut')
        it1=367L*(year-1900)
        it2=((month+9)/12+year)*7/4
        it3=275L*month/9
        t1=double(it1-it2+it3+day) +ut/24.0d0
; t1 = JD -2418313.5
        t =(t1+3293.5d0)/36525.0d0 -1.0d0
; t = (JD -2451545)/36525
	am = 357.528d0 + 35999.050d0*t
	al = 280.460d0 + 36000.772d0*t
	ecllon = al + (1.915d0 -0.0048d0*t)*sin(rd*am) + $
		0.020d0*sin(2*rd*am)
	while (ecllon lt -180.0) do ecllon =ecllon+360.0
	while (ecllon ge  180.0) do ecllon =ecllon-360.0
	eps = 23.439d0 -0.013d0*t
	ra = atan( cos(rd*eps)*sin(rd*ecllon), cos(rd*ecllon) )/rd
	dec = asin( sin(rd*eps)*sin(rd*ecllon) )/rd
	if ra lt 0.0 then ra=ra+360.0
	if ecllon lt 0.0 then ecllon =ecllon+360.0

	if n_params() eq 0 then begin
	  tsysu=' '+strtrim(strupcase(tsys),2)
	  print, '(proc.suncal) date=',date,' time=',time, tsysu
	  print, '     Sun''s position:'
	  print, '     RA = ', strtime(ra/15.0)
	  print, '     Dec= ', strang(dec)
	  print, '     ecliptic longitude=', strang(ecllon)
	endif
	return
end
