;+
; NAME :	suntrans
; PURPOSE :
;	calculate the meridian transit time in UT of the sun at a specified
;	date and at a specified longitude
; CATEGORY :
; 	practical astronomy
; CALLING SEQUENCE :
;	tmt = suntrans(date, longitude=longitude)
; INPUTS :
;	date given as 'yy/mm/dd' or 'yyyymmdd'
; OPTIONAL INPUT PARAMETERS : 
;	longitude :	if not specified, assume longitude=0.0
; KEYWORD PARAMETERS :
; OUTPUTS :
;	tmt = meridian transit time of the sun in UT hour
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
; RESTRICTIONS :
;	accuracy is about 2 seconds
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	based on Almanac for Computors (U.S.Naval Observatory)
;	originally coded by K.Shibasaki 10/28/83 as koyomi/zs for OAO Melcom 70
;	T.Sakurai, 1993
;	TS 97/12/22 parameters updated
;	TS 99/01/04 parameters updated, format of year can be 'yyyymmdd'
;-
;====================================================================

function strtime, time, sec=sec
	hour=fix(abs(time))
	min=fix(60.0*abs(time)-60*hour)
	s =fix(3600.0*abs(time)-3600.0*hour-60.0*min)
	if time lt 0.0 then hour=-hour
	str = string(format='(i3,"h ",i2,"m ")', hour,min)
	if keyword_set(sec) then str=str+string(format='(i2,"s")',s)
	return, str
end

function strang, angle, sec=sec
	deg=fix(abs(angle))
	min=fix(60.0*abs(angle)-60*deg)
	s = fix(3600.0*abs(angle)-3600.0*deg-60.0*min)
	if angle lt 0.0 then deg=-deg
	str = string(format='(i4,"deg ",i2,"min")', deg,min)
	if keyword_set(sec) then str=str+string(format='(i2,"sec")',s)
	return, str
end

function suntrans, date, longitude=longitude, tokyo=tokyo

	rd=3.14159265358979d0/180.0d0

	if n_params() eq 0 then begin
	  date = ' '
	  print, 'enter date (yy/mm/dd or yyyymmdd)'
	  read, date
	  ldeg=0 & lmin=0 & lsec=0
	  print, 'enter longitude (deg, min, sec)'
	  read, ldeg,lmin,lsec
	  longitude = abs(ldeg)+lmin/60.0 +lsec/3600.0
	  if ldeg lt 0 then longitude = -longitude
	endif

	if not keyword_set(longitude) then longitude=0.0

	if keyword_set(tokyo) then longitude =139.745	; 139deg 44'41"

	if (strmid(date,2,1) eq '/') then begin
		year	= fix(strmid(date,0,2))
		if (year le 50) then year=year+2000 else year=year+1900
		month	= fix(strmid(date,3,2))
		day	= fix(strmid(date,6,2))
	endif else begin
		year	= fix(strmid(date,0,4))
		month	= fix(strmid(date,4,2))
		day	= fix(strmid(date,6,2))
	endelse

	time = 12.0 - longitude/15.0
	if time lt 0.0 then time=time+24
	eqt= eqtime(date, time, tsys='ut')

	tmt=12.0 -eqt-longitude/15.0
	tmt1 =tmt
	if(tmt lt 0) then tmt1=tmt1+24.0

	if n_params() eq 0 then begin
	  print, '(func.suntrans) date=',date, $
			' longitude=',strang(longitude,/sec)
	  print, 'meridian transit time of the sun = ', $
		strtime(tmt1,/sec), ' UT'
	endif
	return, tmt
end
