;+
; NAME :	transm
; PURPOSE :
;	coordinate and component transforms of magnetograms
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	transm, mparam, rdata [,kindmo=kindm2] [,mesho=mesh2]
;			[,rcutoff=rcutoff]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	kindmo : target value of kindm
;	mesho : target value of mesh
;	rcutoff : data with r>rcutoff are set to ufl (unfilled value)
; OUTPUTS :
; COMMON BLOCKS : 	blkd, blkt, blks, blkr are used
; SIDE EFFECTS : 	results are overwritten in rdata
;					mparam may be modified
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/10
;
;------------------------------------------------------------------------
;-
;  gridpoints
;  return value (x,y,z) are in cartesian
;  the current grid system is specified by mparam

pro gridpt, mparam, x,y,z

	nx = mparam.nx
	ny = mparam.ny
	xs = mparam.xs
	ys = mparam.ys
	dx = mparam.dx
	dy = mparam.dy
	mesh = mparam.mesh

	x1 = fltarr(nx,ny)
	y1 = fltarr(nx,ny)
	z1 = fltarr(nx,ny)

	for j=0,ny-1 do x1(*,j)=xs+dx*indgen(nx)
	for i=0,nx-1 do y1(i,*)=ys+dy*indgen(ny)

	if mesh eq 0 then begin
		dtoc, x1,y1,z1, x,y,z
	endif else $
	if mesh eq 1 then begin
		ttoc, x1,y1,z1, x,y,z
	endif else $
	if mesh eq 2 then begin
		stoc, x1,alofy(y1),z1, x,y,z
	endif else $
	if mesh ge 3 then begin
		rtoc, x1,alofy(y1),z1, x,y,z
	endif else $
	if mesh eq -1 then begin
		x=x1
		y=y1
		z=z1
		projt, x,y,z,xt,yt
	endif else $
	if mesh eq -2 then begin
		x=x1
		y=y1
		z=z1
		projs, x,y,z,ap,al
	endif

	return
end

;  convert b-components (b1,b2,b3) defined in a coordinate system
;  (e.g. (bl,bt) ) into the cartesian vector components bx,by,bz

;  m5 (usually the 5-th digit of kindm) specifies the coordinate system
;  defining the components of b

;  m5 need not be equal to mesh.

;  (x,y,z) should contain the C-coordinates of the point where b is defined.
;  if m5 = 0 or 1, (x,y,z) are not actually used because the conversion does
;  not depend on the position vector.

pro ctovec, b1,b2,b3, bx,by,bz,m5, x,y,z

	common blkd, coord_d
	common blkt, coord_t
	common blks, coord_s
	common blkr, coord_r

;  convert (b1,b2,b3) into cartesian vector components bx,by,bz

	if m5 eq 0 then begin
		xdu = coord_d.xdu
		ydu = coord_d.ydu
		zdu = coord_d.zdu
		bx= xdu(0)*b1 +ydu(0)*b2 +zdu(0)*b3
		by= xdu(1)*b1 +ydu(1)*b2 +zdu(1)*b3
		bz= xdu(2)*b1 +ydu(2)*b2 +zdu(2)*b3

	endif else $
	if m5 eq 1 then begin
		xtu = coord_t.xtu
		ytu = coord_t.ytu
		ztu = coord_t.ztu
		bx= xtu(0)*b1 +ytu(0)*b2 +ztu(0)*b3
		by= xtu(1)*b1 +ytu(1)*b2 +ztu(1)*b3
		bz= xtu(2)*b1 +ytu(2)*b2 +ztu(2)*b3

	endif else $
	if m5 eq 2 then begin
		ctos, x,y,z, ap,al,h
		svu, ap,al, apu,alu,hu
		bx= apu(0,*)*b1 +alu(0,*)*b2 +hu(0,*)*b3
		by= apu(1,*)*b1 +alu(1,*)*b2 +hu(1,*)*b3
		bz= apu(2,*)*b1 +alu(2,*)*b2 +hu(2,*)*b3

	endif else $
	if m5 ge 3 then begin
		ctor, x,y,z, ap,al,h
		rvu, ap,al, apru,alru,hu
		bx= apru(0,*)*b1 +alru(0,*)*b2 +hu(0,*)*b3
		by= apru(1,*)*b1 +alru(1,*)*b2 +hu(1,*)*b3
		bz= apru(2,*)*b1 +alru(2,*)*b2 +hu(2,*)*b3
	endif
	return
end

;  projection of cartesian vector (bx,by,bz) into a coordinate system
;  specified by m5 ( =5-th digit of kindm)

;  m5 need not be equal to mesh.

;  (x,y,z) should contain the C-coordinates of the point where b is defined.
;  if m5 = 0 or 1, (x,y,z) are not actually used because the conversion does
;  not depend on the position vector.

pro vectoc, bx,by,bz, b1,b2,b3, m5, x,y,z

	common blkd, coord_d
	common blkt, coord_t
	common blks, coord_s
	common blkr, coord_r

	if m5 eq 0 then begin
;  b=I*bb
		xdu = coord_d.xdu
		ydu = coord_d.ydu
		zdu = coord_d.zdu
		b1= bx*xdu(0)+by*xdu(1)+bz*xdu(2)
		b2= bx*ydu(0)+by*ydu(1)+bz*ydu(2)
		b3= bx*zdu(0)+by*zdu(1)+bz*zdu(2)
	endif else $
	if m5 eq 1 then begin
;  b=atinv*bb
		xtu = coord_t.xtu
		ytu = coord_t.ytu
		ztu = coord_t.ztu
		b1= bx*xtu(0)+by*xtu(1)+bz*xtu(2)
		b2= bx*ytu(0)+by*ytu(1)+bz*ytu(2)
		b3= bx*ztu(0)+by*ztu(1)+bz*ztu(2)
	endif else $
	if m5 eq 2 then begin
		ctos, x,y,z, ap,al,h
		svu, ap,al, apu,alu,hu
		b1=bx*apu(0,*)+by*apu(1,*)+bz*apu(2,*)
		b2=bx*alu(0,*)+by*alu(1,*)+bz*alu(2,*)
		b3=bx* hu(0,*)+by* hu(1,*)+bz* hu(2,*)
	endif else $
	if m5 ge 3 then begin
		ctor, x,y,z, ap,al,h
		rvu, ap,al, apru,alru,hu
		b1=bx*apru(0,*)+by*apru(1,*)+bz*apru(2,*)
		b2=bx*alru(0,*)+by*alru(1,*)+bz*alru(2,*)
		b3=bx*  hu(0,*)+by*  hu(1,*)+bz*  hu(2,*)
	endif
	return
end

pro transm, mparam, rdata, kindmo=kindm2, mesho=mesh2, $
	rcutoff=rcutoff

	common blkd, coord_d
	common blkt, coord_t
	common blks, coord_s
	common blkr, coord_r

	uflval, ufl
	pi=3.141592
	if not keyword_set(rcutoff) then rcutoff=0.95

	kindm1 = mparam.kindm
	mesh1  = mparam.mesh
	if n_elements(kindm2) eq 0 then kindm2=kindm1
	if n_elements(mesh2) eq 0 then mesh2=mesh1

	if kindm1*kindm2 lt 0 then begin
		print, '(proc.transm) incompatible specification,', $
		'kindm1,kindm2=', kindm1,kindm2
		return
	endif

;  transform of grids
;  data are first transformed to D-grid if necessary, and then
;  transformed to the final grid specified by mesh2

	setdu, mparam
	settu, mparam
	setas, mparam
	setar, mparam

	dimdata = size(rdata)
	if dimdata(0) eq 3 then nframe0 = dimdata(3) else nframe0=1

	kx = kloc(mparam,/x)
	ky = kloc(mparam,/y)
	kz = kloc(mparam,/z)
	kxp = kloc(mparam, /px)
	kyp = kloc(mparam, /py)
	kzp = kloc(mparam, /pz)

	if kindm1 gt 0 and kindm2 gt 0 then begin
		oldm = umake(kindm1, m5,m4,m3,m2,m1)
		newm = umake(kindm2, m5new,m4new,m3new,m2new,m1new)

		if m5 gt 4 or m5new gt 4 or $
		chktrans(oldm,newm) eq 0 then begin
			print, '(proc.transm) incompatible specification,', $
			'kindm1,kindm2=', kindm1,kindm2
			return
		endif
	endif

	if  mesh1 eq -1 and mesh2 eq 0 then begin
		print,'(proc.transm) change mesh1 from -1 to 0'
		mesh1=mesh2
	endif
	if  mesh2 eq mesh1 then goto, vectortrans

	xs = mparam.xs
	ys = mparam.ys
	dx = mparam.dx
	dy = mparam.dy
	nx = mparam.nx
	ny = mparam.ny
	rsp = mparam.rsp
	xs0 = xs
	ys0 = ys
	dx0 = dx
	dy0 = dy
	nx0 = nx
	ny0 = ny

	kmem,mparam,rdata,karr=kmemarr,/nolist
	nframe = mparam.nframe

	print, '  old parameters (nx,ny)=',nx0,ny0
	print, '                 (xs,ys)=',xs0,ys0
	print, '                 (dx,dy)=',dx,dy

;  from T- to D-grid
	if mesh1 eq 1 then begin
		transf, mparam, 0, nx ,ny ,xs ,ys ,dx ,dy ,xe,ye
		newdata = fltarr(nx,ny, nframe0)

		for km=0, nframe-1 do begin
			k=kmemarr(km)
			xd = fltarr(nx,ny)
			yd = fltarr(nx,ny)
			zd = fltarr(nx,ny)
			for jnew=0,ny-1 do xd(*,jnew)=xs+dx*indgen(nx)
			for inew=0,nx-1 do yd(inew,*)=ys+dy*indgen(ny)
			dtoc, xd,yd,zd, x,y,z
			projt, x,y,z, xt,yt

			loc =  getij (nx0,ny0,xs0,ys0,dx0,dy0,xt,yt,ii,jj,xr,yr)
			b1=shift(rdata(*,*,k), 0, 0)
			b2=shift(rdata(*,*,k),-1, 0)
			b3=shift(rdata(*,*,k),-1,-1)
			b4=shift(rdata(*,*,k), 0,-1)
			b0 = ufl+fltarr(nx,ny)
			if (loc(0) ge 0) then begin
				i=ii(loc)
				j=jj(loc)
				b0(loc)= b1(i,j)+ $
					(b2(i,j)-b1(i,j))*xr(loc)+ $
					(b4(i,j)-b1(i,j))*yr(loc)+ $
					(b3(i,j)-b4(i,j)-b2(i,j)+b1(i,j))* $
						xr(loc)*yr(loc)
			endif else stop
			newdata(*,*,k)=b0
		endfor
		rdata = newdata
	endif

;  from S/R- to D-grid
	if mesh1 ge 2 then begin
		if mesh1 eq 2 then begin
			psave=mparam.prad
			b0save=mparam.b0rad
			mparam.prad=mparam.ypos
			mparam.b0rad=0.0
			setar, mparam
		endif
		transf, mparam, 0, nx ,ny ,xs ,ys ,dx ,dy ,xe,ye
		newdata = fltarr(nx,ny, nframe0)
		xxsp = coord_r.xxsp
		xe0 = xs0+dx0*(nx0-1)

		for km=0, nframe-1 do begin
			k=kmemarr(km)
			xd = fltarr(nx,ny)
			yd = fltarr(nx,ny)
			zd = fltarr(nx,ny)
			for jnew=0,ny-1 do xd(*,jnew)=xs+dx*indgen(nx)
			for inew=0,nx-1 do yd(inew,*)=ys+dy*indgen(ny)
			dtoc, xd,yd,zd, x,y,z
			projr, x,y,z, ap,al
			loc = where(ap lt xs0, count)
			if count gt 0 then ap(loc)=ap(loc)+2.0*pi
			loc = where(ap gt xe0, count)
			if count gt 0 then ap(loc)=ap(loc)-2.0*pi
			loc =  getij(nx0,ny0,xs0,ys0,dx0,dy0,ap,yofal(al),ii,jj,xr,yr)
			b1=shift(rdata(*,*,k), 0, 0)
			b2=shift(rdata(*,*,k),-1, 0)
			b3=shift(rdata(*,*,k),-1,-1)
			b4=shift(rdata(*,*,k), 0,-1)
			b0 = ufl+fltarr(nx,ny)
			if (loc(0) ge 0) then begin
				ap1 = xs0 + dx0*ii(loc)
				al1 = alofy(ys0 + dy0*jj(loc))
				rtoc, ap1,al1, 0.0*al1, x1,y1,z1
				vis = where(z1 gt xxsp(2))
			;	retain data in visible hemisphere
				if vis(0) lt 0 then stop
				i = ii(loc(vis))
				j = jj(loc(vis))
				b0(loc(vis))= b1(i,j)+ $
					(b2(i,j)-b1(i,j))*xr(loc(vis))+ $
					(b4(i,j)-b1(i,j))*yr(loc(vis))+ $
					(b3(i,j)-b4(i,j)-b2(i,j)+b1(i,j))* $
						xr(loc(vis))*yr(loc(vis))
			endif else stop
			newdata(*,*,k)=b0
		endfor
		rdata = newdata

		if mesh1 eq 2 then begin
			mparam.prad = psave
			mparam.b0rad = b0save
			setar, mparam
		endif
		xtod = xs+0.5*dx*(nx-1)
		ytod = ys+0.5*dy*(ny-1)
		mparam.xxtod=[xtod,ytod]
	endif

	nx0=nx
	ny0=ny
	xs0=xs
	ys0=ys
	dx0=dx
	dy0=dy

;  from D- to T-grid
	if mesh2 eq 1 then begin
		transf, mparam, mesh2, $
			nx ,ny ,xs ,ys ,dx ,dy ,xe,ye
		newdata = fltarr(nx,ny, nframe0)

		for km=0, nframe-1 do begin
			k=kmemarr(km)
			xt = fltarr(nx,ny)
			yt = fltarr(nx,ny)
			zt = fltarr(nx,ny)
			for jnew=0,ny-1 do xt(*,jnew)=xs+dx*indgen(nx)
			for inew=0,nx-1 do yt(inew,*)=ys+dy*indgen(ny)
			ttod, xt,yt,zt, xd,yd,zd

			loc =  getij (nx0,ny0,xs0,ys0,dx0,dy0,xd,yd,ii,jj,xr,yr)
			b1=shift(rdata(*,*,k), 0, 0)
			b2=shift(rdata(*,*,k),-1, 0)
			b3=shift(rdata(*,*,k),-1,-1)
			b4=shift(rdata(*,*,k), 0,-1)
			b0 = ufl+fltarr(nx,ny)
			if (loc(0) ge 0) then begin
				i=ii(loc)
				j=jj(loc)
				b0(loc)= b1(i,j)+ $
					(b2(i,j)-b1(i,j))*xr(loc)+ $
					(b4(i,j)-b1(i,j))*yr(loc)+ $
					(b3(i,j)-b4(i,j)-b2(i,j)+b1(i,j))* $
						xr(loc)*yr(loc)
			endif else stop

			newdata(*,*,k)=b0
		endfor
		rdata = newdata
		settu, mparam
	endif

;  from D- to S/R-grid
	if mesh2 ge 2 then begin
		if mesh2 eq 2 then begin
			psave = mparam.prad
			b0save = mparam.b0rad
			mparam.prad = mparam.ypos
			mparam.b0rad = 0.0
		endif
		setar, mparam
;	setar sets cl0eff based on mparam.mesh
;	here cl0eff is set based on mesh2
		if mesh2 gt 2 then coord_r.cl0eff = coord_r.cl0 $
			else		coord_r.cl0eff = 0.0

		transf, mparam, mesh2, $
			nx ,ny ,xs ,ys ,dx ,dy ,xe,ye
		newdata = fltarr(nx,ny, nframe0)
		xxsp = coord_r.xxsp

		for km=0, nframe-1 do begin
			k=kmemarr(km)
			ap = fltarr(nx,ny)
			al = fltarr(nx,ny)
			h  = fltarr(nx,ny)
			for jnew=0,ny-1 do ap(*,jnew)=xs+dx*indgen(nx)
			for inew=0,nx-1 do al(inew,*)=alofy(ys+dy*indgen(ny))
			rtod, ap,al,h, xd,yd,zd
			r = sqrt((xd-xxsp(0))^2 + (yd-xxsp(1))^2)
			loc =  getij (nx0,ny0,xs0,ys0,dx0,dy0,xd,yd,ii,jj,xr,yr)
			b1=shift(rdata(*,*,k), 0, 0)
			b2=shift(rdata(*,*,k),-1, 0)
			b3=shift(rdata(*,*,k),-1,-1)
			b4=shift(rdata(*,*,k), 0,-1)
			b0 = ufl+fltarr(nx,ny)
			if (loc(0) ge 0) then begin
				r = r(loc)
				zd = zd(loc)
				valid = where(zd ge xxsp(2) and r lt rsp*rcutoff)
				if valid(0) lt 0 then stop
				i = ii(loc(valid))
				j = jj(loc(valid))
				b0(loc(valid))= b1(i,j)+ $
					(b2(i,j)-b1(i,j))*xr(loc(valid))+ $
					(b4(i,j)-b1(i,j))*yr(loc(valid))+ $
					(b3(i,j)-b4(i,j)-b2(i,j)+b1(i,j))* $
						xr(loc(valid))*yr(loc(valid))
			endif else stop
			newdata(*,*,k)=b0
		endfor
		rdata = newdata
		if mesh2 eq 2 then begin
			mparam.prad = psave
			mparam.b0rad = b0save
			mparam.mesh = mesh2
			setar, mparam
		endif
	endif

	mparam.xs=xs
	mparam.ys=ys
	mparam.nx=nx
	mparam.ny=ny
	mparam.dx=dx
	mparam.dy=dy
	mparam.mesh = mesh2
	print, '  new parameters (nx,ny)=',nx,ny
	print, '                 (xs,ys)=',xs,ys
	print, '                 (xe,ye)=',xe,ye
	print, '                 (dx,dy)=',dx,dy

vectortrans:

	if kindm1 ge 0 and kindm2 ge 0 and $
	  (m5new ne m5 or m1new ne m1) then begin

		if m5new eq 0 and m1new eq 9 or $
	   	   m5new gt 0 and (m1new eq 6 or m1new eq 9) then begin

;  transform of B-components by assuming that B is radial
;  this is the same transform as the doppler velocity
;  so that proc.transv is used

;  m5=0(BL),1(Bn),2-4(Br)
;  m3=1(VL),2(Vn),3(Vr)

			mm3   =(m5+1) < 3
			mm3new=(m5new+1) < 3

			v=rdata(*,*,kz)
			gridpt, mparam, x,y,z
			transv, mm3,mm3new,v,vnew,vxnew,vynew,x,y,z
			loc = where(v eq ufl, count)
			if count gt 0 then vnew(loc)=ufl
			rdata(*,*,kz)=vnew
			if kx ge 0 and ky ge 0 then begin
				rdata(*,*,kx)=vxnew
				rdata(*,*,ky)=vynew
			endif

			mparam.status.(lspec(mparam, kz))=-1
			if m5new eq 0 then begin
			  	mparam.status.bl = kz
			endif else $
			if m5new eq 1 then begin
		  		mparam.status.bn = kz
			endif else $
			if m5new ge 2 then begin
		  		mparam.status.br = kz
			endif

			if kx ge 0 and ky ge 0 then begin
		  		mparam.status.(lspec(mparam, kx)) =-1
		  		mparam.status.(lspec(mparam, ky)) =-1
				if m1new eq 9 then begin
		  			if m5new eq 0 then begin
		    				mparam.status.bx = kx
		    				mparam.status.by = ky
		  			endif else $
					if m5new eq 1 then begin
		    				mparam.status.bhx = kx
		    				mparam.status.bhy = ky
		  			endif else $
					if m5new ge 2 then begin
		    				mparam.status.bph = kx
		    				mparam.status.bth = ky
		  			endif
				endif else mparam.nframe=mparam.nframe-2
			endif
			print, '(proc.transm) magnetic field conversion with radial assumption'

		endif else begin

;  transform of B-vector components

			b1=rdata(*,*,kx)
			b2=rdata(*,*,ky)
			b3=rdata(*,*,kz)
			loc = where(b1 eq ufl or b2 eq ufl or b3 eq ufl, count)
			gridpt, mparam, x,y,z
			ctovec, b1,b2,b3, bx,by,bz, m5,x,y,z
			vectoc, bx,by,bz, b1,b2,b3, m5new,x,y,z
			if count gt 0 then begin
				b1(loc) = ufl
				b2(loc) = ufl
				b3(loc) = ufl
			endif

			rdata(*,*,kx)=b1
			rdata(*,*,ky)=b2
			rdata(*,*,kz)=b3

			if kx eq kxp and ky eq kyp and kz eq kzp then goto, potfield

			mparam.status.(lspec(mparam, kx)) =-1
			mparam.status.(lspec(mparam, ky)) =-1
			mparam.status.(lspec(mparam, kz)) =-1
			if m5new eq 0 then begin
				mparam.status.bx = kx
				mparam.status.by = ky
				mparam.status.bl = kz
			endif else $
			if m5new eq 1 then begin
				mparam.status.bhx = kx
				mparam.status.bhy = ky
				mparam.status.bn  = kz
			endif else $
			if m5new ge 2 then begin
				mparam.status.bph = kx
				mparam.status.bth = ky
				mparam.status.br  = kz
			endif

			print,'(proc.transm) conversion of magnetic field vectors completed'

			if kxp ge 0 and kyp ge 0 then begin
				b1=rdata(*,*,kxp)
				b2=rdata(*,*,kyp)
				b3=rdata(*,*,kzp)
				loc = where(b1 eq ufl or b2 eq ufl or b3 eq ufl, count)
				gridpt, mparam, x,y,z
				ctovec, b1,b2,b3, bx,by,bz, m5,x,y,z
				vectoc, bx,by,bz, b1,b2,b3, m5new,x,y,z
				if count gt 0 then begin
					b1(loc) = ufl
					b2(loc) = ufl
					b3(loc) = ufl
				endif
				rdata(*,*,kxp)=b1
				rdata(*,*,kyp)=b2
				rdata(*,*,kzp)=b3
potfield:
				mparam.status.(lspec(mparam, kxp)) =-1
				mparam.status.(lspec(mparam, kyp)) =-1
				mparam.status.(lspec(mparam, kzp)) =-1
				if m5new eq 0 then begin
					mparam.status.bxp = kxp
					mparam.status.byp = kyp
					mparam.status.blp = kzp
				endif else $
				if m5new eq 1 then begin
					mparam.status.bhxp = kxp
					mparam.status.bhyp = kyp
					mparam.status.bnp  = kzp
				endif else $
				if m5new ge 2 then begin
					mparam.status.bphp = kxp
					mparam.status.bthp = kyp
					mparam.status.brp  = kzp
				endif
				print,'(proc.transm) conversion of potential field vectors completed'
			endif
		endelse
	endif

;  transformation of Doppler velocity

	kd=kloc(mparam,/d)
	if kd ge 0 and m3new ne m3 then begin
		v=rdata(*,*,kd)
		loc = where(v eq ufl, count)
		gridpt, mparam, x,y,z
		transv, m3,m3new,v,vnew,vxnew,vynew,x,y,z
		if count gt 0 then vnew(loc)=ufl
		rdata(*,*,kd)=vnew

		mparam.status.(lspec(mparam, kd)) =-1
		if m3new eq 0 then begin
			mparam.status.vl = kd
		endif else $
		if m3new eq 1 then begin
			mparam.status.vn = kd
		endif else $
		if m3new eq 2 or m3new eq 3 then begin
			mparam.status.vr = kd
		endif

		print,'(proc.transm) conversion of velocity field completed'

	endif
	mparam.kindm = kindm2

	print,'(proc.transm) updated kindm,mesh=',mparam.kindm,mparam.mesh
	return
end

