;+
; NAME :	vector1
; PURPOSE :
;	vectorize a magnetogram
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	vector1, mparam, rdata
;		[,/faceon] [,/usefff] [,btlim=btlim] [,threshold=threshold]
;		[,nalpha=nalpha] [,/show] [,alphamap=alpha2d] [,fitmap=map]
;		[,divb=divb] [,db2=db2] [,flip=flip]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	/faceon : if set, assume the region is at the disk center
;	/usefff : if set, use linear force-free field as reference,
;	         otherwise, current-free field is used as reference
;	btlim :  noise level of transverse magnetic fields
;	         data points with Bt>btlim are used to determine
;	         the best fit linear force-free field
;	thresh: data points with abs(BL)<thresh*max(|BL|) are used to
;	        determine the best fit linear force-free field
;	        this parameter is to avoid data affected by Faraday rotation
;	nalpha: number of alpha values to estimate the best fit value of
;	        alpha 
;	alphamap : map of alpha point by point which best fits the observed Bt
;	fitmap : 8-color map of degree of fitting of linear force-free field
;		     to the data (fitmap.pro)
;	/show : if set, fitmap is displayed
;	divb : direction of Bt is reversed 
;		where divb (divbp1.pro) is negative (divb=3)
;		and cos(Bt,Btcal) >0.5 (divb=2)
;		and Bt > Btlim (divb=1)
;	db2 : direction of Bt is reversed 
;		where db2dz (db2dz.pro) is negative (db2=3)
;		and cos(Bt,Btcal) >0.5 (db2=2)
;		and Bt > Btlim (db2=1)
; OUTPUTS :
;	vectorized BX and BY will overwrite pseudo vector BX and BY
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	mparam will be modified
; RESTRICTIONS :	none
; PROCEDURE :
;	vectorization by referring to the current-free field
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 1994/10: bug fix (nx&ny are made into long integer)
;
;------------------------------------------------------------------------
;-
pro vector1, mparam, rdata, faceon=faceon, $
	usefff=usefff, btlim=btlim, threshold=threshold, nalpha=nalpha, $
	show=show, alphamap=alpha2d, fitmap=map, divb=divb, db2=db2, $
	flip=flip

	if n_elements(btlim) eq 0 then btlim =150.0
	if not keyword_set(threshold) then threshold=1.0
	if not keyword_set(nalpha) then nalpha=5

	if mparam.kindm lt 0 then pol2b, mparam, rdata

	kindm = mparam.kindm
	kindm = (kindm/10) *10

	nx = long(mparam.nx)	; bug fix 94/10/14
	ny = long(mparam.ny)	; previously these were short integers
				; and sometimes nx*ny*ky > 32767
	dx = mparam.dx
	dy = mparam.dy

	kx = kloc( mparam, /x)
	ky = kloc( mparam, /y)
	kz = kloc( mparam, /z)

	if kx eq -1 or ky eq -1 or kz eq -1 then begin
		print,'(proc.vector) error:kx,ky,kz=',kx,ky,kz
		return
	endif

	thresh=threshold*max(abs(rdata(*,*,kz)))
	bt2 = rdata(*,*,kx)^2 + rdata(*,*,ky)^2

	if keyword_set(usefff) then begin
		lff1, mparam, rdata, faceon=faceon, /tight, $
			maxalpha=maxalpha, /getmaxalpha

		loc = where(bt2 gt btlim^2 and abs(rdata(*,*,kz)) le thresh, count0)
		if count0 eq 0 then begin
			print,'(proc.vector) no datapoints with bt>btlim, use cff1'
			cff1, mparam, rdata, bxp, byp, faceon=faceon, /nosave
			goto, escape
		endif

		alpha2d=0*bt2
		alpha2d(loc)=-maxalpha		; initialize to -maxalpha

		cosb2 = fltarr(2*nalpha+1)
		na2 = 2*nalpha
		alphas = maxalpha*(indgen(na2+1)-nalpha)/float(nalpha+1)

		for i=0,na2 do begin
			lff1, mparam,rdata,bxp,byp,faceon=faceon,/tight,/nosave, $
				alpha=alphas(i)
			cosb1 = (rdata(*,*,kx)*bxp + rdata(*,*,ky)*byp)/ $
				(sqrt(bt2*(bxp^2+byp^2)) + 0.001)
			if i eq 0 then begin
				cosbm=cosb1
				cosb0=cosb1
				count=0
			endif else begin
				cosb=cosb0
				inc = where(bt2 gt btlim^2 and $
					abs(rdata(*,*,kz)) le thresh and $
					cosb1^2 gt cosb0^2, count)
				if count gt 0 then begin
					cosb(inc) = cosb1(inc)
					alpha2d(inc)=alphas(i)
				endif
				cosb0 = cosb
			endelse
			cosb2(i) = total(cosb1(loc)^2)/n_elements(loc)

			print,i,alphas(i),cosb2(i),count

			if i eq na2 then cosbp=cosb
		endfor

		findmin, /max, cosb2,alphas, loc=alpham
;			alpham makes cosb2 maximum

		cosb2min = min(cosb2)
		alphaa = total(alphas*(cosb2-cosb2min))/total(cosb2-cosb2min)
;			alphaa is the center of gravity of cosb2-curve

		alpha2 = total(alpha2d(loc))/count0
;			alpha2 is the mean of alpha2d

		alpha = alphaa
;			use alphaa

		lff1, mparam,rdata,bxp,byp,faceon=faceon,/tight,/nosave, $
			alpha=alpha

		cosb0 = (rdata(*,*,kx)*bxp + rdata(*,*,ky)*byp)/ $
			(sqrt(bt2*(bxp^2+byp^2)) + 0.001)
		cosb1 = 0*cosb0
		cosb1(loc) = cosb0(loc)

;	cosb0 : cos(bt,btp) for best alpha
;	cosb1 : same as cosb0 for datapoints bt > btlim
;	cosb  : pointwise maximum of cos(bt,btp) over a range of alpha values
;	cosbm : cos(bt,btp) for -maxalpha
;	cosbp : cos(bt,btp) for +maxalpha

		if keyword_set(show) then begin
			plot,alphas,cosb2,/ynozero,title=mparam.dateo
			oplot,[alphaa,alphaa],[0,1], linestyle=2
			oplot,[alpha,alpha],[0,1]
			yn=''
			read,yn

			print,'Btobs < btlim   : black, ',n_elements(bt2)-count0

			map = fitmap( mparam,rdata, cosbp,cosbm,cosb1, bxp,byp,bt2, $
				alpha2d, alphas,btlim)

			print,'cos^2 max,min=',max(cosb2),cosb2min
		endif

		print,'(proc.vector) from cos^2 distr.:alpha, alpha/alphamax=', $
				alphaa, alphaa/maxalpha
		print,'(proc.vector) from alpha2d: alpha, alpha/alphamax=', $
				alpha2, alpha2/maxalpha
	endif else begin
		alpha=0.0
		cff1, mparam, rdata, bxp, byp, faceon=faceon, /nosave
		cosb0 = (rdata(*,*,kx)*bxp + rdata(*,*,ky)*byp)/ $
			(sqrt(bt2*(bxp^2+byp^2)) + 0.001)
	endelse

escape:


	flip = where( $
	rdata(*,*,kx)*bxp + rdata(*,*,ky)*byp lt 0.0, count )

	if count gt 0 then begin
		rdata( flip+nx*ny*kx ) = -rdata( flip+nx*ny*kx )
		rdata( flip+nx*ny*ky ) = -rdata( flip+nx*ny*ky )
	end

	if keyword_set(divb) then begin
		divbp1, mparam,rdata, divbp, divbpa, method=2, /nosave
		if divb eq 1 then begin
			flip = where(divbpa gt divbp and bt2 gt btlim^2 and $ 
			 	abs(cosb0) lt 0.5, count1)
			print,'flip by divbp1, bt>btlim, cos>0.5',count1
		endif else $
		if div eq 2 then begin
			flip = where(divbpa gt divbp and bt2 gt btlim^2, $
				count1)
			print,'flip by divbp1, bt>btlim',count1
		endif else begin
			flip = where(divbpa gt divbp, count1)
			print,'flip by divbp1, all points',count1
		endelse
		if count1 gt 0 then begin
			rdata( flip+nx*ny*kx ) = -rdata( flip+nx*ny*kx )
			rdata( flip+nx*ny*ky ) = -rdata( flip+nx*ny*ky )
		endif
	endif

	if keyword_set(db2) then begin
		db2dz, mparam,rdata, grad, grada, method=2, /nosave
		if db2 eq 1 then begin
			flip = where(grada gt grad and bt2 gt btlim^2 and $
				abs(cosb0) lt 0.5, count2)
			print,'flip by db2dz, bt>btlim, cos>0.5',count2
		endif else $
		if db2 eq 2 then begin
			flip = where(grada gt grad and bt2 gt btlim^2, count2)
			print,'flip by db2dz, bt>btlim',count2
		endif else begin
			flip = where(grada gt grad, count2)
			print,'flip by db2dz, all points',count2
		endelse
		if count2 gt 0 then begin
			rdata( flip+nx*ny*kx ) = -rdata( flip+nx*ny*kx )
			rdata( flip+nx*ny*ky ) = -rdata( flip+nx*ny*ky )
		endif
	endif
	return
end

