;+
; NAME :	virial1
; PURPOSE :
;  check the validity of virial theorem
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	vector1, mparam, rdata
;		[,btlim=btlim] [,/list]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	btlim :  noise level of transverse magnetic fields
;	         Bt<btlim are replaced with potential field vectors
;	/list : if set, detail of virial relations are reported
; OUTPUTS : none
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;	virial theorem in planar geometry
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/10
;
;------------------------------------------------------------------------
;-

pro virial1, mparam,rdata, btlim=btlim, verbose=list

	common blkd, coord_d
	common blkt, coord_t

	if not keyword_set(btlim) then btlim=100.0

	kindm = mparam.kindm
	mesh = mparam.mesh
	nx = mparam.nx
	ny = mparam.ny
	xs = mparam.xs
	ys = mparam.ys
	dx = mparam.dx
	dy = mparam.dy
	rsp = mparam.rsp

	ms = umake(kindm, m5, m4, m3, m2, m1)
	kx=kloc(mparam,/x)
	ky=kloc(mparam,/y)
	kz=kloc(mparam,/z)
	kxp=kloc(mparam,/px)
	kyp=kloc(mparam,/py)

	if mesh gt 1 then begin
	  print, '(proc. virial1) error: mesh=',mesh
	  return
	endif

	if mesh le 0 and m5 ne 0 then begin
	  print, '(proc.virial1) error: kindm=',kindm
	  return
	endif

	if mesh eq 1 and m5 ne 1 then begin
	  print, '(proc.virial1) error: kindm=',kindm
	  return
	endif

	if kz eq 0 or $
	  kx eq 0 or ky eq 0 and (kxp eq 0 or kyp eq 0) then begin
	  print, '(proc.virial1) error: kx,ky,kz=',kx,ky,kz
	  return
	endif

	setdu, mparam
	settu, mparam
;  settu sets at-matrix

	if mesh eq -1 then amat=coord_t.atmat else $
		amat=[[1.0,0.0,0.0], [0.0,1.0,0.0], [0.0,0.0,1.0]]

;  amat=unit matrix if mesh=1, 0)
;  amat=atmat if mesh=-1

;  aj = Jacobian
	aj=amat(0,0)*amat(1,1)-amat(0,1)*amat(1,0)
	ulen = 1.0e9	; 10^4 km
	rsun = 69.6	; solar radius
	ulen1 = ulen*rsun/rsp
	area=dx*dy*ulen1^2/aj
	vol =dx*dy*ulen1^3/(4.0*3.141592 *aj)
	nx2=2*nx
	ny2=2*ny
	vn = amat(*,2)

	buf = rdata(*,*,kz)
	pos = where(buf gt 0.0, count)
	if count gt 0 then bzsump=total(buf(pos)) else bzsump=0.0
	neg = where(buf lt 0.0, count)
	if count gt 0 then bzsumm=total(buf(neg)) else bzsumm=0.0
	bl00=(bzsump-bzsumm)/(float(nx)*float(ny))

;  bl00 is the dc component of bz
	bn00=bl00/amat(2,2)
	bnn0=bn00^2*float(nx2)*float(ny2)
	bx00=amat(0,2)*bn00
	by00=amat(1,2)*bn00
;  bx00 and by00 have been added to bxp and byp in subr.vector
;  (via subr.cff1)

	if mparam.alpha ne 0.0 then begin
	  bx00=0.0
	  by00=0.0
	  bl00=0.0
	endif
;  in the case of force-free field, DC component is not allowed

	x = fltarr(nx,ny)
	y = fltarr(nx,ny)
	z = fltarr(nx,ny)
	xs1=-dx*0.5*(nx-1)
	ys1=-dy*0.5*(ny-1)

	if mesh eq -1 then begin
	  for j=0,ny-1 do x(*,j)=xs + dx*indgen(nx)
	  for i=0,nx-1 do y(i,*)=ys + dy*indgen(ny)
	  projt, x,y,z, xt,yt
	  x = x-coord_t.xto
	  y = y-coord_t.yto
	  z = z-coord_t.zto
	endif else $
	if mesh eq 0 then begin
	  for j=0,ny-1 do x(*,j)=xs1 + dx*indgen(nx)
	  for i=0,nx-1 do y(i,*)=ys1 + dy*indgen(ny)
	  xt=x
	  yt=y
	endif else $
	if mesh eq 1 then begin
	  for j=0,ny-1 do x(*,j)=xs + dx*indgen(nx)
	  for i=0,nx-1 do y(i,*)=ys + dy*indgen(ny)
	  xt=x
	  yt=y
	endif

;  potential field or linear force-free field

	resp=fltarr(3)
	engyp = 0.0
	engypp= 0.0
	if kxp lt 0 or kyp lt 0 then goto, nopot

;  bb = B-field in C-coordinates
;  b  = B-field possibly in T-coordinates

	bb1 = rdata(*,*,kxp)
	bb2 = rdata(*,*,kyp)

;	bb(1)=rdata(i,j,kxp)
;	bb(2)=rdata(i,j,kyp)
;	bb(3)=rdata(i,j,kz)
;  b=ainv*bb=bb* transpose of amat 
;	b(1)=bb(1)*amat(1,1)+bb(2)*amat(2,1)+bb(3)*amat(3,1)
;	b(2)=bb(1)*amat(1,2)+bb(2)*amat(2,2)+bb(3)*amat(3,2)
;	b(3)=bb(1)*amat(1,3)+bb(2)*amat(2,3)+bb(3)*amat(3,3)

	b1 = bb1*amat(0,0) $
	   + bb2*amat(1,0) $
	   + rdata(*,*,kz)*amat(2,0)
	b2 = bb1*amat(0,1) $
	   + bb2*amat(1,1) $
	   + rdata(*,*,kz)*amat(2,1)
	b3 = bb1*amat(0,2) $
	   + bb2*amat(1,2) $
	   + rdata(*,*,kz)*amat(2,2)

;  b=ainv*bb=bb* transpose of amat 
;  xxt=ainv*xx

	engypp=total( b3*(x*(bb1-bx00) $
	                 +y*(bb2-by00) $
	                 +z*(rdata(*,*,kz)-bl00)) )
	engyp =total( b3*(xt*b1+yt*b2) )
	bd2=(bb1-by00)^2+(bb2-by00)^2+(rdata(*,*,kz)-bl00)^2
	trpp=0.5*total(bd2)
	resp(0)=0.5*total( bd2*vn(0)-b3*(rdata(*,*,kx)-bx00) )
	resp(1)=0.5*total( bd2*vn(1)-b3*(rdata(*,*,ky)-bx00) )
	resp(2)=0.5*total( bd2*vn(2)-b3*(rdata(*,*,kz)-bx00) )

	engyp =engyp *vol
	engypp=engypp*vol
	resp=resp/trpp
	reshxp=resp(0)*amat(0,0)+resp(1)*amat(1,0)+resp(2)*amat(2,0)
	reshyp=resp(0)*amat(0,1)+resp(1)*amat(1,1)+resp(2)*amat(2,1)
	resnp =resp(0)*amat(0,2)+resp(1)*amat(1,2)+resp(2)*amat(2,2)
	reshp =sqrt(reshxp^2+reshyp^2)

	if mparam.alpha eq 0.0 then $
	    print, '(proc.virial1) potential field, flat boundary:' $
	  else $
	    print, '(proc.virial1) linear force-free field, flat boundary:'

	print, '  resx,y,z          =',resp
	print, '  energy (blp,btp)  =',engypp,' erg'
	if keyword_set(list) then begin
	  print, '  resn              =',resnp 
	  print, '  resh              =',reshp
	  print, '  energy (bnp,bhp)  =',engyp ,' erg'
	endif

nopot:

;  calculation of energy by virial theorem

	res = fltarr(3)

	bb1 = rdata(*,*,kx)
	bb2 = rdata(*,*,ky)
	bt = sqrt(bb1^2 + bb2^2)

	if kxp ge 0 and kyp ge 0 then begin
	  loc=where(bt lt btlim, count)
	  if count gt 0 then begin
	    bbp1=rdata(*,*,kxp)
	    bbp2=rdata(*,*,kyp)
	    bb1(loc)=bbp1(loc)
	    bb2(loc)=bbp2(loc)
	  endif
	endif
;  bt<btlim are replaced with btp, if available, for virial calculation
	bb3=rdata(*,*,kz)
	b1 = bb1*amat(0,0) $
	   + bb2*amat(1,0) $
	   + rdata(*,*,kz)*amat(2,0)
	b2 = bb1*amat(0,1) $
	   + bb2*amat(1,1) $
	   + rdata(*,*,kz)*amat(2,1)
	b3 = bb1*amat(0,2) $
	   + bb2*amat(1,2) $
	   + rdata(*,*,kz)*amat(2,2)

	engybb=total( b3*(x*(bb1-bx00) $
	                 +y*(bb2-by00) $
	                 +z*(rdata(*,*,kz)-bl00)) )
	engyb =total( b3*(xt*b1+yt*b2) )
	bd2=(bb1-bx00)^2+(bb2-by00)^2+(rdata(*,*,kz)-bl00)^2
	trbb=0.5*total(bb2)
	res(0)=0.5*total( bd2*vn(0)-b3*(bb1-bx00) )
	res(1)=0.5*total( bd2*vn(1)-b3*(bb2-bx00) )
	res(2)=0.5*total( bd2*vn(2)-b3*(rdata(*,*,kz)-bx00) )

      engyb =engyb *vol
      engybb=engybb*vol
      res=res/trbb
      reshx=res(0)*amat(0,0)+res(1)*amat(1,0)+res(2)*amat(2,0)
      reshy=res(0)*amat(0,1)+res(1)*amat(1,1)+res(2)*amat(2,1)
      resn =res(0)*amat(0,2)+res(1)*amat(1,2)+res(2)*amat(2,2)
      resh =sqrt(reshx^2+reshy^2)

	if engyp ne 0.0 and engypp ne 0.0 then begin
	  engyr =(engyb -engyp )/engyp
	  engyrr=(engybb-engypp)/engypp
	endif

	print, '(proc.virial1) based on observed data, flat boundary:'
	print,  '  average bz         =',bn00,' gauss'
	print, '  cut-off level of bt=',btlim,' gauss'
      print, '  resx,y,z           =',res
      print, '  energy (bl,bt)     =',engybb,' erg'
	if engypp ne 0.0 then $
	print, '  (w-w0)/w0          =',engyrr
	if keyword_set(list) then begin
	  print, '  resn               =',resn
	  print, '  resh               =',resh
	  print, '  energy (bn,bh)     =',engyb ,' erg'
	  if engyp ne 0.0 then $
	  print, '  (w-w0)/w0          =',engyr
	endif

	return
	end
