;+
; NAME :	wtfits
; PURPOSE :
;	write magnetogram file in FITS format
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	wtfits, mparam,rdata, [,filename=fitsfile] [,/list=list]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	filename : FITS magnetogram file name
;	/list : if set, parameter values in the header are printed
; OUTPUTS :
;	results are written to a file
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
;	if filename is not specified, it should be typed in from keyboard
; RESTRICTIONS :
;	All the data in rdata are written to a file.
;	Selective output which is possible in the fortran version
;	is not implemented in the IDL version.
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/09
;	unit of length in output has been changed from ulen (approx.10=4 km)
;	to arcsec 2001/04/12
;-
;------------------------------------------------------------------------
;

function padhdr,str,len
	return, strmid(strtrim(str)+string(' ',format='(a80)'),0,len)
end

function puthdrs,tag,str,comment,noquote=noquote
	if keyword_set(noquote) then begin
		hdr=padhdr(tag,8) +'= ' $
		+ padhdr(str,20) $
		+ '          / ' + padhdr(comment,38)
	endif else begin
		hdr=padhdr(tag,8) +'= ' $
		+ "'"+padhdr(str,18) +"'" $
		+ '          / ' + padhdr(comment,38)
	endelse
	return,hdr
end

function puthdri,tag,num,comment
	hdr=padhdr(tag,8) +'= ' $
	+ string(num,format='(i20)') $
	+ '          / ' + padhdr(comment,38)
	return,hdr
end

function puthdre,tag,num,comment
	hdr=padhdr(tag,8) +'= ' $
	+ string(num,format='(e20.5)') $
	+ '          / ' + padhdr(comment,38)
	return,hdr
end

function puthdrf,tag,num,comment
	hdr=padhdr(tag,8) +'= ' $
	+ string(num,format='(f20.2)') $
	+ '          / ' + padhdr(comment,38)
	return,hdr
end

function puthdrc,comment
	hdr=padhdr('COMMENT',10) $
	+ padhdr(comment,70)
	return,hdr
end

pro wtfits, mparam,rdata,filename=filename,list=list

	if not keyword_set(filename) then begin
		filename=''
		read,'enter filename (FITS output) ',filename
	endif

	factr0 = 0.0
	nonmag=0
	mag=0
	nframe = mparam.nframe
        specs = defspecs(nspec, units=units)
	speca = strarr(nframe)
	unita = strarr(nframe)
	kmem,mparam,rdata,karr=karr,larr=larr,specf=specf

	for iframe=0, nframe -1 do begin
		l = larr(iframe)
		factor=mparam.factors.(l)
		speca(iframe)=specs(l)
		unita(iframe)=units(l)
		if factr0 eq 0.0 then factr0=factor
		if strmid(speca(iframe),0,1) eq 'B' then begin
			factr0=factor
			mag=1
		endif else nonmag=1
	endfor

	nbit=8*abs(mparam.nbytem)
	naxis=3
	zlevel=0.0
	one=1.0
	arcsec=0.07
	ftag=mktag()
	rd=3.141592/180.0
	pdeg=mparam.prad/rd
	b0deg = mparam.b0rad/rd
	cl0deg = mparam.cl0rad/rd
	ydeg = mparam.ypos/rd

	hdr1=strarr(36)
	hdr2=strarr(36)

	hdr1( 0)=puthdrs('SIMPLE', '                   T', $
			'STANDARD FITS',/noquote)
	hdr1( 1)=puthdri('BITPIX',  nbit, $
			'NUMBER OF BITS PER PIXEL')
	hdr1( 2)=puthdri('NAXIS',   naxis, $
			'NUMBER OF AXIS')
	hdr1( 3)=puthdri('NAXIS1',  mparam.nx, $
			'NUMBER OF PIXELS/ROW(EW)')
	hdr1( 4)=puthdri('NAXIS2',  mparam.ny, $
			'NUMBER OF ROWS(NS)')
	hdr1( 5)=puthdri('NAXIS3',  mparam.nframe, $
			'NUMBER OF FRAMES')
	hdr1( 6)=puthdre('BSCALE',  factr0, $
			'REAL = TAPE*BSCALE+BZERO')
	hdr1( 7)=puthdre('BZERO',   zlevel,'')
	if mag eq 1 and nonmag eq 0 then $
		hdr1( 8)=puthdrs('BUNIT',   'GAUSS','')
	if mag eq 1 and nonmag eq 1 then $
		hdr1( 8)=puthdrs('BUNIT',   'GAUSS','FOR MAGNETIC FIELDS')
	if mag eq 0 then $
		hdr1( 8)=puthdrs('BUNIT',   '','')
	hdr1( 9)=puthdrs('OBJECT',  mparam.mfile, $
			'OBJECT OR FILE-ID')
	hdr1(10)=puthdre('CRVAL1',  mparam.xs/arcsec, $
			'REF POINT VALUE (ARCSEC)')
	hdr1(11)=puthdre('CRPIX1',  one, $
			'REF POINT PIXEL LOCATION')
	hdr1(12)=puthdrs('CTYPE1',  'LL', $
			'COORD TYPE:X(EW)')
	hdr1(13)=puthdre('CDELT1',  mparam.dx/arcsec, $
			'COORD VALUE INCREMENT (ARCSEC)')
	hdr1(14)=puthdre('CRVAL2',  mparam.ys/arcsec, $
			'REF POINT VALUE (ARCSEC)')
	hdr1(15)=puthdre('CRPIX2',  one, $
			'REF POINT PIXEL LOCATION')
	hdr1(16)=puthdrs('CTYPE2',  'MM', $
			'COORD TYPE:Y(NS)')
	hdr1(17)=puthdre('CDELT2',  mparam.dy/arcsec, $
			'COORD VALUE INCREMENT (ARCSEC)')
	hdr1(18)=puthdre('CRVAL3',  one, $
			'REF POINT VALUE')
	hdr1(19)=puthdre('CRPIX3',  one, $
			'REF POINT PIXEL LOCATION')
	hdr1(20)=puthdrs('CTYPE3',  specf, $
			'COORD TYPE:IMAGE FRAMES')
	hdr1(21)=puthdre('CDELT3',  one, $
			'COORD VALUE INCREMENT')
	hdr1(22)=puthdrf('P',       pdeg, $
			'SOLAR P-ANGLE (DEG)')
	hdr1(23)=puthdrf('B0',      b0deg, $
			'SOLAR B0-ANGLE (DEG)')
	hdr1(24)=puthdrf('CL0',     cl0deg, $
			'CARRINGTON LONGITUDE OF DISK CENTER')
	hdr1(25)=puthdrf('RSUN',    mparam.rsp/arcsec, $
			'SOLAR RADIUS (ARCSEC)')
	hdr1(26)=puthdrs('INSTRUME',mparam.inst, $
			'INSTRUMENT/TELESCOPE')
	hdr1(27)=puthdrs('TELESCOP',mparam.inst, $
			'INSTRUMENT/TELESCOPE')
	hdr1(28)=puthdrf('WAVELEN', mparam.wavel, $
			'WAVELENGTH, ANGSTROM')
	hdr1(29)=puthdrs('DATEOBS', mparam.dateo,'')
	hdr1(30)=puthdrs('TIMEOBS', mparam.stime+'-'+mparam.etime,'')
	hdr1(31)=puthdrs('SYSTIME', strtrim(mparam.tsys,2), $
			'TIME ZONE FOR DATEOBS,TIMEOBS')
	hdr1(32)=puthdrs('DATE',ftag,'')
	hdr1(33)=puthdrs('ORIGIN','MAGPACK2 FITS','')
;  non-standard parametes and comments
	hdr1(34)=puthdrc(mparam.title)
	hdr1(35)=puthdrc('NON-STANDARD PARAMETERS')
	hdr2( 0)=puthdri('KINDM',   mparam.kindm, $
			'DATA SPECIFIER')
	hdr2( 1)=puthdri('MESH',    mparam.mesh, $
			'COORDINATE SYSTEM SPECIFIER')
	hdr2( 2)=puthdrs('MFILE',   mparam.mfile, $
			'FILE ID')
	hdr2( 3)=puthdrf('YDEG',    ydeg, $
			'POSITION ANGLE OF Y-DIRECTION')
	hdr2( 4)=puthdre('XTOD',    mparam.xxtod(0)/arcsec, $
			'X-ORIGIN OF T-COORDINATES (ARCSEC)')
	hdr2( 5)=puthdre('YTOD',    mparam.xxtod(1)/arcsec, $
			'Y-ORIGIN OF T-COORDINATES (ARCSEC)')

	loc=6
	for iframe=0,nframe-1 do begin
		factor=mparam.factors.(larr(iframe))
		hdr2(loc)=puthdre('FACTOR'+string(iframe,format='(i1)'), $
			factor,'')
		loc=loc+1
	endfor

	if mparam.inst eq 'MTK ft1 ' then begin
		dwl = mparam.mpint(0)
		blfact = mparam.mpreal(0)
		if dwl eq 0 then dwl =-80
		if blfact eq 0.0 then blfact=1.0
		hdr2(loc) =puthdri('DWL', dwl, $
			'PASSBAND POSISION (MILLI-ANGSTROM)')
		loc=loc+1
		hdr2(loc)=puthdre('BLFACT', blfact, $
			'CALIB.FACTOR TO BE MULTIPLIED ON BL')
		loc=loc+1
	endif
	hdr2(loc)=padhdr('END',80)
	for i=loc+1,35 do hdr2(i)=string(' ',format='(a80)')

	if keyword_set(list) then begin
	  print,'(proc.wtfits) FITS HEADER'
	  for i=0,35 do print,strtrim(hdr1(i))
	  for i=0,loc do print,strtrim(hdr2(i))
	endif

	openw,lun,filename,/get_lun
	writeu,lun,hdr1
	writeu,lun,hdr2

	bytecount=2*2880L

	if mparam.nbytem eq 2 then a = intarr(mparam.nx,mparam.ny)
	if mparam.nbytem eq 4 then a = lonarr(mparam.nx,mparam.ny)
	for iframe=0,nframe-1 do begin
		factor=mparam.factors.(larr(iframe))
		a(*,*) = rdata(*,*,karr(iframe))/factor
		if myint() lt 0 then byteorder,a
		writeu,lun,a
		bytecount=bytecount+ $
		long(mparam.nx)*long(mparam.ny)*long(mparam.nbytem)
	endfor
;  pad incomplete record with zero
	nrec = bytecount mod 36*80
	if nrec gt 0 then writeu,lun,replicate(' ',nrec)

	close,lun
	free_lun,lun
	print,'(proc.wtfits) ',bytecount,' bytes written'

	return
end

