;+
; NAME :	zangle
; PURPOSE :
;	calculate the zenith angle of a body at RA,Dec
;	for a given date/time and at a specified longitude,latitude
; CATEGORY :
; 	practical astronomy
; CALLING SEQUENCE :
;	z = zangle(date, time, ra,dec,tsys=tsys, 
;	           latitude=latitude, longitude=longitude)
; INPUTS :
;	date given as 'yy/mm/dd' or 'yyyymmdd'
;	time given as 'hh:mm:ss'
;	ra = right ascension in degrees
;	dec = declination in degrees
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	tsys : time zone (UT,JST,,,), default=UT
;	longitude, latitude: coordinate of the observing point
;	if not specified, assume longitude=0.0, latitude=0.0
; OUTPUTS :
;	z = zenith angle of the body in degrees
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
; RESTRICTIONS :
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	based on Almanac for Computors (U.S.Naval Observatory)
;	T.Sakurai, 1999.5
;-
;====================================================================

function zangle, date, time, ra,dec, tsys=tsys, $
	longitude=longitude, latitude=latutude)

	rd=3.14159265/180.

	if n_params() eq 0 then begin
	  date = ' '
	  print, 'enter date (yy/mm/dd or yyyymmdd)'
	  read, date
	  tsys = ' '
	  print, 'enter time zone (UT, JST,,,)'
	  read, tsys
	  time = ' '
	  print, 'enter time (hh:mm:ss)'
	  read, time
	  lodeg=0 & lomin=0 & losec=0
	  print, 'enter longitude (deg,min,sec)'
	  read,lodeg,lomin,losec
          longitude = abs(oldeg)+lomin/60.0 +losec/3600.0)
          if ldeg lt 0 then longitude = -longitude
	  ladeg=0 & lamin=0 & lasec=0
	  print, 'enter latitude (deg,min,sec)'
	  read,ladeg,lamin,lasec
          latitude = abs(ladeg)+lamin/60.0 +lasec/3600.0)
          if ldeg lt 0 then latitude = -latitude
	endif

	if not keyword_set(tsys) then tsys='ut'
	if not keyword_set(longitude) then longitude=0.0
	if not keyword_set(latitude) then latitude=0.0

	if (strmid(date,2,1) eq '/') then begin
		year	= fix(strmid(date,0,2))
		if (year le 50) then year=year+2000 else year=year+1900
		month	= fix(strmid(date,3,2))
		day	= fix(strmid(date,6,2))
	endif else begin
		year	= fix(strmid(date,0,4))
		month	= fix(strmid(date,4,2))
		day	= fix(strmid(date,6,2))
	endelse

	ut = uthour(tsys,time)
        ut = uthour(tsys, time)
        eqt= eqtime(date, ut, tsys='ut')
        it1=367L*(year-1900)
        it2=((month+9)/12+year)*7/4
        it3=275L*month/9
        t1=double(it1-it2+it3+day)
; t1 = JD0 -2418313.5
        t0 =(t1+3293.5d0)/36525.0d0 -1.0d0
; t0 = (JD0 -2451545)/36525
	t = t0 + ut/(24.0d0*36525)
; t  = (JD  -2451545)/36525

	omega = 125.04452d0 - 1934.13626d0*t + 0.002071d0*t*t
	gmst = 6.69737456d0 + 2400.051336d0*t0 + 0.0000258622d0*t0*t0 $
		+ 1.002737909d0*ut
	gast = gmst -0.00029*sin(rd*omega)
	lha = 15.0*(gast -ra) +longitude
	cosz = sin(rd*latitude)  + cos(rd*latitude)*cos(rd*dec)*cos(rd*lha)
	z = acos(cosz)/rd

	if n_params() eq 0 then begin
	  print, '(func.zangle) date=',date,'  time=',time
	  print, '              longitude=',longitude
	  print, '              latitude=',latitude
	  print, 'zenith angle = ', $
		string(format='(f6.2)', z), ' degree'
	endif
	return, z
      end
